@@ -476,8 +476,8 @@ gw = {
       var boundingRect = elt.getBoundingClientRect();
       var baseRect = offsetBase.getBoundingClientRect();
       found = true;
-      left = boundingRect.left - baseRect.left;
-      top = boundingRect.top - baseRect.top;
+      left = boundingRect.left - baseRect.left; //- document.documentElement.scrollLeft;
+      top = boundingRect.top - baseRect.top; //- document.documentElement.scrollTop;
       width = boundingRect.right - boundingRect.left;
       height = boundingRect.bottom - boundingRect.top;
     }
@@ -686,6 +686,8 @@ gw = {
       }
       
       gw.window.refresh();
+      
+      $(id).scrollIntoView({behavior:"auto", block:"nearest"});      
     },
 
     close: function(id)
@@ -1035,7 +1037,7 @@ gw = {
       {
         /*elt.style.left = (Math.max(0, c.x + c.cx - e.clientX)) + 'px';
         elt.style.top = (Math.max(0, c.y + c.cy - e.clientY)) + 'px';*/
-        elt.style.transform = 'translate(' + (Math.max(0, c.x + c.cx - e.clientX)) + 'px,' + (Math.max(0, c.y + c.cy - e.clientY)) + 'px)';
+        elt.style.transform = 'translate(' + (Math.max(0, c.x + c.cx - e.clientX)) + 'px,' + (Math.min(window.innerHeight - elt.firstElementChild.offsetHeight, Math.max(0, c.y + c.cy - e.clientY))) + 'px)';
         return;
       }
     
