@@ -727,7 +727,10 @@ static void set_character_set(DB_DATABASE *db)
 	if (search_result(res, "character_set_client", &row))
 		return;
 	
-	db->charset = GB.NewZeroString(row[1]);
+	if (strncasecmp(row[1], "utf8", 4) == 0)
+		db->charset = GB.NewString("utf8", 4);
+	else
+		db->charset = GB.NewZeroString(row[1]);
 	//fprintf(stderr, "charset is '%s'\n", db->charset);
 	mysql_free_result(res);
 }
@@ -1798,7 +1801,7 @@ static int table_create(DB_DATABASE *db, const char *table, DB_FIELD *fields, ch
 
 			if (fp->def.type != GB_T_NULL)
 			{
-				DB.Query.Add(" NOT NULL DEFAULT");
+				DB.Query.Add(" NOT NULL DEFAULT ");
 				DB.FormatVariant(&_driver, &fp->def, DB.Query.AddLength);
 			}
 			else if (DB.StringArray.Find(primary, fp->name) >= 0)
