@@ -54,15 +54,6 @@
 #include <stdarg.h>
 #include <string.h>
 
-#ifdef PACKAGE_NAME
-	#undef PACKAGE_NAME
-	#undef PACKAGE_BUGREPORT
-	#undef PACKAGE_STRING
-	#undef PACKAGE_TARNAME
-	#undef PACKAGE_VERSION
-	#undef PACKAGE_URL
-#endif
-
 #ifdef Max
 	#undef Max
 #endif
@@ -92,6 +83,16 @@ static DB_DRIVER _driver;
 static int _last_error;
 /*static int _print_query = FALSE;*/
 
+// Get the SQL expression returning the default value of a field
+
+static const char *get_default_value(const char *old_way)
+{
+	if (DB.GetCurrentDatabase()->version >= 90600)
+		return "pg_get_expr(adbin, adrelid) AS adsrc";
+	else
+		return old_way;
+}
+
 /* Internal function to check the result of a query */
 
 static int check_result(PGresult *res, const char *err)
@@ -544,11 +545,11 @@ static void conv_data(const char *data, int len, GB_VARIANT_VALUE *val, Oid type
 
 /* Internal function to substitute the table name into a query */
 
-static char *query_param[3];
+static char *query_param[4];
 
 static void query_get_param(int index, char **str, int *len, char quote)
 {
-	if (index > 3)
+	if (index > 4)
 		return;
 
 	index--;
@@ -576,8 +577,8 @@ static int do_query(DB_DATABASE *db, const char *error, PGresult **pres, const c
 	if (nsubst)
 	{
 		va_start(args, nsubst);
-		if (nsubst > 3)
-			nsubst = 3;
+		if (nsubst > 4)
+			nsubst = 4;
 		for (i = 0; i < nsubst; i++)
 			query_param[i] = va_arg(args, char *);
 
@@ -606,19 +607,28 @@ static int do_query(DB_DATABASE *db, const char *error, PGresult **pres, const c
 
 static int db_version(DB_DATABASE *db)
 {
-	//Check db version
-	const char *vquery =
-		"select substring(version(),12,5)";
-	int dbversion =0;
+	unsigned int verMain = 0, verMajor = 0, verMinor = 0;
+	const char *query = "select version()";
+	const char *version;
+	int dbversion = 0;
 	PGresult *res;
 
-	if (!do_query(db, NULL, &res, vquery, 0))
+	if (!do_query(db, NULL, &res, query, 0))
 	{
-		unsigned int verMain, verMajor, verMinor;
-		sscanf(PQgetvalue(res, 0, 0), "%2u.%2u.%2u", &verMain, &verMajor, &verMinor);
-		dbversion = ((verMain * 10000) + (verMajor * 100) + verMinor);
+		version = PQgetvalue(res, 0, 0);
+		
+		while (*version && !isdigit(*version))
+			version++;
+		
+		if (*version)
+		{
+			sscanf(version, "%2u.%2u.%2u", &verMain, &verMajor, &verMinor);
+			dbversion = ((verMain * 10000) + (verMajor * 100) + verMinor);
+		}
+		
 		PQclear(res);
 	}
+	
 	return dbversion;
 }
 
@@ -1049,10 +1059,11 @@ static void query_init(DB_RESULT result, DB_INFO *info, int *count)
 
 	<result> is the handle of the query result.
 	<info> points to the info structure.
+	<invalid> tells if the associated connection has been closed.
 
 *****************************************************************************/
 
-static void query_release(DB_RESULT result, DB_INFO *info)
+static void query_release(DB_RESULT result, DB_INFO *info, bool invalid)
 {
 	PQclear((PGresult *)result);
 }
@@ -1342,7 +1353,7 @@ static int begin_transaction(DB_DATABASE *db)
 
 /*****************************************************************************
 
-	commi_transaction()
+	commit_transaction()
 
 	Commit a transaction.
 
@@ -1414,7 +1425,7 @@ static int rollback_transaction(DB_DATABASE *db)
 
 	This function must initialize the following info fields:
 	- info->nfield must contain the number of fields in the table.
-	- info->fields is a char*[] pointing at the name of each field.
+	- info->field is an array of DB_FIELD, one element for each field.
 
 	This function returns TRUE if the command has failed, and FALSE if
 	everything was OK.
@@ -1432,10 +1443,10 @@ static int table_init(DB_DATABASE *db, const char *table, DB_INFO *info)
 	{
 		qfield_all=
 				"SELECT col.attname, col.atttypid::int, col.atttypmod, "
-						"col.attnotnull, def.adsrc, col.atthasdef "
+						"col.attnotnull, &1, col.atthasdef "
 				"FROM pg_catalog.pg_class tbl, pg_catalog.pg_attribute col "
 								"LEFT JOIN pg_catalog.pg_attrdef def ON (def.adnum = col.attnum AND def.adrelid = col.attrelid) "
-				"WHERE tbl.relname = '&1' AND "
+				"WHERE tbl.relname = '&2' AND "
 						"col.attrelid = tbl.oid AND "
 						"col.attnum > 0 AND "
 						"not col.attisdropped "
@@ -1443,11 +1454,11 @@ static int table_init(DB_DATABASE *db, const char *table, DB_INFO *info)
 
 		qfield_schema_all =
 			"select pg_attribute.attname, pg_attribute.atttypid::int, pg_attribute.atttypmod, "
-							"pg_attribute.attnotnull, pg_attrdef.adsrc, pg_attribute.atthasdef "
+							"pg_attribute.attnotnull, &1, pg_attribute.atthasdef "
 					"from pg_class, pg_attribute "
 							"LEFT JOIN pg_catalog.pg_attrdef  ON (pg_attrdef.adnum = pg_attribute.attnum AND pg_attrdef.adrelid = pg_attribute.attrelid) "
-					"where pg_class.relname = '&1' "
-							"and (pg_class.relnamespace in (select oid from pg_namespace where nspname = '&2')) "
+					"where pg_class.relname = '&2' "
+							"and (pg_class.relnamespace in (select oid from pg_namespace where nspname = '&3')) "
 							"and pg_attribute.attnum > 0 and not pg_attribute.attisdropped "
 							"and pg_attribute.attrelid = pg_class.oid ";
 	}
@@ -1455,11 +1466,11 @@ static int table_init(DB_DATABASE *db, const char *table, DB_INFO *info)
 	{
 		qfield_all=
 				"SELECT col.attname, col.atttypid::int, col.atttypmod, "
-						"col.attnotnull, def.adsrc, col.atthasdef, pg_collation.collname "
+						"col.attnotnull, &1, col.atthasdef, pg_collation.collname "
 				"FROM pg_catalog.pg_class tbl, pg_catalog.pg_attribute col "
 								"LEFT JOIN pg_catalog.pg_attrdef def ON (def.adnum = col.attnum AND def.adrelid = col.attrelid) "
 								"LEFT JOIN pg_collation ON (pg_collation.oid = col.attcollation) "
-				"WHERE tbl.relname = '&1' AND "
+				"WHERE tbl.relname = '&2' AND "
 						"col.attrelid = tbl.oid AND "
 						"col.attnum > 0 AND "
 						"not col.attisdropped "
@@ -1467,12 +1478,12 @@ static int table_init(DB_DATABASE *db, const char *table, DB_INFO *info)
 
 		qfield_schema_all =
 			"select pg_attribute.attname, pg_attribute.atttypid::int, pg_attribute.atttypmod, "
-							"pg_attribute.attnotnull, pg_attrdef.adsrc, pg_attribute.atthasdef, pg_collation.collname "
+							"pg_attribute.attnotnull, &1, pg_attribute.atthasdef, pg_collation.collname "
 					"from pg_class, pg_attribute "
 							"LEFT JOIN pg_catalog.pg_attrdef  ON (pg_attrdef.adnum = pg_attribute.attnum AND pg_attrdef.adrelid = pg_attribute.attrelid) "
 								"LEFT JOIN pg_collation ON (pg_collation.oid = pg_attribute.attcollation) "
-					"where pg_class.relname = '&1' "
-							"and (pg_class.relnamespace in (select oid from pg_namespace where nspname = '&2')) "
+					"where pg_class.relname = '&2' "
+							"and (pg_class.relnamespace in (select oid from pg_namespace where nspname = '&3')) "
 							"and pg_attribute.attnum > 0 and not pg_attribute.attisdropped "
 							"and pg_attribute.attrelid = pg_class.oid ";
 	}
@@ -1487,12 +1498,12 @@ static int table_init(DB_DATABASE *db, const char *table, DB_INFO *info)
 
 	if (get_table_schema(&table, &schema))
 	{
-		if (do_query(db,"Unable to get table fields: &1", &res, qfield_all, 1, table))
+		if (do_query(db,"Unable to get table fields: &1", &res, qfield_all, 2, get_default_value("def.adsrc"), table))
 			return TRUE;
 	}
 	else
 	{
-		if (do_query(db, "Unable to get table fields: &1", &res, qfield_schema_all, 2, table, schema))
+		if (do_query(db, "Unable to get table fields: &1", &res, qfield_schema_all, 3, get_default_value("pg_attrdef.adsrc"), table, schema))
 			return TRUE;
 	}
 
@@ -2239,23 +2250,23 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 	{
 		query =
 			"select pg_attribute.attname, pg_attribute.atttypid::int, "
-			"pg_attribute.atttypmod, pg_attribute.attnotnull, pg_attrdef.adsrc, pg_attribute.atthasdef "
+			"pg_attribute.atttypmod, pg_attribute.attnotnull, &1, pg_attribute.atthasdef "
 			"from pg_class, pg_attribute "
 			"left join pg_attrdef on (pg_attrdef.adrelid = pg_attribute.attrelid and pg_attrdef.adnum = pg_attribute.attnum) "
-			"where pg_class.relname = '&1' "
+			"where pg_class.relname = '&2' "
 			"and (pg_class.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
-			"and pg_attribute.attname = '&2' "
+			"and pg_attribute.attname = '&3' "
 			"and pg_attribute.attnum > 0 and not pg_attribute.attisdropped "
 			"and pg_attribute.attrelid = pg_class.oid";
 
 		query_schema =
 			"select pg_attribute.attname, pg_attribute.atttypid::int, "
-			"pg_attribute.atttypmod, pg_attribute.attnotnull, pg_attrdef.adsrc, pg_attribute.atthasdef "
+			"pg_attribute.atttypmod, pg_attribute.attnotnull, &1, pg_attribute.atthasdef "
 			"from pg_class, pg_attribute "
 			"left join pg_attrdef on (pg_attrdef.adrelid = pg_attribute.attrelid and pg_attrdef.adnum = pg_attribute.attnum) "
-			"where pg_class.relname = '&1' "
-			"and (pg_class.relnamespace in (select oid from pg_namespace where nspname = '&3')) "
-			"and pg_attribute.attname = '&2' "
+			"where pg_class.relname = '&2' "
+			"and (pg_class.relnamespace in (select oid from pg_namespace where nspname = '&4')) "
+			"and pg_attribute.attname = '&3' "
 			"and pg_attribute.attnum > 0 and not pg_attribute.attisdropped "
 			"and pg_attribute.attrelid = pg_class.oid";
 	}
@@ -2263,25 +2274,25 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 	{
 		query =
 			"select pg_attribute.attname, pg_attribute.atttypid::int, "
-			"pg_attribute.atttypmod, pg_attribute.attnotnull, pg_attrdef.adsrc, pg_attribute.atthasdef, pg_collation.collname "
+			"pg_attribute.atttypmod, pg_attribute.attnotnull, &1, pg_attribute.atthasdef, pg_collation.collname "
 			"from pg_class, pg_attribute "
 			"left join pg_attrdef on (pg_attrdef.adrelid = pg_attribute.attrelid and pg_attrdef.adnum = pg_attribute.attnum) "
 			"left join pg_collation on (pg_collation.oid = pg_attribute.attcollation) "
-			"where pg_class.relname = '&1' "
+			"where pg_class.relname = '&2' "
 			"and (pg_class.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
-			"and pg_attribute.attname = '&2' "
+			"and pg_attribute.attname = '&3' "
 			"and pg_attribute.attnum > 0 and not pg_attribute.attisdropped "
 			"and pg_attribute.attrelid = pg_class.oid";
 
 		query_schema =
 			"select pg_attribute.attname, pg_attribute.atttypid::int, "
-			"pg_attribute.atttypmod, pg_attribute.attnotnull, pg_attrdef.adsrc, pg_attribute.atthasdef, pg_collation.collname "
+			"pg_attribute.atttypmod, pg_attribute.attnotnull, &1, pg_attribute.atthasdef, pg_collation.collname "
 			"from pg_class, pg_attribute "
 			"left join pg_attrdef on (pg_attrdef.adrelid = pg_attribute.attrelid and pg_attrdef.adnum = pg_attribute.attnum) "
 			"left join pg_collation on (pg_collation.oid = pg_attribute.attcollation) "
-			"where pg_class.relname = '&1' "
-			"and (pg_class.relnamespace in (select oid from pg_namespace where nspname = '&3')) "
-			"and pg_attribute.attname = '&2' "
+			"where pg_class.relname = '&2' "
+			"and (pg_class.relnamespace in (select oid from pg_namespace where nspname = '&4')) "
+			"and pg_attribute.attname = '&3' "
 			"and pg_attribute.attnum > 0 and not pg_attribute.attisdropped "
 			"and pg_attribute.attrelid = pg_class.oid";
 	}
@@ -2292,12 +2303,12 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 
 	if (get_table_schema(&table, &schema))
 	{
-		if (do_query(db, "Unable to get field info: &1", &res, query, 2, table, field))
+		if (do_query(db, "Unable to get field info: &1", &res, query, 3, get_default_value("pg_attrdef.adsrc"), table, field))
 			return TRUE;
 	}
 	else
 	{
-		if (do_query(db, "Unable to get field info: &1", &res, query_schema, 3, table, field, schema))
+		if (do_query(db, "Unable to get field info: &1", &res, query_schema, 4, get_default_value("pg_attrdef.adsrc"), table, field, schema))
 			return TRUE;
 	}
 
