@@ -2,14 +2,11 @@
 
   main.h
 
-  MySQL driver
-
-  Hacked by Nigel Gerrard from original code by
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -19,32 +16,24 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
 #ifndef __MAIN_H
 #define __MAIN_H
 
-extern "C"
-{
 #include "gambas.h"
 #include "gb_common.h"
 #include "../gb.db.h"
+#include "sqlite3.h"
 
 #ifndef __MAIN_C
 	extern GB_INTERFACE GB;
 	extern DB_INTERFACE DB;
 #endif
-}
 
-#define QUOTE_STRING "'"
-
-#define MAX_PATH 132						/* MAX LENGTH OF FILNAME PATH */
-#define TRUE 1
-#define FALSE 0
-
-/* Prototypes Required to allow cpp compilation */
-#include "gb.db.proto.h"
+#define QUOTE_STRING "\""
 
 #endif /* __MAIN_H */
