@@ -653,7 +653,7 @@ END_METHOD
 
 GB_DESC X11Desc[] =
 {
-  GB_DECLARE("X11", 0), GB_VIRTUAL_CLASS(),
+  GB_DECLARE_STATIC("_X11"),
   
   //GB_STATIC_METHOD("Init", NULL, CDESKTOP_init, NULL),
   
