@@ -247,11 +247,21 @@ BEGIN_METHOD(Font_TextHeight, GB_STRING text)
 END_METHOD
 
 
-BEGIN_METHOD(Font_RichTextWidth, GB_STRING text)
+BEGIN_METHOD(Font_TextSize, GB_STRING text)
+
+	GEOM_RECT *rect = GEOM.CreateRect();
+	rect->w = FONT->width(STRING(text), LENGTH(text));
+	rect->h = FONT->height(STRING(text), LENGTH(text));
+	GB.ReturnObject(rect);
+
+END_METHOD
+
+
+BEGIN_METHOD(Font_RichTextWidth, GB_STRING text; GB_INTEGER width)
 
 	float w;
 	
-	FONT->richTextSize(STRING(text), LENGTH(text), -1, &w, NULL);
+	FONT->richTextSize(STRING(text), LENGTH(text), VARGOPT(width, -1), &w, NULL);
 	GB.ReturnInteger(ceil(w));
 
 END_METHOD
@@ -267,6 +277,21 @@ BEGIN_METHOD(Font_RichTextHeight, GB_STRING text; GB_INTEGER width)
 END_METHOD
 
 
+BEGIN_METHOD(Font_RichTextSize, GB_STRING text; GB_INTEGER width)
+
+	float w, h;
+	GEOM_RECT *rect = GEOM.CreateRect();
+	
+	FONT->richTextSize(STRING(text), LENGTH(text), VARGOPT(width, -1), &w, &h);
+	
+	rect->w = ceil(w);
+	rect->h = ceil(h);
+	
+	GB.ReturnObject(rect);
+
+END_METHOD
+
+
 BEGIN_PROPERTY(Font_Fixed)
 
 	GB.ReturnBoolean(FONT->fixed());
@@ -398,9 +423,11 @@ GB_DESC CFontDesc[] =
 
 	GB_METHOD("TextWidth", "i", Font_TextWidth, "(Text)s"),
 	GB_METHOD("TextHeight", "i", Font_TextHeight, "(Text)s"),
+	GB_METHOD("TextSize", "Rect", Font_TextSize, "(Text)s"),
 
-	GB_METHOD("RichTextWidth", "i", Font_RichTextWidth, "(Text)s"),
+	GB_METHOD("RichTextWidth", "i", Font_RichTextWidth, "(Text)s[(Width)i]"),
 	GB_METHOD("RichTextHeight", "i", Font_RichTextHeight, "(Text)s[(Width)i]"),
+	GB_METHOD("RichTextSize", "Rect", Font_RichTextSize, "(Text)s[(Width)i]"),
 
 	GB_STATIC_METHOD("_get", "Font", Font_get, "(Font)s"),
 	#if 0
