@@ -3,16 +3,10 @@
   CSlider.cpp
 
   (c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
-  
-  GTK+ component
-  
-  Realizado para la Junta de Extremadura. 
-  Consejería de Educación Ciencia y Tecnología. 
-  Proyecto gnuLinEx
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -22,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -47,30 +42,24 @@ void gb_raise_slider_Click(gSlider *sender)
 }
 
 
-BEGIN_METHOD(CSLIDER_new, GB_OBJECT parent)
+BEGIN_METHOD(Slider_new, GB_OBJECT parent)
 
-	CCONTAINER *Parent=(CCONTAINER*)VPROP(GB_OBJECT);
-	Parent=(CCONTAINER*)GetContainer ((CWIDGET*)Parent);
-
-	THIS->widget=new gSlider(Parent->widget);
-	InitControl(THIS->widget,(CWIDGET*)THIS);
+	InitControl(new gSlider(CONTAINER(VARG(parent))),(CWIDGET*)THIS);
+	
 	SLIDER->onChange=gb_raise_slider_Click;
 	
 END_METHOD
 
-BEGIN_METHOD(CSCROLLBAR_new, GB_OBJECT parent)
+BEGIN_METHOD(ScrollBar_new, GB_OBJECT parent)
 
-	CCONTAINER *Parent=(CCONTAINER*)VPROP(GB_OBJECT);
-	Parent=(CCONTAINER*)GetContainer ((CWIDGET*)Parent);
-
-	THIS->widget=new gScrollBar(Parent->widget);
-	InitControl(THIS->widget,(CWIDGET*)THIS);
+	InitControl(new gScrollBar(CONTAINER(VARG(parent))),(CWIDGET*)THIS);
+	
 	SBAR->onChange=gb_raise_slider_Click;
 
 END_METHOD
 
 
-BEGIN_PROPERTY(CSLIDER_tracking)
+BEGIN_PROPERTY(Slider_Tracking)
 
 	if (READ_PROPERTY) { GB.ReturnBoolean(SLIDER->tracking()); return; }
 	SLIDER->setTracking(VPROP(GB_BOOLEAN));
@@ -78,7 +67,7 @@ BEGIN_PROPERTY(CSLIDER_tracking)
 END_PROPERTY
 
 
-BEGIN_PROPERTY(CSLIDER_value)
+BEGIN_PROPERTY(Slider_Value)
 
 	if (READ_PROPERTY) { GB.ReturnInteger(SLIDER->value()); return; }
 	SLIDER->setValue(VPROP(GB_INTEGER));
@@ -86,28 +75,28 @@ BEGIN_PROPERTY(CSLIDER_value)
 END_PROPERTY
 
 
-BEGIN_PROPERTY(CSLIDER_minval)
+BEGIN_PROPERTY(Slider_MinValue)
 
 	if (READ_PROPERTY) { GB.ReturnInteger(SLIDER->min()); return; }
 	SLIDER->setMin(VPROP(GB_INTEGER));	
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CSLIDER_maxval)
+BEGIN_PROPERTY(Slider_MaxValue)
 
 	if (READ_PROPERTY) { GB.ReturnInteger(SLIDER->max()); return; }
 	SLIDER->setMax(VPROP(GB_INTEGER));
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CSLIDER_linestep)
+BEGIN_PROPERTY(Slider_LineStep)
 
 	if (READ_PROPERTY) { GB.ReturnInteger(SLIDER->step()); return; }
 	SLIDER->setStep(VPROP(GB_INTEGER));
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CSLIDER_pagestep)
+BEGIN_PROPERTY(Slider_PageStep)
 
 	if (READ_PROPERTY) { GB.ReturnInteger(SLIDER->pageStep()); return; }
 	SLIDER->setPageStep(VPROP(GB_INTEGER));
@@ -121,43 +110,68 @@ BEGIN_PROPERTY(CSLIDER_mark)
 
 END_PROPERTY
 
-GB_DESC CSliderDesc[] =
+BEGIN_PROPERTY(Slider_DefaultSize)
+
+	GB.ReturnInteger(SLIDER->getDefaultSize());
+
+END_PROPERTY
+
+BEGIN_PROPERTY(Slider_Orientation)
+
+	if (READ_PROPERTY)
+		GB.ReturnInteger(SLIDER->orientation());
+	else
+		SLIDER->setOrientation(VPROP(GB_INTEGER));
+
+END_PROPERTY
+
+//-------------------------------------------------------------------------
+
+GB_DESC SliderDesc[] =
 {
   GB_DECLARE("Slider", sizeof(CSLIDER)), GB_INHERITS("Control"),
 
-  GB_METHOD("_new", NULL, CSLIDER_new, "(Parent)Container;"),
+  GB_METHOD("_new", 0, Slider_new, "(Parent)Container;"),
 
-  GB_PROPERTY("Tracking", "b", CSLIDER_tracking),
-  GB_PROPERTY("Value", "i", CSLIDER_value),
+  GB_PROPERTY("Tracking", "b", Slider_Tracking),
+  GB_PROPERTY("Value", "i", Slider_Value),
   GB_PROPERTY("Mark", "b", CSLIDER_mark),
-  GB_PROPERTY("MinValue", "i", CSLIDER_minval),
-  GB_PROPERTY("MaxValue", "i", CSLIDER_maxval),
-  GB_PROPERTY("Step", "i", CSLIDER_linestep),
-  GB_PROPERTY("PageStep", "i", CSLIDER_pagestep),
+  GB_PROPERTY("MinValue", "i", Slider_MinValue),
+  GB_PROPERTY("MaxValue", "i", Slider_MaxValue),
+  GB_PROPERTY("Step", "i", Slider_LineStep),
+  GB_PROPERTY("PageStep", "i", Slider_PageStep),
+  GB_PROPERTY("Orientation", "i", Slider_Orientation),
 
-  GB_EVENT("Change", NULL, NULL, &EVENT_Change),
+  GB_EVENT("Change", 0, 0, &EVENT_Change),
 
-	SLIDER_DESCRIPTION,
+  SLIDER_DESCRIPTION,
 
   GB_END_DECLARE
 };
 
-GB_DESC CScrollBarDesc[] =
+GB_DESC ScrollBarDesc[] =
 {
   GB_DECLARE("ScrollBar", sizeof(CSCROLLBAR)), GB_INHERITS("Control"),
 
-  GB_METHOD("_new", NULL, CSCROLLBAR_new, "(Parent)Container;"),
+  GB_METHOD("_new", 0, ScrollBar_new, "(Parent)Container;"),
 
-  GB_PROPERTY("Tracking", "b", CSLIDER_tracking),
-  GB_PROPERTY("Value", "i", CSLIDER_value),
-  GB_PROPERTY("MinValue", "i", CSLIDER_minval),
-  GB_PROPERTY("MaxValue", "i", CSLIDER_maxval),
-  GB_PROPERTY("Step", "i", CSLIDER_linestep),
-  GB_PROPERTY("PageStep", "i", CSLIDER_pagestep),
+  GB_PROPERTY_READ("DefaultSize", "i", Slider_DefaultSize),
 
-  GB_EVENT("Change", NULL, NULL, &EVENT_Change),
+  GB_PROPERTY("Tracking", "b", Slider_Tracking),
+  GB_PROPERTY("Value", "i", Slider_Value),
+  GB_PROPERTY("MinValue", "i", Slider_MinValue),
+  GB_PROPERTY("MaxValue", "i", Slider_MaxValue),
+  GB_PROPERTY("Step", "i", Slider_LineStep),
+  GB_PROPERTY("PageStep", "i", Slider_PageStep),
+  GB_PROPERTY("Orientation", "i", Slider_Orientation),
 
-	SCROLLBAR_DESCRIPTION,
+  GB_EVENT("Change", 0, 0, &EVENT_Change),
+
+  SCROLLBAR_DESCRIPTION,
+
+	GB_CONSTANT("Auto", "i", ORIENTATION_AUTO),
+	GB_CONSTANT("Horizontal", "i", ORIENTATION_HORIZONTAL),
+	GB_CONSTANT("Vertical", "i", ORIENTATION_VERTICAL),
 
   GB_END_DECLARE
 };
