@@ -3,16 +3,10 @@
   CWatcher.cpp
 
   (c) 2004-2005 - Daniel Campos Fernández <dcamposf@gmail.com>
-  
-  GTK+ component
-  
-  Realizado para la Junta de Extremadura. 
-  Consejería de Educación Ciencia y Tecnología. 
-  Proyecto gnuLinEx
-  
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -22,13 +16,13 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
 #define __CWATCHER_CPP
 
-#include <gtk/gtk.h>
 #include "main.h"
 #include "CWatcher.h"
 
@@ -38,18 +32,30 @@ DECLARE_EVENT(EVENT_Show);
 DECLARE_EVENT(EVENT_Hide);
 //DECLARE_EVENT(EVENT_Remove);
 
+static void connect_signals(GtkWidget *wid, void *_object);
+
+static bool is_disabled(CWATCHER *_object)
+{
+	gControl *ctrl = THIS->wid->widget;
+	return ctrl->_hidden_temp;
+}
+
 static void raise_show(GtkWidget *widget, CWATCHER *_object)
 {
-	GB.Raise(THIS, EVENT_Show, 0);
+	if (!is_disabled(THIS))
+		GB.Raise(THIS, EVENT_Show, 0);
 }
 
 static void raise_hide(GtkWidget *widget, CWATCHER *_object)
 {
-	GB.Raise(THIS, EVENT_Hide, 0);
+	if (!is_disabled(THIS))
+		GB.Raise(THIS, EVENT_Hide, 0);
 }
 
 static void raise_configure(GtkWidget *widget, GdkEventConfigure *e, CWATCHER *_object)
 {
+	GB.Ref(_object);
+	
 	if (THIS->x != e->x || THIS->y != e->y)
 	{
 		THIS->x = e->x;
@@ -63,43 +69,68 @@ static void raise_configure(GtkWidget *widget, GdkEventConfigure *e, CWATCHER *_
 		THIS->h = e->height;
 		GB.Raise(THIS, EVENT_Resize, 0);
 	}
+	
+	GB.Unref(POINTER(&_object));
 }
 
+static void cb_destroy(GtkWidget *widget, CWATCHER *_object)
+{
+	gControl *ctrl = THIS->wid->widget;
+	
+	if (ctrl->_no_delete)
+	{
+		connect_signals(ctrl->border, _object);
+	}
+	else
+	{
+		GB.Unref(POINTER(&THIS->wid));
+		THIS->wid = 0;
+	}
+}
+
+static void connect_signals(GtkWidget *wid, void *_object)
+{
+	g_signal_connect(G_OBJECT(wid), "map", G_CALLBACK(raise_show), _object);
+	g_signal_connect(G_OBJECT(wid), "unmap", G_CALLBACK(raise_hide), _object);
+	g_signal_connect(G_OBJECT(wid), "configure-event", G_CALLBACK(raise_configure), _object);
+	g_signal_connect(G_OBJECT(wid), "destroy", G_CALLBACK(cb_destroy), _object);
+}
 
 /** Watcher class *********************************************************/
 
 BEGIN_METHOD(CWATCHER_new, GB_OBJECT control)
 
-	GtkWidget *wid;
 	gControl *control;
 
-	THIS->wid=(CWIDGET*)VARG(control);
-	if (!THIS->wid) return;
+	THIS->wid = (CWIDGET*)VARG(control);
+	
+	if (GB.CheckObject(THIS->wid))
+		return;
+
+	//fprintf(stderr, "Watcher %p: Ref %p (%s %p)\n", _object, THIS->wid, GB.GetClassName(THIS->wid), THIS->wid);
+	GB.Ref((void *)THIS->wid);
+
 	control = THIS->wid->widget;
-	if (!control || !control->border)
-	{ 
-		THIS->wid=NULL; 
-		return; 
-	}
-
-	GB.Ref((void*)THIS->wid);
-
+	
 	THIS->x = control->left() - 1;
 	THIS->y = control->top() - 1;
 	THIS->w = control->width() - 1;
 	THIS->h = control->height() - 1;
 
-	wid = THIS->wid->widget->border;
-	g_signal_connect(G_OBJECT(wid), "show", G_CALLBACK(raise_show), _object);
-	g_signal_connect(G_OBJECT(wid), "hide", G_CALLBACK(raise_hide), _object);
-	g_signal_connect(G_OBJECT(wid), "configure-event", G_CALLBACK(raise_configure), _object);
+	connect_signals(control->border, THIS);
 
 END_METHOD
 
 BEGIN_METHOD_VOID(CWATCHER_free)
 
-	if (THIS->wid) 
+	//fprintf(stderr, "Watcher %p: UnRef %p %p ?\n", THIS, THIS->wid, THIS->wid ? THIS->wid->widget : 0);
+	
+	if (THIS->wid)
+	{
+		if (THIS->wid->widget)
+			g_signal_handlers_disconnect_matched(G_OBJECT(THIS->wid->widget->border), G_SIGNAL_MATCH_DATA, 0, (GQuark)0, NULL, NULL, _object);
 		GB.Unref(POINTER(&THIS->wid));
+	}
 
 END_METHOD
 
@@ -113,17 +144,18 @@ GB_DESC CWatcherDesc[] =
 {
   GB_DECLARE("Watcher", sizeof(CWATCHER)),
 
-  GB_METHOD("_new", NULL, CWATCHER_new, "(Control)Control;"),
-  GB_METHOD("_free", NULL, CWATCHER_free, NULL),
+  GB_METHOD("_new", 0, CWATCHER_new, "(Control)Control;"),
+  GB_METHOD("_free", 0, CWATCHER_free, 0),
 
   GB_PROPERTY("Control", "Control", CWATCHER_control),
 
-  GB_EVENT("Move", NULL, NULL, &EVENT_Move),
-  GB_EVENT("Resize", NULL, NULL, &EVENT_Resize),
-  GB_EVENT("Show", NULL, NULL, &EVENT_Show),
-  GB_EVENT("Hide", NULL, NULL, &EVENT_Hide),
+  GB_EVENT("Move", 0, 0, &EVENT_Move),
+  GB_EVENT("Resize", 0, 0, &EVENT_Resize),
+  GB_EVENT("Show", 0, 0, &EVENT_Show),
+  GB_EVENT("Hide", 0, 0, &EVENT_Hide),
 
   GB_CONSTANT("_DefaultEvent", "s", "Resize"),
 
   GB_END_DECLARE
 };
+
