@@ -3,16 +3,10 @@
   CWindow.h
 
   (c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
-  
-  GTK+ component
-  
-  Realizado para la Junta de Extremadura. 
-  Consejería de Educación Ciencia y Tecnología. 
-  Proyecto gnuLinEx
-  
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -22,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -37,11 +32,8 @@
 typedef  
 	struct
 	{
-		GB_BASE ob;
-		gControl *widget;
-		GB_VARIANT_VALUE tag;
+		CWIDGET ob;
 		int ret;
-		unsigned embed : 1;
 	}  
 	CWINDOW;
 
@@ -53,20 +45,24 @@ typedef
 extern GB_DESC CWindowMenusDesc[];
 extern GB_DESC CWindowControlsDesc[];
 extern GB_DESC CWindowDesc[];
+//extern GB_DESC CWindowTypeDesc[];
 extern GB_DESC CWindowsDesc[];
 extern GB_DESC CFormDesc[];
 
 extern CWINDOW *CWINDOW_Active;
+extern CWINDOW *CWINDOW_Main;
 
 #else
 
 #define THIS ((CWINDOW *)_object)
-#define WINDOW ((gMainWindow*)(THIS->widget))
+#define WINDOW ((gMainWindow*)THIS->ob.widget)
   
 #endif
 
-CWINDOW* WINDOW_get_main();
-void WINDOW_kill(CWINDOW *win);
+void CWINDOW_check_main_window(CWINDOW *win);
+bool CWINDOW_must_quit();
+void CWINDOW_close_all();
+void CWINDOW_delete_all();
 
 #endif
 
