@@ -48,6 +48,8 @@ static GtkWidget *_widget = NULL;
 
 static STYLE_T *_stl = NULL;
 
+static bool _internal = false;
+
 #ifdef GTK3
 static GtkStyleContext *get_style(GType type = G_TYPE_NONE)
 {
@@ -118,6 +120,9 @@ static GtkStyle *get_style(GType type = G_TYPE_NONE)
 #ifdef GTK3
 static bool begin_draw(int *x, int *y)
 {
+	if (_internal)
+		return FALSE;
+	
 	void *device = PAINT_get_current_device();
 	if (!device)
 		return TRUE;
@@ -184,6 +189,28 @@ static bool begin_draw(int *x, int *y)
 
 		_widget = wid->widget;
 	}
+	else if (GB.Is(device, CLASS_UserControl))
+	{
+		gContainer *wid;
+		GtkAllocation *a;
+		
+		if (PAINT_is_internal())
+		{
+			GB.Error("Cannot draw outside of 'Draw' event handler");
+			return TRUE;
+		}
+		
+		wid = (gContainer *)((CDRAWINGAREA *)device)->ob.widget;
+
+		_dr = wid->widget->window;
+		a = &wid->widget->allocation;
+		_dr_x = a->x;
+		_dr_y = a->y;
+		*x += _dr_x;
+		*y += _dr_y;
+
+		_widget = wid->widget;
+	}
 	else if (GB.Is(device, CLASS_Picture))
 	{
 		gPicture *pic = ((CPICTURE *)device)->picture;
@@ -208,7 +235,8 @@ static bool begin_draw(int *x, int *y)
 static void end_draw()
 {
 #ifdef GTK3
-	cairo_restore(_cr);
+	if (!_internal)
+		cairo_restore(_cr);
 	_cr = NULL;
 	if (_stl)
 	{
@@ -227,9 +255,12 @@ static void end_draw()
 #endif
 
 #ifndef GTK3
-	cairo_t *context = PAINT_get_current_context();
-	cairo_restore(context);
-	cairo_surface_mark_dirty(cairo_get_target(context));
+	if (!_internal)
+	{
+		cairo_t *context = PAINT_get_current_context();
+		cairo_restore(context);
+		cairo_surface_mark_dirty(cairo_get_target(context));
+	}
 #endif
 }
 
@@ -588,10 +619,11 @@ static void style_panel(int x, int y, int w, int h, int border, int state)
 	gColor col = 0;
 
 	if (border == BORDER_PLAIN)
-	{
+		col = gDesktop::getColor(gDesktop::LIGHT_FOREGROUND);
+	/*{
 		col = IMAGE.MergeColor(gDesktop::bgColor(), gDesktop::fgColor(), 0.5);
 		col = IMAGE.LighterColor(col);
-	}
+	}*/
 
 	gt_draw_border(_cr, style, get_state(state), border, col, x, y, w, h);
 
@@ -615,9 +647,7 @@ static void style_panel(int x, int y, int w, int h, int border, int state)
 		GdkGCValues values;
 		uint col;
 
-		col = IMAGE.MergeColor(gDesktop::bgColor(), gDesktop::fgColor(), 0.5);
-		col = IMAGE.LighterColor(col);
-
+		col = gDesktop::getColor(gDesktop::LIGHT_FOREGROUND);
 		fill_gdk_color(&values.foreground, col, gdk_drawable_get_colormap(_dr));
 		gc = gtk_gc_get(gdk_drawable_get_depth(_dr), gdk_drawable_get_colormap(_dr), &values, GDK_GC_FOREGROUND);
 		gdk_draw_rectangle(_dr, gc, FALSE, x, y, w - 1, h - 1);
@@ -649,7 +679,7 @@ static void style_box(int x, int y, int w, int h, int state, GB_COLOR color)
 {
 	STYLE_T *style = get_style(GTK_TYPE_ENTRY);
 
-	if (gApplication::fix_oxygen)
+	if (gApplication::_fix_oxygen)
 	{
 		x -= 3;
 		w += 6;
@@ -692,7 +722,7 @@ static void style_box(int x, int y, int w, int h, int state, GB_COLOR color)
 
 #else
 
-	if (gApplication::fix_breeze)
+	if (gApplication::_fix_breeze)
 		state &= ~GB_DRAW_STATE_HOVER;
 
 	GtkStateType st = get_state(state);
@@ -732,6 +762,43 @@ static void style_box(int x, int y, int w, int h, int state, GB_COLOR color)
 #endif
 }
 
+//-------------------------------------------------------------------------
+
+#ifdef GTK3
+void CSTYLE_paint_check(cairo_t *cr, int x, int y, int w, int h, int value, int state)
+{
+	_cr =cr;
+#else
+void CSTYLE_paint_check(GdkDrawable *dr, int x, int y, int w, int h, int value, int state)
+{
+	_dr = dr;
+#endif
+	_internal = true;
+	begin_draw(&x, &y);
+	style_check(x, y, w, h, value, state);
+	end_draw();
+	_internal = false;
+}
+
+#ifdef GTK3
+void CSTYLE_paint_option(cairo_t *cr, int x, int y, int w, int h, int value, int state)
+{
+	_cr =cr;
+#else
+void CSTYLE_paint_option(GdkDrawable *dr, int x, int y, int w, int h, int value, int state)
+{
+	_dr = dr;
+#endif
+	_internal = true;
+	begin_draw(&x, &y);
+	style_option(x, y, w, h, value, state);
+	end_draw();
+	_internal = false;
+}
+
+//-------------------------------------------------------------------------
+
+
 BEGIN_PROPERTY(Style_ScrollbarSize)
 
 	GB.ReturnInteger(gApplication::getScrollbarSize());
@@ -864,7 +931,7 @@ BEGIN_METHOD(Style_StateOf, GB_OBJECT control)
 
 	widget = control->widget;
 	state = GB_DRAW_STATE_NORMAL;
-	design = widget->design();
+	design = widget->isDesign();
 
 	if (!widget->isEnabled())
 		state |= GB_DRAW_STATE_DISABLED;
