@@ -21,7 +21,6 @@
 
 ***************************************************************************/
 
-#include <gtk/gtk.h>
 #include "widgets.h"
 
 #include <stdio.h>
@@ -32,24 +31,48 @@
 #include "gmainwindow.h"
 #include "gcontainer.h"
 
-static gControl* get_next_child_widget (gContainer *gtk_control, int *gtk_list, int gtk_count)
+static GList *_arrange_list = NULL;
+
+static gControl *get_next_child_widget (gContainer *gtk_control, int *gtk_index)
 {
 	gControl *ctrl;
 	
-	while ( (*gtk_list) < gtk_count )
+	for(;;)
 	{
-		ctrl = gtk_control->child(*gtk_list);
-		//fprintf(stderr, "get_next_child_widget: %d: %p: %s\n", *gtk_list, ctrl, ctrl->name());
-		(*gtk_list)++;
+		ctrl = gtk_control->child(*gtk_index);
+		if (!ctrl)
+			return NULL;
 		
-		if (!ctrl->border || !ctrl->widget || !ctrl->isVisible())
-			continue;
-		
-		return ctrl;
+		(*gtk_index)++;
+		if (ctrl->border && ctrl->widget && ctrl->isVisible())
+			return ctrl;
 	}
-	
-	//fprintf(stderr, "get_next_child_widget: ==> NULL\n");
-	return NULL; 
+}
+
+#ifdef GTK3
+static gboolean cb_draw(GtkWidget *wid, cairo_t *cr, gContainer *data)
+{
+	CUSERCONTROL_cb_draw(data, cr);
+	return false;
+}
+#else
+static gboolean cb_expose(GtkWidget *wid, GdkEventExpose *e, gContainer *data)
+{
+	CUSERCONTROL_cb_draw(data, e->region, wid->allocation.x, wid->allocation.y);
+	return false;
+}
+#endif
+
+static void cb_map(GtkWidget *widget, gContainer *sender)
+{
+	sender->setShown(true);
+	if (!sender->isTempHidden())
+		sender->arrangeLater();
+}
+
+static void cb_unmap(GtkWidget *widget, gContainer *sender)
+{
+	sender->setShown(false);
 }
 
 static void cb_arrange(gContainer *sender)
@@ -64,8 +87,11 @@ static void cb_before_arrange(gContainer *sender)
 		(*(sender->onBeforeArrange))(sender);
 }
 
-static void resize_container(gControl *cont, int w, int h)
+static void resize_container(gContainer *cont, int w, int h)
 {
+	/*w += cont->width() - cont->containerWidth();
+	h += cont->height() - cont->containerHeight();*/
+	
 	if (w >= 0 && h >= 0)
 		cont->resize(w, h);
 }
@@ -78,12 +104,13 @@ static void resize_container(gControl *cont, int w, int h)
 #define GET_WIDGET(_object) _object
 #define GET_CONTAINER(_object) _object
 #define GET_ARRANGEMENT(_object) (((gContainer*)_object)->getArrangement())
-#define IS_EXPAND(_object) (((gControl*)_object)->expand())
-#define IS_IGNORE(_object) (((gControl*)_object)->ignore())
-#define IS_DESIGN(_object) (((gControl*)_object)->design())
+#define IS_EXPAND(_object) (((gControl*)_object)->isExpand())
+#define IS_IGNORE(_object) (((gControl*)_object)->isIgnore())
+#define IS_DESIGN(_object) (((gControl*)_object)->isDesign())
 #define IS_WIDGET_VISIBLE(_widget)  (((gControl*)_widget)->isVisible())
 
-#define CAN_ARRANGE(_object) (IS_WIDGET_VISIBLE(GET_WIDGET(_object)) || (((gControl *)_object)->isTopLevel() && ((gMainWindow *)_object)->opened))
+#define CAN_ARRANGE(_object) (((gContainer *)_object)->isShown() && !((gControl *)_object)->isDestroyed())
+//|| (((gControl *)_object)->isTopLevel() && ((gMainWindow *)_object)->opened))
 
 // BM: ClientX() & ClientY() are relative to the border.
 // We need X & Y relative to the container widget.
@@ -98,20 +125,19 @@ static void resize_container(gControl *cont, int w, int h)
 #define GET_WIDGET_W(_widget)  (((gControl*)_widget)->width())
 #define GET_WIDGET_H(_widget)  (((gControl*)_widget)->height())
 #define MOVE_WIDGET(_object, _widget, _x, _y)  (((gControl*)_widget)->move( _x, _y))
-#define RESIZE_WIDGET(_object, _widget, _w, _h)  (((gControl*)_widget)->resize( _w, _h))
-#define MOVE_RESIZE_WIDGET(_object, _widget, _x, _y, _w, _h) (((gControl*)_widget)->moveResize( _x, _y, _w, _h))
-#define RESIZE_CONTAINER(_widget, _cont, _w, _h)  resize_container((gControl *)(_cont), _w, _h) 
+#define RESIZE_WIDGET(_object, _widget, _w, _h)  (((gControl*)_widget)->resize( _w, _h, true))
+#define MOVE_RESIZE_WIDGET(_object, _widget, _x, _y, _w, _h) (((gControl*)_widget)->moveResize(_x, _y, _w, _h, true))
+#define RESIZE_CONTAINER(_object, _cont, _w, _h)  resize_container((gContainer *)(_cont), _w, _h) 
 
 #define INIT_CHECK_CHILDREN_LIST(_widget) \
-	gContainer *gtk_control=(gContainer*)_widget; \
-	int gtk_list=0; \
-	int gtk_count = gtk_control->childCount();
+	gContainer *gtk_control = (gContainer*)_widget; \
+	int gtk_index = 0; \
 
-#define HAS_CHILDREN() (gtk_count != 0)
+#define HAS_CHILDREN() (gtk_control->childCount() != 0)
 
-#define RESET_CHILDREN_LIST() gtk_list=0;
+#define RESET_CHILDREN_LIST() gtk_index = 0;
 
-#define GET_NEXT_CHILD_WIDGET() get_next_child_widget (gtk_control,&gtk_list,gtk_count)
+#define GET_NEXT_CHILD_WIDGET() get_next_child_widget(gtk_control, &gtk_index)
 
 #define GET_OBJECT_FROM_WIDGET(_widget) ((void*)_widget)
 
@@ -126,21 +152,15 @@ static void resize_container(gControl *cont, int w, int h)
 
 #include "gb.form.arrangement.h"
 
-int gContainer::_arrangement_level = 0;
-
 void gContainer::performArrange()
 {
 	if (_no_arrangement)
-	{
 		_did_arrangement = true;
-		return;
+	else
+	{
+		_did_arrangement = false;
+		arrangeContainer((void *)this);
 	}
-
-	if (!gApplication::allEvents()) return;
-
-	_did_arrangement = false;
-
-	arrangeContainer((void*)this);
 }
 
 
@@ -222,6 +242,25 @@ void gContainer::getMaxSize(int xc, int yc, int wc, int hc, int *w, int *h)
 	arr->locked = locked;
 }
 
+void gContainer::decide(gControl *child, bool *width, bool *height)
+{
+	*width = *height = FALSE;
+	
+	if (!isShown() || !child->_allow_show || child->isIgnore() || autoResize())
+		return;
+	
+	if ((arrange() == ARRANGE_VERTICAL)
+	    || (arrange() == ARRANGE_HORIZONTAL && child->isExpand())
+	    || (arrange() == ARRANGE_ROW && child->isExpand())
+		  || (arrange() == ARRANGE_FILL))
+		*width = TRUE;
+	
+	if ((arrange() == ARRANGE_HORIZONTAL)
+	    || (arrange() == ARRANGE_VERTICAL && child->isExpand())
+	    || (arrange() == ARRANGE_COLUMN && child->isExpand())
+		  || (arrange() == ARRANGE_FILL))
+		*height = TRUE;
+}
 
 void gContainer::initialize()
 {
@@ -230,6 +269,7 @@ void gContainer::initialize()
 	radiogroup = NULL;
 	onArrange = NULL;
 	onBeforeArrange = NULL;
+	
 	_proxyContainer = NULL;
 	_proxyContainerFor = NULL;
 	_client_x = -1;
@@ -237,17 +277,23 @@ void gContainer::initialize()
 	_client_w = 0;
 	_client_h = 0;
 	_no_arrangement = 0;
-	//onInsert = NULL;
+	_did_arrangement = false;
+	_is_container = true;
+	_user_container = false;
+	_shown = false;
+	_arrange_later = false;
 	
 	arrangement.mode = 0;
-	arrangement.spacing = 0;
+	arrangement.spacing = false;
 	arrangement.padding = 0;
 	arrangement.autoresize = false;
 	arrangement.locked = false;
 	arrangement.user = false;
 	arrangement.margin = false;
-	arrangement.indent = 0;
+	arrangement.indent = false;
+	arrangement.centered = false;
 	arrangement.invert = false;
+	arrangement.paint = false;
 }
 
 gContainer::gContainer() 
@@ -266,10 +312,13 @@ gContainer::~gContainer()
 {
 	int i;
 	
+	resetArrangeLater();
+	
 	for (i = 0; i < childCount(); i++)
 		child(i)->removeParent();
 	
 	g_ptr_array_unref(_children);
+	_children = NULL;
 	
 	if (radiogroup) { g_object_unref(G_OBJECT(radiogroup)); radiogroup=NULL; }
 }
@@ -348,17 +397,24 @@ void gContainer::setMargin(bool vl)
 	}
 }
 
-void gContainer::setIndent(int vl)
+void gContainer::setIndent(bool vl)
 {
-	if (vl < 0)
-		vl = 1;
-	if (vl >= 0 && vl <= 7 && vl != arrangement.indent) 
+	if (vl != arrangement.indent) 
 	{
 		arrangement.indent = vl;
 		performArrange();
 	}
 }
 
+void gContainer::setCentered(bool vl)
+{
+	if (vl != arrangement.centered) 
+	{
+		arrangement.centered = vl;
+		performArrange();
+	}
+}
+
 void gContainer::setAutoResize(bool vl)
 {
 	if (vl != arrangement.autoresize)
@@ -368,13 +424,20 @@ void gContainer::setAutoResize(bool vl)
 	}
 }
 
-void gContainer::setUser(bool vl)
+void gContainer::setUser()
 {
-	if (vl != arrangement.user)
-	{
-		arrangement.user = vl;
-		performArrange();
-	}
+	if (arrangement.user)
+		return;
+	
+	arrangement.user = true;
+	performArrange();
+	updateDesignChildren();
+}
+
+void gContainer::setPaint()
+{
+	arrangement.paint = true;
+	ON_DRAW(border, this, cb_expose, cb_draw);
 }
 
 void gContainer::setInvert(bool vl)
@@ -486,7 +549,7 @@ int gContainer::clientWidth()
 		    && a.width > 0 && a.height > 0)
 		{
 			//g_debug("clientWidth: %s: %d", name(), width());
-			a.x = x(); a.y = y(); a.width = width(); a.height = height();
+			a.width = width(); a.height = height();
 			gt_disable_warnings(true);
 			gtk_widget_size_allocate(widget, &a);
 			gt_disable_warnings(false);
@@ -527,7 +590,7 @@ int gContainer::clientHeight()
 		    && a.width > 0 && a.height > 0)
 		{
 			//g_debug("clientHeight: %s: %d", name(), height());
-			a.x = x(); a.y = y(); a.width = width(); a.height = height();
+			a.width = width(); a.height = height();
 			//gt_disable_warnings(true);
 			gtk_widget_size_allocate(widget, &a);
 			//gt_disable_warnings(false);
@@ -565,17 +628,21 @@ void gContainer::insert(gControl *child, bool realize)
 	g_ptr_array_add(_children, child);
 	
 	if (realize)
-		child->visible = true;
+		child->_visible = true;
     
 	//g_debug("gContainer::insert: visible = %d", isReallyVisible());
-	performArrange();
+	if (!realize)
+		performArrange();
 	//fprintf(stderr, "--> %d %d %d %d\n", child->x(), child->y(), child->width(), child->height());
-	updateFocusChain();
 
 	if (realize)
 	{
-    gtk_widget_realize(child->border);
-		gtk_widget_show_all(child->border);
+    //gtk_widget_realize(child->border);
+		//gtk_widget_show(child->border);
+		if (child->frame)
+			gtk_widget_show(child->frame);
+		if (child->widget != child->border)
+			gtk_widget_show(child->widget);
 	}
 
 #ifndef GTK3
@@ -583,12 +650,14 @@ void gContainer::insert(gControl *child, bool realize)
 	if (hasForeground() && !child->_fg_set) child->setForeground();
 #endif
   child->updateFont();
+	
+	if ((isUser() && isDesign()) || isDesignIgnore())
+		child->setDesign(true);
 }
 
 void gContainer::remove(gControl *child)
 {
 	g_ptr_array_remove(_children, child);
-	updateFocusChain();
 }
 
 
@@ -647,7 +716,7 @@ void gContainer::setBackground(gColor color)
 }
 #endif
 
-#ifdef GTK3
+/*#ifdef GTK3
 void gContainer::updateColor()
 {
 	int i;
@@ -655,7 +724,7 @@ void gContainer::updateColor()
 	for (i = 0; i < childCount(); i++)
 		gContainer::child(i)->updateColor();
 }
-#endif
+#endif*/
 
 void gContainer::setForeground(gColor color)
 {
@@ -677,58 +746,38 @@ GtkWidget *gContainer::getContainer()
 	return widget;
 }
 
-gControl *gContainer::findFirstFocus()
+void gContainer::connectBorder()
 {
-	int i;
-	gControl *ch;
-	
-	for (i = 0; i < childCount(); i++)
-	{
-		ch = child(i);
-		if (ch->isContainer())
-		{
-			ch = ((gContainer *)ch)->findFirstFocus();
-			if (ch)
-				return ch;
-		}
-		else
-		{
-			if (gtk_widget_get_can_focus(ch->widget) && !((ch->getClass() == Type_gButton) && ((gButton *)ch)->hasShortcut()))
-				return ch;
-		}
-	}
-	
-	return NULL;
+	g_signal_connect_after(G_OBJECT(border), "map", G_CALLBACK(cb_map), (gpointer)this);	
+	g_signal_connect_after(G_OBJECT(border), "unmap", G_CALLBACK(cb_unmap), (gpointer)this);	
 }
 
-void gContainer::resize(int w, int h)
+bool gContainer::resize(int w, int h, bool no_decide)
 {
-	if (w == bufW && h == bufH)
-		return;
-		
+	if (gControl::resize(w, h, no_decide))
+		return true;
+
 	_client_w = 0;
 	_client_h = 0;
 	
-	gControl::resize(w, h);
 	performArrange();
+	return false;
 }
 
 void gContainer::setVisible(bool vl)
 {
-	bool arr;
+	//bool arr;
 	
 	if (vl == isVisible())
 		return;
 	
-	arr = vl && !isVisible();
-	
 	gControl::setVisible(vl);
 	
-	if (arr)
-		performArrange();
+	/*if (arr)
+		performArrange();*/
 }
 
-void gContainer::updateFocusChain()
+/*void gContainer::updateFocusChain()
 {
 	GList *chain = NULL;
 	int i;
@@ -747,7 +796,7 @@ void gContainer::updateFocusChain()
 	gtk_container_set_focus_chain(GTK_CONTAINER(widget), chain);
 	
 	g_list_free(chain);
-}
+}*/
 
 void gContainer::updateFont()
 {
@@ -757,6 +806,9 @@ void gContainer::updateFont()
 
 	for (i = 0; i < childCount(); i++)
 		child(i)->updateFont();
+
+	if (arrangement.paint)
+		CUSERCONTROL_cb_font(this);
 }
 
 void gContainer::moveChild(gControl *child, int x, int y)
@@ -840,3 +892,80 @@ void gContainer::clear()
 		ch->destroy();
 	}
 }
+
+void gContainer::updateDesignChildren()
+{
+	int i;
+	gContainer *cont;
+
+	if (!isDesign())
+		return;
+	
+	if (!isUser() && !isDesignIgnore())
+		return;
+	
+	if (isUserContainer() && !_proxyContainer)
+		return;
+	
+	cont = isDesignIgnore() ? this : proxyContainer();
+	
+	for (i = 0; i < cont->childCount(); i++)
+		cont->child(i)->setDesign(true);
+}
+
+void gContainer::setDesign(bool ignore)
+{
+	if (isDesign())
+		return;
+	
+	gControl::setDesign(ignore);
+	updateDesignChildren();
+}
+
+void gContainer::setProxyContainer(gContainer *proxy)
+{
+	if (_proxyContainer != this)
+		_proxyContainer = proxy;
+	else
+		_proxyContainer = NULL;
+	
+	updateDesignChildren();
+}
+
+void gContainer::resetArrangeLater()
+{
+	if (_arrange_later)
+	{
+		_arrange_later = false;
+		_arrange_list = g_list_remove(_arrange_list, this);
+	}
+}
+
+void gContainer::arrangeLater()
+{
+	if (!_arrange_later)
+	{
+		_arrange_later = true;
+		_arrange_list = g_list_prepend(_arrange_list, (gpointer)this);
+	}
+}
+
+void gContainer::postArrange()
+{
+	GList *iter;
+	gContainer *cont;
+
+	if (!_arrange_list) return;
+
+	for(;;)
+	{
+		iter = g_list_first(_arrange_list);
+		if (!iter)
+			break;
+		cont = (gContainer *)iter->data;
+		cont->resetArrangeLater();
+		cont->performArrange();
+	}
+
+	_arrange_list = NULL;
+}
