@@ -2,13 +2,11 @@
 
   CProxy.h
 
-  Advanced Network component
-
   (c) 2003-2008 Daniel Campos Fernández <dcamposf@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,9 +16,11 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
+
 #ifndef __CPROXY_H
 #define __CPROXY_H
 
@@ -33,10 +33,6 @@
 
 extern GB_DESC CProxyDesc[];
 
-#else
-
-#define THIS (&((CCURL *)_object)->proxy)
-
 #endif
 
 #endif
