@@ -21,6 +21,8 @@
 
 ***************************************************************************/
 
+#define __GB_CURL_C
+
 /*****************************
  NOTE THAT :
  libcurl <= 7.10.7 lacks CURLE_LDAP_INVALID_URL and CURLE_FILESIZE_EXCEEDED constants
@@ -33,6 +35,8 @@
 #include "gbcurl.h"
 #include "CCurl.h"
 
+CURL_PROXY CURL_default_proxy = { CURLPROXY_HTTP, CURLAUTH_NONE, NULL, NULL, NULL, NULL };
+
 static char *_protocols[] = { "ftp://", "ftps://", "http://", "https://", NULL };
 
 static void warning(const char *msg)
@@ -219,15 +223,13 @@ bool CURL_check_userpwd(CURL_USER *user)
 
 void CURL_proxy_init(CURL_PROXY *proxy)
 {
-	proxy->type = CURLPROXY_HTTP;
-	proxy->auth = CURLAUTH_NONE;
-	proxy->user = NULL;
-	proxy->pwd = NULL;
-	proxy->host = NULL;
-	proxy->userpwd = NULL;
+	*proxy = CURL_default_proxy;
+	GB.RefString(proxy->host);
+	GB.RefString(proxy->user);
+	GB.RefString(proxy->pwd);
+	GB.RefString(proxy->userpwd);
 }
 
-
 void CURL_proxy_clear(CURL_PROXY *proxy)
 {
 	GB.FreeString(&proxy->host);
@@ -236,6 +238,11 @@ void CURL_proxy_clear(CURL_PROXY *proxy)
 	GB.FreeString(&proxy->userpwd);
 }
 
+void CURL_default_proxy_clear()
+{
+	CURL_proxy_clear(&CURL_default_proxy);
+}
+
 
 void CURL_proxy_set(CURL_PROXY *proxy, CURL *curl)
 {
