@@ -60,16 +60,16 @@ DECLARE_EVENT (EVENT_Write);
 
 GB_STREAM_DESC SocketStream = 
 {
-	open: CSocket_stream_open,
-	close: CSocket_stream_close,
-	read: CSocket_stream_read,
-	write: CSocket_stream_write,
-	seek: CSocket_stream_seek,
-	tell: CSocket_stream_tell,
-	flush: CSocket_stream_flush,
-	/*eof: CSocket_stream_eof,
-	lof: CSocket_stream_lof,*/
-	handle: CSocket_stream_handle
+	.open = CSocket_stream_open,
+	.close = CSocket_stream_close,
+	.read = CSocket_stream_read,
+	.write = CSocket_stream_write,
+	.seek = CSocket_stream_seek,
+	.tell = CSocket_stream_tell,
+	.flush = CSocket_stream_flush,
+	/*.eof = CSocket_stream_eof,
+	.lof = CSocket_stream_lof,*/
+	.handle = CSocket_stream_handle
 };
 
 #if DEBUG_ME
@@ -177,10 +177,18 @@ static void CSocket_close(CSOCKET *_object)
 		set_status(THIS, NET_INACTIVE);
 	}
 	
+	if (THIS->timer)
+		GB.Unref(POINTER(&THIS->timer));
+	
 	if (THIS->OnClose)
 		THIS->OnClose(_object);
 }
 
+static int connect_timeout(void *_object)
+{
+	CSocket_stream_internal_error(THIS, NET_CONNECTION_TIMEOUT, TRUE);
+	return TRUE;
+}
 
 /*
 	This function is called by DnsClient to inform
@@ -217,9 +225,11 @@ void CSocket_CallBackFromDns(void *_object)
 	myval = connect(SOCKET->socket,(struct sockaddr*)&(THIS->Server), sizeof(struct sockaddr));
 	SOCKET_set_blocking(SOCKET, TRUE);
 	
-	if (!myval || errno == EINPROGRESS) // Rhis is the good answer : connect in progress
+	if (!myval || errno == EINPROGRESS) // this is the good answer : connect in progress
 	{
 		set_status(THIS, NET_CONNECTING);
+		if (SOCKET->timeout > 0)
+			THIS->timer = GB.Every(SOCKET->timeout, (GB_TIMER_CALLBACK)connect_timeout, (intptr_t)THIS);
 		GB.Watch(SOCKET->socket, GB_WATCH_WRITE, (void *)CSocket_CallBackConnecting, (intptr_t)THIS);
 	}
 	else
@@ -294,6 +304,8 @@ void CSocket_CallBackConnecting(int t_sock,int type,intptr_t param)
 	
 	GB.Ref(THIS);
 	GB.Post(CSocket_post_connected,(intptr_t)THIS);
+	
+	GB.Unref(POINTER(&THIS->timer));
 }
 
 /*******************************************************************
