@@ -2,14 +2,12 @@
 
   regexp.h
 
-  PCRE Regexp routines
-
   (c) 2004 Rob Kudla <pcre-component@kudla.org>
-  (c) 2000-2003 Benot Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -19,7 +17,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -39,26 +38,26 @@ extern GB_DESC CRegexpSubmatchDesc[];
 #else
 
 typedef
-struct {
-  GB_BASE ob;
-  char *subject;
-  char *pattern;
-  int *ovector;
-  int rc;
-  int compiled;
-  int eopts;
-  int copts;
-  pcre *code;
-  char **smcache;
-  int _submatch;
-}
-CREGEXP;
+	struct 
+	{
+		GB_BASE ob;
+		char *subject;
+		char *pattern;
+		int *ovector;
+		int ovecsize;
+		int count;
+		int eopts;
+		int copts;
+		pcre *code;
+		int _submatch;
+		int error;
+	}
+	CREGEXP;
 
-#define THIS  OBJECT(CREGEXP)
+#define THIS OBJECT(CREGEXP)
 
 #endif
 
-void REGEXP_init(void);
-void REGEXP_exit(void);
+bool REGEXP_match(const char *subject, int lsubject, const char *pattern, int lpattern, int coptions, int eoptions);
 
 #endif
