@@ -35,6 +35,11 @@
 #include "CWindow.h"
 #include "CMenu.h"
 
+#include "main.h"
+#ifdef QT5
+#include <QWindow>
+#endif
+
 //#define DEBUG_MENU 1
 
 DECLARE_EVENT(EVENT_Click);
@@ -391,11 +396,12 @@ BEGIN_METHOD(Menu_new, GB_OBJECT parent; GB_BOOLEAN hidden)
 
 		if (!menu->menu)
 		{
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0) && QT_VERSION < QT_VERSION_CHECK(5,5,0)
-			menu->menu = new MyMenu();
-#else
-			menu->menu = new QMenu();
-#endif
+			#if QT_VERSION >= QT_VERSION_CHECK(5,0,0) && QT_VERSION < QT_VERSION_CHECK(5,5,0)
+				menu->menu = new MyMenu();
+			#else
+				menu->menu = new QMenu();
+			#endif
+			
 			menu->menu->setSeparatorsCollapsible(true);
 			((QAction *)(menu->widget.widget))->setMenu(menu->menu);
 
@@ -422,6 +428,7 @@ BEGIN_METHOD(Menu_new, GB_OBJECT parent; GB_BOOLEAN hidden)
 		if (!menuBar)
 		{
 			menuBar = new QMenuBar(topLevel);
+			menuBar->setNativeMenuBar(false);
 			//menuBar->setAutoFillBackground(true);
 			window->menuBar = menuBar;
 		}
@@ -1026,9 +1033,25 @@ void CMenu::slotShown(void)
 {
 	static bool init = FALSE;
 
-	//qDebug("slotShown: sender = %p  menuAction = %p", sender(), ((QMenu *)sender())->menuAction());
 	GET_MENU_SENDER(menu);
+	if (!menu)
+		return;
 	HANDLE_PROXY(menu);
+
+	//fprintf(stderr, "slotShown: %s: menuAction = %s\n", menu->widget.name, TO_UTF8(((QMenu *)sender())->menuAction()->text()));
+	
+	#ifdef QT5
+	if (menu->menu->windowHandle())
+	{
+		QWidget *parent = qApp->activePopupWidget();
+		if (!parent) parent = qApp->activeWindow();
+		if (parent)
+		{
+			//fprintf(stderr, "set menu transient to %p\n", parent->windowHandle());
+			menu->menu->windowHandle()->setTransientParent(parent->windowHandle());
+		}
+	}
+	#endif
 	
 	GB.Ref(menu);
 	
@@ -1052,6 +1075,8 @@ void CMenu::slotHidden(void)
 {
 	//qDebug("slotHidden: sender = %p  menuAction = %p", sender(), ((QMenu *)sender())->menuAction());
 	GET_MENU_SENDER(menu);
+	if (!menu)
+		return;
 	HANDLE_PROXY(menu);
 
 	menu->opened = FALSE;
