@@ -51,6 +51,7 @@
 #include <QEventLoop>
 #include <QDesktopWidget>
 #include <QPaintDevice>
+#include <QLibraryInfo>
 
 #include "gb.image.h"
 #include "gb.qt.h"
@@ -63,7 +64,6 @@
 #include "CWindow.h"
 #include "CButton.h"
 #include "CContainer.h"
-#include "CLabel.h"
 #include "CTextBox.h"
 #include "CTextArea.h"
 #include "CMenu.h"
@@ -73,20 +73,18 @@
 #include "CColor.h"
 #include "CConst.h"
 #include "CCheckBox.h"
-#include "CFrame.h"
 #include "CRadioButton.h"
 #include "CTabStrip.h"
 #include "CDialog.h"
 #include "CPicture.h"
 #include "CImage.h"
+#include "canimation.h"
 #include "CClipboard.h"
 #include "CDraw.h"
 #include "CWatch.h"
 #include "CDrawingArea.h"
-#include "CMessage.h"
 #include "CSlider.h"
 #include "CScrollBar.h"
-#include "CMovieBox.h"
 #include "CWatcher.h"
 #include "cprinter.h"
 #include "csvgimage.h"
@@ -107,7 +105,7 @@
 #include <QAbstractNativeEventFilter>
 #endif
 
-#include "fix_breeze.h"
+#include "fix_style.h"
 #include "main.h"
 
 /*#define DEBUG*/
@@ -128,6 +126,7 @@ int MAIN_x11_last_key_code = 0;
 bool MAIN_debug_busy = false;
 bool MAIN_init = false;
 bool MAIN_key_debug = false;
+bool MAIN_right_to_left = false;
 
 GB_CLASS CLASS_Control;
 GB_CLASS CLASS_Container;
@@ -144,6 +143,7 @@ GB_CLASS CLASS_Printer;
 GB_CLASS CLASS_Image;
 GB_CLASS CLASS_SvgImage;
 GB_CLASS CLASS_TextArea;
+GB_CLASS CLASS_ComboBox;
 
 static bool in_event_loop = false;
 static int _no_destroy = 0;
@@ -154,7 +154,7 @@ static bool _check_quit_posted = false;
 static int _prevent_quit = 0;
 
 #ifndef NO_X_WINDOW
-static void (*_x11_event_filter)(XEvent *) = 0;
+static int (*_x11_event_filter)(XEvent *) = 0;
 #endif
 
 static QHash<void *, void *> _link_map;
@@ -178,6 +178,9 @@ static void myMessageHandler(QtMsgType type, const QMessageLogContext &context,
 	
 	if (msg == "QXcbClipboard: SelectionRequest too old")
 		return;
+	
+	if (msg.startsWith("QXcbConnection: ") && msg.contains("(TranslateCoords)"))
+		return;
 
 	_previousMessageHandler(type, context, msg);
 }
@@ -571,7 +574,7 @@ void MyApplication::commitDataRequested(QSessionManager &session)
 
 //---------------------------------------------------------------------------
 
-static void x11_set_event_filter(void (*filter)(XEvent *))
+static void x11_set_event_filter(int (*filter)(XEvent *))
 {
 	_x11_event_filter = filter;
 }
@@ -681,6 +684,7 @@ public:
 					xev.xconfigure.override_redirect = e->override_redirect;
 					break;
 				}
+				
 				case XCB_PROPERTY_NOTIFY:
 				{
 					xcb_property_notify_event_t *e = (xcb_property_notify_event_t *)ev;
@@ -729,16 +733,20 @@ public:
 					xev.xclient.window = e->window;
 					xev.xclient.message_type = e->type;
 					xev.xclient.format = e->format;
-					memcpy(&xev.xclient.data, &e->data, 20);
+					xev.xclient.data.l[0] = e->data.data32[0];
+					xev.xclient.data.l[1] = e->data.data32[1];
+					xev.xclient.data.l[2] = e->data.data32[2];
+					xev.xclient.data.l[3] = e->data.data32[3];
+					xev.xclient.data.l[4] = e->data.data32[4];
 					break;
 				}
 
 				default:
-					//qDebug("gb.qt5: warning: unhandled xcb event: %d", type);
+					qDebug("gb.qt5: warning: unhandled xcb event: %d", type);
 					return false;
 			}
 
-			(*_x11_event_filter)(&xev);
+			return (*_x11_event_filter)(&xev) != 0;
 		}
 
 		return false;
@@ -758,7 +766,7 @@ bool MyApplication::x11EventFilter(XEvent *e)
 		MAIN_x11_last_key_code = e->xkey.keycode;
 
 	if (_x11_event_filter)
-		(*_x11_event_filter)(e);
+		return (*_x11_event_filter)(e);
 
 	return false;
 }
@@ -887,7 +895,9 @@ static void QT_InitEventLoop(void)
 
 static bool try_to_load_translation(QString &locale)
 {
-#ifdef QT5
+	// QLocale::system().name()
+	return _translator->load("qt_" + locale, QLibraryInfo::location(QLibraryInfo::TranslationsPath));
+/*#ifdef QT5
 	return (!_translator->load(QString("qt_") + locale, QString(getenv("QTDIR")) + "/translations")
 		  && !_translator->load(QString("qt_") + locale, QString("/usr/lib/qt5/translations"))
 		  && !_translator->load(QString("qt_") + locale, QString("/usr/share/qt5/translations")));
@@ -895,7 +905,7 @@ static bool try_to_load_translation(QString &locale)
 	return (!_translator->load(QString("qt_") + locale, QString(getenv("QTDIR")) + "/translations")
 		  && !_translator->load(QString("qt_") + locale, QString("/usr/lib/qt4/translations"))
 		  && !_translator->load(QString("qt_") + locale, QString("/usr/share/qt4/translations")));
-#endif
+#endif*/
 }
 
 static void init_lang(char *lang, bool rtl)
@@ -903,6 +913,8 @@ static void init_lang(char *lang, bool rtl)
 	int pos;
 	QString locale(lang);
 
+	MAIN_right_to_left = rtl;
+	
 	pos = locale.lastIndexOf(".");
 	if (pos >= 0) locale = locale.left(pos);
 
@@ -996,6 +1008,7 @@ static void hook_quit()
 
 	CWINDOW_close_all(true);
 	CWINDOW_delete_all(true);
+	CMOUSE_set_control(NULL);
 
 	qApp->sendPostedEvents(); //processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::DeferredDeletion, 0);
 	qApp->sendPostedEvents(0, QEvent::DeferredDelete);
@@ -1025,6 +1038,8 @@ static void hook_loop()
 
 static void hook_wait(int duration)
 {
+	static bool _warning = FALSE;
+	
 	if (MyDrawingArea::inAnyDrawEvent())
 	{
 		GB.Error("Wait is forbidden during a repaint event");
@@ -1032,15 +1047,25 @@ static void hook_wait(int duration)
 	}
 
 	MAIN_in_wait++;
+	
 	if (duration > 0)
 	{
 		if (CKEY_is_valid())
-			fprintf(stderr, QT_NAME ": warning: calling the event loop during a keyboard event handler is ignored\n");
+		{
+			if (!_warning)
+			{
+				fprintf(stderr, QT_NAME ": warning: calling the event loop during a keyboard event handler is ignored\n");
+				_warning = TRUE;
+			}
+		}
 		else
 			qApp->processEvents(QEventLoop::AllEvents, duration);
 	}
+	else if (duration < 0)
+		qApp->processEvents(QEventLoop::AllEvents | QEventLoop::WaitForMoreEvents);
 	else
 		qApp->processEvents(QEventLoop::ExcludeUserInputEvents, duration);
+	
 	MAIN_in_wait--;
 }
 
@@ -1082,9 +1107,10 @@ static void hook_post(void)
 }
 
 
-static void hook_error(int code, char *error, char *where)
+static bool hook_error(int code, char *error, char *where, bool in_event_loop)
 {
 	QString msg;
+	int ret;
 
 	qApp->restoreOverrideCursor();
 	while (qApp->activePopupWidget())
@@ -1095,23 +1121,23 @@ static void hook_error(int code, char *error, char *where)
 
 	if (code > 0)
 	{
-		msg = msg + "[%1] %2.<br>%3";
+		msg = msg + "[%1] %2.<br><br><tt>%3</tt>";
 		msg = msg.arg(code).arg(TO_QSTRING(error)).arg(where);
 	}
 	else
 	{
-		msg = msg + "%1.<br>%2";
+		msg = msg + "%1.<br><br><tt>%2</tt>";
 		msg = msg.arg(TO_QSTRING(error)).arg(where);
 	}
 
 	release_grab();
 	MAIN_in_message_box++;
-	QMessageBox::critical(0, TO_QSTRING(GB.Application.Name()), msg);
+	ret = QMessageBox::critical(0, TO_QSTRING(GB.Application.Name()), msg, in_event_loop ? QMessageBox::Close | QMessageBox::Ignore : QMessageBox::Ok);
 	MAIN_in_message_box--;
 	unrelease_grab();
 	MAIN_check_quit();
 
-	//qApp->exit();
+	return ret == QMessageBox::Ignore;
 }
 
 static void QT_Init(void)
@@ -1119,6 +1145,7 @@ static void QT_Init(void)
 	static bool init = false;
 	QFont f;
 	char *env;
+	bool fix_style;
 
 	if (init)
 		return;
@@ -1136,6 +1163,8 @@ static void QT_Init(void)
 
 	/*fcntl(ConnectionNumber(qt_xdisplay()), F_SETFD, FD_CLOEXEC);*/
 
+	fix_style = false;
+	
 	if (::strcmp(qApp->style()->metaObject()->className(), "Breeze::Style") == 0)
 	{
 		env = getenv("GB_QT_NO_BREEZE_FIX");
@@ -1143,6 +1172,7 @@ static void QT_Init(void)
 		{
 			CSTYLE_fix_breeze = TRUE;
 			qApp->setStyle(new FixBreezeStyle);
+		fix_style = true;
 		}
 	}
 	else if (::strcmp(qApp->style()->metaObject()->className(), "Oxygen::Style") == 0)
@@ -1152,8 +1182,12 @@ static void QT_Init(void)
 		{
 			CSTYLE_fix_oxygen = TRUE;
 			qApp->setStyle(new FixBreezeStyle);
+			fix_style = true;
 		}
 	}
+	
+	if (!fix_style)
+		qApp->setStyle(new FixStyle);
 
 	MAIN_update_scale(qApp->desktop()->font());
 
@@ -1257,6 +1291,11 @@ static void *QT_CreatePicture(const QPixmap &p)
 	return CPICTURE_create(&p);
 }
 
+/*static void *QT_CreateImage(const Image &p)
+{
+	return CIMAGE_create(&p);
+}*/
+
 void MyApplication::linkDestroyed(QObject *qobject)
 {
 	void *object = _link_map.value(qobject, 0);
@@ -1307,14 +1346,18 @@ static void declare_tray_icon()
 	GB.Component.Declare(TrayIconsDesc);
 }
 
+static int QT_GetDesktopScale(void)
+{
+	return MAIN_scale;
+}
+
 extern "C" {
 
 GB_DESC *GB_CLASSES[] EXPORT =
 {
-	CBorderDesc, CColorDesc,
-	CAlignDesc, CArrangeDesc, CScrollDesc, CKeyDesc, CSelectDesc,
-	CMessageDesc,
-	CImageDesc, CPictureDesc,
+	BorderDesc, CColorDesc,
+	AlignDesc, ArrangeDesc, ScrollDesc, CKeyDesc, SelectDesc,
+	CImageDesc, CPictureDesc, AnimationDesc,
 	CFontDesc, CFontsDesc,
 	CMouseDesc, CCursorDesc, CPointerDesc,
 	CClipboardDesc, CDragDesc,
@@ -1323,15 +1366,14 @@ GB_DESC *GB_CLASSES[] EXPORT =
 	CControlDesc, ContainerChildrenDesc, ContainerDesc,
 	UserControlDesc, UserContainerDesc,
 	CMenuChildrenDesc, CMenuDesc,
-	CLabelDesc, CTextLabelDesc, CSeparatorDesc,
 	CButtonDesc, CToggleButtonDesc, CToolButtonDesc,
 	CCheckBoxDesc, CRadioButtonDesc,
 	CTextBoxSelectionDesc, CTextBoxDesc, CComboBoxItemDesc, CComboBoxDesc,
 	CTextAreaSelectionDesc, CTextAreaDesc,
-	CFrameDesc, CPanelDesc, CHBoxDesc, CVBoxDesc, CHPanelDesc, CVPanelDesc,
+	CPanelDesc, CHBoxDesc, CVBoxDesc, CHPanelDesc, CVPanelDesc,
 	CTabStripContainerChildrenDesc, CTabStripContainerDesc, CTabStripDesc,
 	CDrawingAreaDesc,
-	CSliderDesc, CMovieBoxDesc, CScrollBarDesc,
+	SliderDesc, ScrollBarDesc,
 	CWindowMenusDesc, CWindowControlsDesc, CWindowDesc, CWindowsDesc, CFormDesc,
 	CDialogDesc,
 #ifndef QT5
@@ -1366,6 +1408,7 @@ void *GB_QT4_1[] EXPORT =
 	(void *)QT_CreatePicture,
 	//(void *)QT_MimeSourceFactory,
 	(void *)QT_GetPixmap,
+	//(void *)QT_CreateImage,
 	(void *)QT_ToUtf8,
 	(void *)QT_GetLastUtf8Length,
 	(void *)QT_NewString,
@@ -1379,6 +1422,7 @@ void *GB_QT4_1[] EXPORT =
 	(void *)CWIDGET_get_background,
 	(void *)Control_Mouse,
 	(void *)CWIDGET_after_set_color,
+	(void *)QT_GetDesktopScale,
 	NULL
 };
 
@@ -1431,6 +1475,7 @@ int EXPORT GB_INIT(void)
 	CLASS_Image = GB.FindClass("Image");
 	CLASS_SvgImage = GB.FindClass("SvgImage");
 	CLASS_TextArea = GB.FindClass("TextArea");
+	CLASS_ComboBox = GB.FindClass("ComboBox");
 
 	QT_InitEventLoop();
 
