@@ -121,6 +121,18 @@ BEGIN_PROPERTY(TextBox_Text)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(TextBox_Placeholder)
+
+	GET_TEXT_BOX();
+
+	if (READ_PROPERTY)
+		RETURN_NEW_STRING(textbox->placeholderText());
+	else
+		textbox->setPlaceholderText(QSTRING_PROP());
+
+END_PROPERTY
+
+
 BEGIN_PROPERTY(TextBox_Length)
 
 	GB.ReturnInteger(TEXTBOX->text().length());
@@ -447,7 +459,7 @@ static void combo_set_editable(void *_object, bool ed)
 		QObject::connect(COMBOBOX->lineEdit(), SIGNAL(returnPressed()), &CTextBox::manager, SLOT(onActivate()));
 		//QObject::connect(COMBOBOX->lineEdit(), SIGNAL(selectionChanged()), &CTextBox::manager, SLOT(onSelectionChanged()));
 
-		if (CWIDGET_test_flag(THIS, WF_DESIGN))
+		if (CWIDGET_is_design(THIS))
 		{
 			get(_object, &textbox);
 			//textbox->removeEventFilter(COMBOBOX);
@@ -468,7 +480,7 @@ static void combo_set_editable(void *_object, bool ed)
 	if (hasFocus)
 		COMBOBOX->setFocus();
 
-	if (CWIDGET_test_flag(THIS, WF_DESIGN))
+	if (CWIDGET_is_design(THIS))
 		COMBOBOX->setFocusPolicy(Qt::NoFocus);
 
 	COMBOBOX->blockSignals(false);
@@ -754,7 +766,10 @@ MyComboBox::MyComboBox(QWidget *parent) :
 	QComboBox(parent)
 {
 	_sorted = _dirty = false;
+#if QT5
+#else
 	setCompleter(0);
+#endif
 	setInsertPolicy(NoInsert);
 	calcMinimumHeight();
 }
@@ -877,6 +892,7 @@ GB_DESC CTextBoxDesc[] =
 	GB_PROPERTY("Border", "b", TextBox_Border),
 	GB_PROPERTY("Password", "b", TextBox_Password),
 	GB_PROPERTY("MaxLength", "i", TextBox_MaxLength),
+	GB_PROPERTY("Placeholder", "s", TextBox_Placeholder),
 
 	GB_PROPERTY_SELF("Selection", ".TextBox.Selection"),
 	GB_METHOD("Select", NULL, TextBox_Select, "[(Start)i(Length)i]"),
@@ -922,6 +938,7 @@ GB_DESC CComboBoxDesc[] =
 	GB_PROPERTY("Password", "b", TextBox_Password),
 	GB_PROPERTY("MaxLength", "i", TextBox_MaxLength),
 	GB_PROPERTY("Border", "b", ComboBox_Border),
+	GB_PROPERTY("Placeholder", "s", TextBox_Placeholder),
 
 	GB_PROPERTY_SELF("Selection", ".TextBox.Selection"),
 	GB_METHOD("Select", NULL, TextBox_Select, "[(Start)i(Length)i]"),
