@@ -46,6 +46,7 @@
 #include "CPicture.h"
 #include "CImage.h"
 #include "CDrawingArea.h"
+#include "CContainer.h"
 #include "CColor.h"
 #include "CDraw.h"
 #include "cprinter.h"
@@ -83,6 +84,8 @@ typedef
 #define PAINTER(d) EXTRA(d)->painter
 #define PATH(d) EXTRA(d)->path
 //#define CLIP(d) EXTRA(d)->clip
+	
+static bool _internal_paint = false;
 
 static inline qreal to_deg(float angle)
 {
@@ -92,6 +95,7 @@ static inline qreal to_deg(float angle)
 static bool init_painting(GB_PAINT *d, QPaintDevice *device)
 {
 	QPen pen;
+	GB_COLOR col;
 
 	d->area.width = device->width();
 	d->area.height = device->height();
@@ -133,8 +137,14 @@ static bool init_painting(GB_PAINT *d, QPaintDevice *device)
 	pen.setMiterLimit(10.0);
 	pen.setWidthF(1.0);
 	PAINTER(d)->setPen(pen);
-	PAINTER(d)->setBrush(Qt::black);
-
+	
+	if (GB.Is(d->device, CLASS_Control))
+		col = CWIDGET_get_real_foreground((CWIDGET *)d->device);
+	else
+		col = 0;
+	
+	PAINTER(d)->setBrush(CCOLOR_make(col));
+	
 	return FALSE;
 }
 
@@ -143,7 +153,7 @@ static QColor get_color(GB_PAINT *d, GB_COLOR col)
 	if (col == GB_COLOR_DEFAULT)
 	{
 		if (GB.Is(d->device, CLASS_Control))
-			col = CWIDGET_get_real_background((CWIDGET *)d->device);
+			col = CWIDGET_get_real_foreground((CWIDGET *)d->device);
 		else
 			col = 0xFFFFFF;
 	}
@@ -237,6 +247,27 @@ static int Begin(GB_PAINT *d)
 		d->area.height = wid->height();
 		return FALSE;
 	}
+	else if (GB.Is(device, CLASS_UserControl))
+	{
+		MyContainer *wid;
+
+		wid = (MyContainer *)(((CWIDGET *)device)->widget);
+
+		if (!_internal_paint)
+		{
+			GB.Error("Cannot paint outside of Draw event handler");
+			return TRUE;
+		}
+
+		target = wid;
+
+		if (init_painting(d, target))
+			return TRUE;
+
+		d->area.width = wid->width();
+		d->area.height = wid->height();
+		return FALSE;
+	}
 	else if (GB.Is(device, CLASS_Printer))
 	{
 		CPRINTER *printer = (CPRINTER *)device;
@@ -252,7 +283,11 @@ static int Begin(GB_PAINT *d)
 		if (init_painting(d, target))
 			return TRUE;
 		// Paint.W / $hPrinter.PaperWidth * 25.4  / $hPrinter.Resolution
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+		QSizeF size = printer->printer->pageLayout().pageSize().size(QPageSize::Millimeter);
+#else		
 		QSizeF size = printer->printer->paperSize(QPrinter::Millimeter);
+#endif
 		//qDebug("d->area.width = %g / paper width = %g / resolution = %d", d->area.width, (floor((double)size.width() * 1E6) / 1E6), printer->printer->resolution());
 		d->fontScale = 25.4 * d->area.width / (floor((double)size.width() * 1E6) / 1E6) / printer->printer->resolution();
 		
@@ -365,6 +400,17 @@ static void Antialias(GB_PAINT *d, int set, int *antialias)
 		*antialias = PAINTER(d)->testRenderHint(QPainter::Antialiasing) ? 1 : 0;
 }
 
+static void set_painter_font(QPainter *p, QFont f)
+{
+	p->setFont(f);	
+	// Strange bug of QT. Sometimes the font does not apply (cf. DrawTextShadow)
+	if (f != p->font())
+	{
+		f.fromString(f.toString());
+		p->setFont(f);
+	}
+}
+
 static void apply_font(QFont &font, void *object = 0)
 {
 	GB_PAINT *d = (GB_PAINT *)DRAW.Paint.GetCurrent();
@@ -373,13 +419,7 @@ static void apply_font(QFont &font, void *object = 0)
 	if (d->fontScale != 1)
 		f.setPointSizeF(f.pointSizeF() * d->fontScale);
 
-	PAINTER(d)->setFont(f);
-	// Strange bug of QT. Sometimes the font does not apply (cf. DrawTextShadow)
-	if (f != PAINTER(d)->font())
-	{
-		f.fromString(f.toString());
-		PAINTER(d)->setFont(f);
-	}
+	set_painter_font(PAINTER(d), f);
 }
 
 static void Font(GB_PAINT *d, int set, GB_FONT *font)
@@ -390,7 +430,7 @@ static void Font(GB_PAINT *d, int set, GB_FONT *font)
 	{
 		if (*font)
 			f = QFont(*((CFONT *)(*font))->font);
-		else if ((GB.Is(d->device, CLASS_DrawingArea)))
+		else if ((GB.Is(d->device, CLASS_DrawingArea) || GB.Is(d->device, CLASS_UserControl)))
 			f = (((CWIDGET *)d->device)->widget)->font();
 		
 		apply_font(f);
@@ -919,7 +959,11 @@ static int get_text_width(QPainter *dp, QString &s)
 	int w, width = 0;
 	int i;
 
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+	text_sl = s.split('\n', Qt::KeepEmptyParts);
+#else
 	text_sl = s.split('\n', QString::KeepEmptyParts);
+#endif
 
 	text_w.resize(text_sl.count());
 
@@ -975,7 +1019,7 @@ static void draw_text(GB_PAINT *d, bool rich, const char *text, int len, float w
 		MyPaintDevice device;
 		QPainter p(&device);
 
-		p.setFont(PAINTER(d)->font());
+		set_painter_font(&p, PAINTER(d)->font());
 		p.setPen(PAINTER(d)->pen());
 		p.setBrush(PAINTER(d)->brush());
 
@@ -1004,8 +1048,10 @@ static void get_text_extents(GB_PAINT *d, bool rich, const char *text, int len,
 	QPainterPath path;
 	MyPaintDevice device;
 	QPainter p(&device);
+	QFont f = PAINTER(d)->font();
 
-	p.setFont(PAINTER(d)->font());
+	set_painter_font(&p, f);
+	
 	_draw_path = &path;
 	GetCurrentPoint(d, &_draw_x, &_draw_y);
 	_draw_y -= PAINTER(d)->fontMetrics().ascent();
@@ -1443,7 +1489,9 @@ GB_PAINT_MATRIX_DESC PAINT_MATRIX_Interface =
 
 void PAINT_begin(void *device)
 {
+	_internal_paint = true;
 	DRAW.Paint.Begin(device);
+	_internal_paint = false;
 }
 
 void PAINT_end()
@@ -1474,7 +1522,7 @@ void PAINT_clip(int x, int y, int w, int h)
 
 /*************************************************************************/
 
-MyPaintEngine::MyPaintEngine() : QPaintEngine(0) {}
+MyPaintEngine::MyPaintEngine() : QPaintEngine() {}
 MyPaintEngine::~MyPaintEngine() {}
 
 void MyPaintEngine::patchFeatures()
