@@ -52,9 +52,9 @@
 #include <QDesktopWidget>
 #include <QPaintDevice>
 #include <QLibraryInfo>
+#include <QThreadPool>
 
 #include "gb.image.h"
-#include "gb.qt.h"
 #include "gb.form.font.h"
 
 #include "CFont.h"
@@ -64,7 +64,6 @@
 #include "CWindow.h"
 #include "CButton.h"
 #include "CContainer.h"
-#include "CLabel.h"
 #include "CTextBox.h"
 #include "CTextArea.h"
 #include "CMenu.h"
@@ -74,40 +73,28 @@
 #include "CColor.h"
 #include "CConst.h"
 #include "CCheckBox.h"
-#include "CFrame.h"
 #include "CRadioButton.h"
 #include "CTabStrip.h"
 #include "CDialog.h"
 #include "CPicture.h"
 #include "CImage.h"
+#include "canimation.h"
 #include "CClipboard.h"
 #include "CDraw.h"
 #include "CWatch.h"
 #include "CDrawingArea.h"
 #include "CSlider.h"
 #include "CScrollBar.h"
-#include "CMovieBox.h"
 #include "CWatcher.h"
 #include "cprinter.h"
 #include "csvgimage.h"
 #include "cpaint_impl.h"
 #include "ctrayicon.h"
 
-#include <QX11Info>
-#ifndef QT5
-#include "CEmbedder.h"
-#endif
-
 #include "desktop.h"
-#include "x11.h"
+#include "gb.qt.platform.h"
 
-#ifdef QT5
-#include <xcb/xcb.h>
-#include <xcb/xproto.h>
-#include <QAbstractNativeEventFilter>
-#endif
-
-#include "fix_breeze.h"
+#include "fix_style.h"
 #include "main.h"
 
 /*#define DEBUG*/
@@ -116,19 +103,19 @@ extern "C" {
 const GB_INTERFACE *GB_PTR EXPORT;
 IMAGE_INTERFACE IMAGE EXPORT;
 GEOM_INTERFACE GEOM EXPORT;
+QT_PLATFORM_INTERFACE PLATFORM EXPORT;
 }
 
 int MAIN_in_wait = 0;
 int MAIN_in_message_box = 0;
 int MAIN_loop_level = 0;
 int MAIN_scale = 6;
-#ifndef NO_X_WINDOW
-int MAIN_x11_last_key_code = 0;
-#endif
 bool MAIN_debug_busy = false;
 bool MAIN_init = false;
 bool MAIN_key_debug = false;
 bool MAIN_right_to_left = false;
+const char *MAIN_platform = NULL;
+bool MAIN_platform_is_wayland = false;
 
 GB_CLASS CLASS_Control;
 GB_CLASS CLASS_Container;
@@ -145,6 +132,7 @@ GB_CLASS CLASS_Printer;
 GB_CLASS CLASS_Image;
 GB_CLASS CLASS_SvgImage;
 GB_CLASS CLASS_TextArea;
+GB_CLASS CLASS_ComboBox;
 
 static bool in_event_loop = false;
 static int _no_destroy = 0;
@@ -154,14 +142,8 @@ static GB_FUNCTION _application_keypress_func;
 static bool _check_quit_posted = false;
 static int _prevent_quit = 0;
 
-#ifndef NO_X_WINDOW
-static int (*_x11_event_filter)(XEvent *) = 0;
-#endif
-
 static QHash<void *, void *> _link_map;
 
-static QPointer<QWidget> MAIN_mouseGrabber = 0;
-static QPointer<QWidget> MAIN_keyboardGrabber = 0;
 
 static QByteArray _utf8_buffer[UTF8_NBUF];
 static int _utf8_count = 0;
@@ -169,8 +151,6 @@ static int _utf8_length = 0;
 
 static void QT_Init(void);
 
-#ifdef QT5
-
 static QtMessageHandler _previousMessageHandler;
 
 static void myMessageHandler(QtMsgType type, const QMessageLogContext &context, const QString &msg )
@@ -185,7 +165,6 @@ static void myMessageHandler(QtMsgType type, const QMessageLogContext &context,
 
 	_previousMessageHandler(type, context, msg);
 }
-#endif
 
 //static MyApplication *myApp;
 
@@ -561,7 +540,7 @@ void MyApplication::commitDataRequested(QSessionManager &session)
 	if (CWINDOW_Main)
 	{
 		cmd += "-session-desktop";
-		cmd += QString::number(X11_window_get_desktop(CWINDOW_Main->widget.widget->winId()));
+		cmd += QString::number(PLATFORM.Window.GetVirtualDesktop(CWINDOW_Main->widget.widget));
 		/*cmd += "-session-data";
 		cmd += QString::number(CWINDOW_Main->x) + ","
 		       + QString::number(CWINDOW_Main->y) + ","
@@ -573,206 +552,6 @@ void MyApplication::commitDataRequested(QSessionManager &session)
 	session.setRestartCommand(cmd);
 }
 
-//---------------------------------------------------------------------------
-
-static void x11_set_event_filter(int (*filter)(XEvent *))
-{
-	_x11_event_filter = filter;
-}
-
-#ifdef QT5
-
-class MyNativeEventFilter: public QAbstractNativeEventFilter
-{
-public:
-
-	static MyNativeEventFilter manager;
-
-	virtual bool nativeEventFilter(const QByteArray &eventType, void *message, long *)
-	{
-		xcb_generic_event_t *ev = static_cast<xcb_generic_event_t *>(message);
-		int type = ev->response_type & ~0x80;
-
-		switch(type)
-		{
-			case XCB_KEY_PRESS:
-			case XCB_KEY_RELEASE:
-				MAIN_x11_last_key_code = ((xcb_key_press_event_t *)ev)->detail;
-				break;
-		}
-
-		if (_x11_event_filter)
-		{
-			XEvent xev;
-
-			CLEAR(&xev);
-			xev.xany.type = type;
-			xev.xany.display = QX11Info::display();
-			xev.xany.send_event = ev->response_type & 0x80 ? 1 : 0;
-
-			switch (type)
-			{
-				//case XCB_KEY_PRESS:
-				//case XCB_KEY_RELEASE:
-				case XCB_EXPOSE:
-				{
-					xcb_expose_event_t *e = (xcb_expose_event_t *)ev;
-					xev.xexpose.window = e->window;
-					xev.xexpose.x = e->x;
-					xev.xexpose.y = e->y;
-					xev.xexpose.width = e->width;
-					xev.xexpose.height = e->height;
-					xev.xexpose.count = e->count;
-					break;
-				}
-
-				case XCB_VISIBILITY_NOTIFY:
-				{
-					xcb_visibility_notify_event_t *e = (xcb_visibility_notify_event_t *)ev;
-					xev.xvisibility.window = e->window;
-					xev.xvisibility.state = e->state;
-					break;
-				}
-
-				case XCB_DESTROY_NOTIFY:
-				{
-					xcb_destroy_notify_event_t *e = (xcb_destroy_notify_event_t *)ev;
-					xev.xdestroywindow.event = e->event;
-					xev.xdestroywindow.window = e->window;
-					break;
-				}
-
-				case XCB_MAP_NOTIFY:
-				{
-					xcb_map_notify_event_t *e = (xcb_map_notify_event_t *)ev;
-					xev.xmap.event = e->event;
-					xev.xmap.window = e->window;
-					xev.xmap.override_redirect = e->override_redirect;
-					break;
-				}
-
-				case XCB_UNMAP_NOTIFY:
-				{
-					xcb_unmap_notify_event_t *e = (xcb_unmap_notify_event_t *)ev;
-					xev.xunmap.event = e->event;
-					xev.xunmap.window = e->window;
-					xev.xunmap.from_configure = e->from_configure;
-					break;
-				}
-
-				case XCB_REPARENT_NOTIFY:
-				{
-					xcb_reparent_notify_event_t *e = (xcb_reparent_notify_event_t *)ev;
-					xev.xreparent.event = e->event;
-					xev.xreparent.window = e->window;
-					xev.xreparent.parent = e->parent;
-					xev.xreparent.x = e->x;
-					xev.xreparent.y = e->y;
-					xev.xreparent.override_redirect = e->override_redirect;
-					break;
-				}
-
-				case XCB_CONFIGURE_NOTIFY:
-				{
-					xcb_configure_notify_event_t *e = (xcb_configure_notify_event_t *)ev;
-					xev.xconfigure.event = e->event;
-					xev.xconfigure.window = e->window;
-					xev.xconfigure.x = e->x;
-					xev.xconfigure.y = e->y;
-					xev.xconfigure.width = e->width;
-					xev.xconfigure.height = e->height;
-					xev.xconfigure.border_width = e->border_width;
-					xev.xconfigure.override_redirect = e->override_redirect;
-					break;
-				}
-				
-				case XCB_PROPERTY_NOTIFY:
-				{
-					xcb_property_notify_event_t *e = (xcb_property_notify_event_t *)ev;
-					xev.xproperty.window = e->window;
-					xev.xproperty.atom = e->atom;
-					xev.xproperty.time = e->time;
-					xev.xproperty.state = e->state;
-					break;
-				}
-
-				case XCB_SELECTION_CLEAR:
-				{
-					xcb_selection_clear_event_t *e = (xcb_selection_clear_event_t *)ev;
-					xev.xselectionclear.window = e->owner;
-					xev.xselectionclear.selection = e->selection;
-					xev.xselectionclear.time = e->time;
-					break;
-				}
-
-				case XCB_SELECTION_REQUEST:
-				{
-					xcb_selection_request_event_t *e = (xcb_selection_request_event_t *)ev;
-					xev.xselectionrequest.owner = e->owner;
-					xev.xselectionrequest.requestor = e->requestor;
-					xev.xselectionrequest.selection = e->selection;
-					xev.xselectionrequest.target = e->target;
-					xev.xselectionrequest.property = e->property;
-					xev.xselectionrequest.time = e->time;
-					break;
-				}
-
-				case XCB_SELECTION_NOTIFY:
-				{
-					xcb_selection_notify_event_t *e = (xcb_selection_notify_event_t *)ev;
-					xev.xselection.requestor = e->requestor;
-					xev.xselection.selection = e->selection;
-					xev.xselection.target = e->target;
-					xev.xselection.property = e->property;
-					xev.xselection.time = e->time;
-					break;
-				}
-
-				case XCB_CLIENT_MESSAGE:
-				{
-					xcb_client_message_event_t *e = (xcb_client_message_event_t *)ev;
-					xev.xclient.window = e->window;
-					xev.xclient.message_type = e->type;
-					xev.xclient.format = e->format;
-					xev.xclient.data.l[0] = e->data.data32[0];
-					xev.xclient.data.l[1] = e->data.data32[1];
-					xev.xclient.data.l[2] = e->data.data32[2];
-					xev.xclient.data.l[3] = e->data.data32[3];
-					xev.xclient.data.l[4] = e->data.data32[4];
-					break;
-				}
-
-				default:
-					qDebug("gb.qt5: warning: unhandled xcb event: %d", type);
-					return false;
-			}
-
-			return (*_x11_event_filter)(&xev) != 0;
-		}
-
-		return false;
-	}
-};
-
-MyNativeEventFilter MyNativeEventFilter::manager;
-
-#else
-
-bool MyApplication::x11EventFilter(XEvent *e)
-{
-	// Workaround for input methods that void the key code of KeyRelease eventFilter
-	if (e->type == XKeyPress)
-		MAIN_x11_last_key_code = e->xkey.keycode;
-	else if (e->type == XKeyRelease)
-		MAIN_x11_last_key_code = e->xkey.keycode;
-
-	if (_x11_event_filter)
-		return (*_x11_event_filter)(e);
-
-	return false;
-}
-
-#endif
 
 //---------------------------------------------------------------------------
 
@@ -795,49 +574,6 @@ void MyTimer::timerEvent(QTimerEvent *e)
 
 //---------------------------------------------------------------------------
 
-static void release_grab()
-{
-	MAIN_mouseGrabber = QWidget::mouseGrabber();
-	MAIN_keyboardGrabber = QWidget::keyboardGrabber();
-
-	if (MAIN_mouseGrabber)
-	{
-		//qDebug("releaseMouse");
-		MAIN_mouseGrabber->releaseMouse();
-	}
-	if (MAIN_keyboardGrabber)
-	{
-		//qDebug("releaseKeyboard");
-		MAIN_keyboardGrabber->releaseKeyboard();
-	}
-
-	#ifndef NO_X_WINDOW
-	if (qApp->activePopupWidget())
-	{
-		XUngrabPointer(QX11Info::display(), CurrentTime);
-		XFlush(QX11Info::display());
-	}
-	#endif
-}
-
-
-static void unrelease_grab()
-{
-	if (MAIN_mouseGrabber)
-	{
-		//qDebug("grabMouse");
-		MAIN_mouseGrabber->grabMouse();
-		MAIN_mouseGrabber = 0;
-	}
-
-	if (MAIN_keyboardGrabber)
-	{
-		//qDebug("grabKeyboard");
-		MAIN_keyboardGrabber->grabKeyboard();
-		MAIN_keyboardGrabber = 0;
-	}
-}
-
 static bool must_quit(void)
 {
 	#if DEBUG_WINDOW
@@ -862,9 +598,6 @@ static void check_quit_now(intptr_t param)
 					GB.Call(&func, 0, FALSE);
 			}
 
-#ifndef QT5
-			qApp->syncX();
-#endif
 			qApp->exit();
 			exit_called = true;
 		}
@@ -884,7 +617,7 @@ void MAIN_check_quit(void)
 
 void MAIN_update_scale(const QFont &font)
 {
-	MAIN_scale = GET_DESKTOP_SCALE(font.pointSize(), QX11Info::appDpiY());
+	MAIN_scale = GET_DESKTOP_SCALE(font.pointSize(), PLATFORM.Desktop.GetResolutionY());
 }
 
 static void QT_InitEventLoop(void)
@@ -984,14 +717,45 @@ static void *_old_hook_main;
 
 static void hook_main(int *argc, char ***argv)
 {
+	QString platform;
+	const char *comp;
 	char *env;
-
-	env = getenv("GB_X11_INIT_THREADS");
-	if (env && atoi(env))
-		XInitThreads();
+	
+	env = getenv("GB_GUI_PLATFORM");
+	if (env && *env)
+	{
+		if (!strcasecmp(env, "X11"))
+			putenv((char *)"QT_QPA_PLATFORM=xcb");
+		else if (!strcasecmp(env, "WAYLAND"))
+			putenv((char *)"QT_QPA_PLATFORM=wayland");
+		else
+			fprintf(stderr, QT_NAME ": warning: unknown platform: %s\n", env);
+	}
 
 	new MyApplication(*argc, *argv);
+	
+	platform = qApp->platformName();
 
+	if (platform == "wayland")
+	{
+		comp = "gb.qt5.wayland";
+		MAIN_platform = "wayland";
+		MAIN_platform_is_wayland = true;
+	}
+	else if (platform == "xcb")
+	{
+		comp = "gb.qt5.x11";
+		MAIN_platform = "x11";
+	}
+	else
+	{
+		fprintf(stderr, QT_NAME ": error: unsupported platform: %s\n", TO_UTF8(qApp->platformName()));
+		::abort();
+	}
+	
+	GB.Component.Load(comp);
+	GB.GetInterface(comp, QT_PLATFORM_INTERFACE_VERSION, &PLATFORM);
+	
 	QT_Init();
 	init_lang(GB.System.Language(), GB.System.IsRightToLeft());
 
@@ -1009,6 +773,7 @@ static void hook_quit()
 
 	CWINDOW_close_all(true);
 	CWINDOW_delete_all(true);
+	CMOUSE_set_control(NULL);
 
 	qApp->sendPostedEvents(); //processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::DeferredDeletion, 0);
 	qApp->sendPostedEvents(0, QEvent::DeferredDelete);
@@ -1038,6 +803,8 @@ static void hook_loop()
 
 static void hook_wait(int duration)
 {
+	static bool _warning = FALSE;
+	
 	if (MyDrawingArea::inAnyDrawEvent())
 	{
 		GB.Error("Wait is forbidden during a repaint event");
@@ -1049,7 +816,13 @@ static void hook_wait(int duration)
 	if (duration > 0)
 	{
 		if (CKEY_is_valid())
-			fprintf(stderr, QT_NAME ": warning: calling the event loop during a keyboard event handler is ignored\n");
+		{
+			if (!_warning)
+			{
+				fprintf(stderr, QT_NAME ": warning: calling the event loop during a keyboard event handler is ignored\n");
+				_warning = TRUE;
+			}
+		}
 		else
 			qApp->processEvents(QEventLoop::AllEvents, duration);
 	}
@@ -1122,11 +895,11 @@ static bool hook_error(int code, char *error, char *where, bool in_event_loop)
 		msg = msg.arg(TO_QSTRING(error)).arg(where);
 	}
 
-	release_grab();
+	PLATFORM.ReleaseGrab();
 	MAIN_in_message_box++;
 	ret = QMessageBox::critical(0, TO_QSTRING(GB.Application.Name()), msg, in_event_loop ? QMessageBox::Close | QMessageBox::Ignore : QMessageBox::Ok);
 	MAIN_in_message_box--;
-	unrelease_grab();
+	PLATFORM.UnreleaseGrab();
 	MAIN_check_quit();
 
 	return ret == QMessageBox::Ignore;
@@ -1137,23 +910,27 @@ static void QT_Init(void)
 	static bool init = false;
 	QFont f;
 	char *env;
+	bool fix_style;
 
 	if (init)
 		return;
 
 	//qApp->setAttribute(Qt::AA_ImmediateWidgetCreation);
 
-	X11_init(QX11Info::display(), QX11Info::appRootWindow());
+	PLATFORM.Init();
 
-#ifdef QT5
 	_previousMessageHandler = qInstallMessageHandler(myMessageHandler);
-#endif
 	
 	/*QX11Info::setAppDpiX(0, 92);
 	QX11Info::setAppDpiY(0, 92);*/
 
 	/*fcntl(ConnectionNumber(qt_xdisplay()), F_SETFD, FD_CLOEXEC);*/
-
+	#if QT_VERSION >= QT_VERSION_CHECK(5, 7, 0)
+	qApp->setDesktopFileName(TO_QSTRING(GB.Application.Name()));
+	#endif
+	
+	fix_style = false;
+	
 	if (::strcmp(qApp->style()->metaObject()->className(), "Breeze::Style") == 0)
 	{
 		env = getenv("GB_QT_NO_BREEZE_FIX");
@@ -1161,6 +938,7 @@ static void QT_Init(void)
 		{
 			CSTYLE_fix_breeze = TRUE;
 			qApp->setStyle(new FixBreezeStyle);
+		fix_style = true;
 		}
 	}
 	else if (::strcmp(qApp->style()->metaObject()->className(), "Oxygen::Style") == 0)
@@ -1170,15 +948,16 @@ static void QT_Init(void)
 		{
 			CSTYLE_fix_oxygen = TRUE;
 			qApp->setStyle(new FixBreezeStyle);
+			fix_style = true;
 		}
 	}
+	
+	if (!fix_style)
+		qApp->setStyle(new FixStyle);
 
 	MAIN_update_scale(qApp->desktop()->font());
 
 	qApp->installEventFilter(&CWidget::manager);
-#ifdef QT5
-	qApp->installNativeEventFilter(&MyNativeEventFilter::manager);
-#endif
 
 	MyApplication::setEventFilter(true);
 
@@ -1325,13 +1104,18 @@ static void declare_tray_icon()
 	GB.Component.Declare(TrayIconsDesc);
 }
 
+static int QT_GetDesktopScale(void)
+{
+	return MAIN_scale;
+}
+
 extern "C" {
 
 GB_DESC *GB_CLASSES[] EXPORT =
 {
-	CBorderDesc, CColorDesc,
-	CAlignDesc, CArrangeDesc, CScrollDesc, CKeyDesc, CSelectDesc,
-	CImageDesc, CPictureDesc,
+	BorderDesc, CColorDesc,
+	AlignDesc, ArrangeDesc, ScrollDesc, CKeyDesc, SelectDesc,
+	CImageDesc, CPictureDesc, AnimationDesc,
 	CFontDesc, CFontsDesc,
 	CMouseDesc, CCursorDesc, CPointerDesc,
 	CClipboardDesc, CDragDesc,
@@ -1340,31 +1124,23 @@ GB_DESC *GB_CLASSES[] EXPORT =
 	CControlDesc, ContainerChildrenDesc, ContainerDesc,
 	UserControlDesc, UserContainerDesc,
 	CMenuChildrenDesc, CMenuDesc,
-	CLabelDesc, CTextLabelDesc, CSeparatorDesc,
 	CButtonDesc, CToggleButtonDesc, CToolButtonDesc,
 	CCheckBoxDesc, CRadioButtonDesc,
 	CTextBoxSelectionDesc, CTextBoxDesc, CComboBoxItemDesc, CComboBoxDesc,
 	CTextAreaSelectionDesc, CTextAreaDesc,
-	CFrameDesc, CPanelDesc, CHBoxDesc, CVBoxDesc, CHPanelDesc, CVPanelDesc,
+	CPanelDesc, CHBoxDesc, CVBoxDesc, CHPanelDesc, CVPanelDesc,
 	CTabStripContainerChildrenDesc, CTabStripContainerDesc, CTabStripDesc,
 	CDrawingAreaDesc,
-	CSliderDesc, CMovieBoxDesc, CScrollBarDesc,
+	SliderDesc, ScrollBarDesc,
 	CWindowMenusDesc, CWindowControlsDesc, CWindowDesc, CWindowsDesc, CFormDesc,
 	CDialogDesc,
-#ifndef QT5
-	CEmbedderDesc,
-#endif
 	CWatcherDesc,
 	PrinterDesc,
 	SvgImageDesc,
 	NULL
 };
 
-#ifdef QT5
 void *GB_QT5_1[] EXPORT =
-#else
-void *GB_QT4_1[] EXPORT =
-#endif
 {
 	(void *)1,
 
@@ -1396,6 +1172,7 @@ void *GB_QT4_1[] EXPORT =
 	(void *)CWIDGET_get_background,
 	(void *)Control_Mouse,
 	(void *)CWIDGET_after_set_color,
+	(void *)QT_GetDesktopScale,
 	NULL
 };
 
@@ -1414,7 +1191,7 @@ int EXPORT GB_INIT(void)
 	env = getenv("GB_GUI_BUSY");
 	if (env && atoi(env))
 		MAIN_debug_busy = true;
-
+	
 	//putenv((char *)"QT_SLOW_TOPLEVEL_RESIZE=1");
 
 	_old_hook_main = GB.Hook(GB_HOOK_MAIN, (void *)hook_main);
@@ -1427,10 +1204,10 @@ int EXPORT GB_INIT(void)
 	GB.Component.Load("gb.gui.base");
 	
 	GB.GetInterface("gb.geom", GEOM_INTERFACE_VERSION, &GEOM);
+
 	GB.GetInterface("gb.image", IMAGE_INTERFACE_VERSION, &IMAGE);
-	
-  IMAGE.SetDefaultFormat(GB_IMAGE_BGRP);
-	
+	IMAGE.SetDefaultFormat(GB_IMAGE_BGRP);
+
 	DRAW_init();
 
 	CLASS_Control = GB.FindClass("Control");
@@ -1448,6 +1225,7 @@ int EXPORT GB_INIT(void)
 	CLASS_Image = GB.FindClass("Image");
 	CLASS_SvgImage = GB.FindClass("SvgImage");
 	CLASS_TextArea = GB.FindClass("TextArea");
+	CLASS_ComboBox = GB.FindClass("ComboBox");
 
 	QT_InitEventLoop();
 
@@ -1460,29 +1238,19 @@ int EXPORT GB_INIT(void)
 
 void EXPORT GB_EXIT()
 {
-	#ifndef NO_X_WINDOW
-		X11_exit();
-	#endif
-	//qApp->setStyle("windows");
-	delete qApp;
+	if (qApp)
+	{
+		PLATFORM.Exit();
+		delete qApp;
+	}
 }
 
 #ifndef NO_X_WINDOW
 int EXPORT GB_INFO(const char *key, void **value)
 {
-	if (!strcasecmp(key, "DISPLAY"))
+	if (!strcasecmp(key, "DECLARE_TRAYICON"))
 	{
-		*value = (void *)QX11Info::display();
-		return TRUE;
-	}
-	else if (!strcasecmp(key, "ROOT_WINDOW"))
-	{
-		*value = (void *)QX11Info::appRootWindow();
-		return TRUE;
-	}
-	else if (!strcasecmp(key, "SET_EVENT_FILTER"))
-	{
-		*value = (void *)x11_set_event_filter;
+		*value = (void *)declare_tray_icon;
 		return TRUE;
 	}
 	else if (!strcasecmp(key, "GET_HANDLE"))
@@ -1490,21 +1258,18 @@ int EXPORT GB_INFO(const char *key, void **value)
 		*value = (void *)CWIDGET_get_handle;
 		return TRUE;
 	}
-	else if (!strcasecmp(key, "TIME"))
-	{
-		*value = (void *)QX11Info::appTime();
-		return TRUE;
-	}
-	else if (!strcasecmp(key, "DECLARE_TRAYICON"))
-	{
-		*value = (void *)declare_tray_icon;
-		return TRUE;
-	}
 	else
 		return FALSE;
 }
 #endif
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
+void EXPORT GB_FORK(void)
+{
+	delete QThreadPool::globalInstance();
+}
+#endif
+
 /*#ifndef NO_X_WINDOW
 extern Time	qt_x_time;
 #endif*/
@@ -1540,20 +1305,17 @@ void EXPORT GB_SIGNAL(int signal, void *param)
 	switch(signal)
 	{
 		case GB_SIGNAL_DEBUG_BREAK:
-			release_grab();
+			PLATFORM.ReleaseGrab();
 			break;
 
 		case GB_SIGNAL_DEBUG_FORWARD:
 			//while (qApp->activePopupWidget())
 			//	delete qApp->activePopupWidget();
-#ifndef QT5
-			qApp->syncX();
-#endif
 			break;
 
 		case GB_SIGNAL_DEBUG_CONTINUE:
 			GB.Post((GB_CALLBACK)activate_main_window, 0);
-			unrelease_grab();
+			PLATFORM.UnreleaseGrab();
 			break;
 	}
 }
