@@ -228,6 +228,9 @@ void CLASS_check_unused_global(CLASS *class)
 		sym = CLASS_get_symbol(class, i);
 		type = sym->global.type;
 		
+		if (sym->global_no_warning)
+			continue;
+		
 		if (sym->global_used && sym->global_assigned)
 			continue;
 		
@@ -283,6 +286,7 @@ int CLASS_add_function(CLASS *class, TRANS_FUNC *decl)
 		sym = CLASS_declare(class, decl->index, TK_FUNCTION, TRUE);
 		sym->global.type = decl->type;
 		sym->global.value = count;
+		sym->global_no_warning = decl->no_warning;
 	}
 
 	if (TYPE_is_static(decl->type))
@@ -311,7 +315,7 @@ int CLASS_add_function(CLASS *class, TRANS_FUNC *decl)
 	func->vararg = decl->vararg;
 	func->fast = decl->fast;
 	func->unsafe = decl->unsafe;
-
+	
 	// Function startup
 
 	CODE_begin_function(func);
@@ -527,16 +531,16 @@ static int add_class(CLASS *class, int index, bool used, bool exported)
 
 		sym->class = num + 1;
 
-		if (JOB->verbose)
-			printf("Adding class %.*s %s%s\n", sym->symbol.len, sym->symbol.name, used ? "" : "Unused ", exported ? "Exported" : "");
+		if (COMP_verbose)
+			fprintf(stderr, "Adding class %.*s %s%s\n", sym->symbol.len, sym->symbol.name, used ? "" : "Unused ", exported ? "Exported" : "");
 		
 		JOB->class->class[num].exported = exported;
 	}
 
 	if (used != JOB->class->class[num].used)
 	{
-		if (JOB->verbose)
-			printf("Switching class %.*s to %s\n", sym->symbol.len, sym->symbol.name, used ? "Used" : "Unused");
+		if (COMP_verbose)
+			fprintf(stderr, "Switching class %.*s to %s\n", sym->symbol.len, sym->symbol.name, used ? "Used" : "Unused");
 		
 		JOB->class->class[num].used = used;
 	}
@@ -729,10 +733,6 @@ void CLASS_add_declaration(CLASS *class, TRANS_DECL *decl)
 
 		var->type = decl->type;
 		var->index = decl->index;
-		//var->pos = class->size_stat;
-		//var->size = TYPE_sizeof(var->type);
-
-		//class->size_stat += var->size;
 
 		CLASS_begin_init_function(class, FUNC_INIT_STATIC);
 
@@ -756,11 +756,7 @@ void CLASS_add_declaration(CLASS *class, TRANS_DECL *decl)
 
 		var->type = decl->type;
 		var->index = decl->index;
-		//var->pos = class->size_dyn;
-		//var->size = TYPE_sizeof(var->type);
 
-		//class->size_dyn += var->size;
-		
 		CLASS_begin_init_function(class, FUNC_INIT_DYNAMIC);
 
 		if (TRANS_has_init_var(decl))
@@ -773,52 +769,6 @@ void CLASS_add_declaration(CLASS *class, TRANS_DECL *decl)
 	}
 }
 
-#if 0
-static void reorder_decl(CLASS *class, VARIABLE *tvar, const char *desc)
-{
-	int count;
-	int pos;
-	VARIABLE *var;
-	int i, j;
-	int n;
-
-	/* variables statiques */
-
-	count = ARRAY_count(tvar);
-	if (count > 1)
-	{
-		if (JOB->verbose)
-			printf("Reordering %s variables:", desc);
-
-		pos = 0;
-		for (i = 0; i < 3; i++)
-		{
-			for (j = 0; j < count; j++)
-			{
-				var = &tvar[j];
-				n = var->size & 3;
-
-				switch (i)
-				{
-					case 0: if (n != 0) continue; else break;
-					case 1: if (n != 2) continue; else break;
-					case 2: if (n != 1 && n != 3) continue; else break;
-				}
-
-				var->pos = pos;
-				pos += var->size;
-
-				if (JOB->verbose)
-					printf(" %s (%d) ", TABLE_get_symbol_name(class->table, var->index), var->pos);
-			}
-		}
-
-		if (JOB->verbose)
-			printf("\n");
-	}
-}
-#endif
-
 // Don't do that! The order of variables must be kept.
 
 void CLASS_sort_declaration(CLASS *class)
