@@ -347,16 +347,6 @@ static void export_signature(int nparam, int npmin, PARAM *param, bool vararg)
 }
 
 
-static void create_file(FILE **fw, const char *file)
-{
-	if (!*fw)
-	{
-		*fw = fopen(file, "w");
-		if (!*fw)
-			THROW("Cannot create file: &1", FILE_cat(FILE_get_dir(COMP_project), file, NULL));
-	}
-}
-
 static void close_file_and_rename(FILE *f, const char *file, const char *dest)
 {
 	if (f)
@@ -472,14 +462,14 @@ static void class_update_exported(CLASS *class)
 
 		if (cmp == 0)
 		{
-			if (JOB->verbose)
+			if (COMP_verbose)
 				printf("Remove '%s' from .list file\n", name);
 			continue;
 		}
 		else if ((cmp > 0) && class->exported && !inserted)
 		{
-			create_file(&fw, ".list#");
-			if (JOB->verbose)
+			COMPILE_create_file(&fw, ".list#");
+			if (COMP_verbose)
 				printf("Insert '%s%s' into .list file\n", class->name, class->optional ? "?" : "");
 			fputs(class->name, fw);
 			if (class->has_static && COMPILE_version >= 0x03060090)
@@ -495,10 +485,10 @@ static void class_update_exported(CLASS *class)
 
 		if (exist_bytecode_file(name))
 		{
-			if (JOB->verbose)
+			if (COMP_verbose)
 				printf("Copy '%s' in .list file\n", name);
 
-			create_file(&fw, ".list#");
+			COMPILE_create_file(&fw, ".list#");
 			fputs(name, fw);
 			if (has_static && COMPILE_version >= 0x03060090)
 				fputc('!', fw);
@@ -508,7 +498,7 @@ static void class_update_exported(CLASS *class)
 		}
 		else
 		{
-			if (JOB->verbose)
+			if (COMP_verbose)
 				printf("Remove '%s' from .list file\n", name);
 		}
 	}
@@ -534,7 +524,7 @@ static void insert_class_info(CLASS *class, FILE *fw)
 	const char *str;
 	int len;
 
-	if (JOB->verbose)
+	if (COMP_verbose)
 		printf("Insert '%s' information into .info file\n", class->name);
 
 	_finfo = fw;
@@ -783,7 +773,7 @@ void CLASS_export(void)
 
 		if (cmp == 0)
 		{
-			if (JOB->verbose)
+			if (COMP_verbose)
 				printf("Remove '%s' information from .info file\n", class->name);
 
 			for(;;)
@@ -798,7 +788,7 @@ void CLASS_export(void)
 
 		if (cmp > 0 && class->exported && !inserted)
 		{
-			create_file(&fw, ".info#");
+			COMPILE_create_file(&fw, ".info#");
 			insert_class_info(class, fw);
 			inserted = TRUE;
 		}
@@ -810,11 +800,11 @@ void CLASS_export(void)
 
 		if (exist_bytecode_file(&line[1]))
 		{
-			if (JOB->verbose)
+			if (COMP_verbose)
 				printf("Copy '%s' information in .info file\n", &line[1]);
 			for(;;)
 			{
-				create_file(&fw, ".info#");
+				COMPILE_create_file(&fw, ".info#");
 				fputs(line, fw);
 				fputc('\n', fw);
 				read_line(&line, &len);
@@ -824,7 +814,7 @@ void CLASS_export(void)
 		}
 		else
 		{
-			if (JOB->verbose)
+			if (COMP_verbose)
 				printf("Remove '%s' information from .info file\n", &line[1]);
 			for(;;)
 			{
