@@ -1,14 +1,12 @@
 /***************************************************************************
 
-  header.h
+  gbc_header.h
 
-  Analyzing class description
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -37,20 +36,23 @@ enum {
   HF_NO_3PTS = 1,
   HF_EVENT = 2,
   HF_NO_OPT = 4,
-  HF_VOID = 8
+  HF_VOID = 8,
+	HF_NO_BYREF = 16
   };
 
 enum {
-  HS_ERROR = 0,
-  HS_PUBLIC = 1,
-  HS_STATIC = 2,
-  HS_DYNAMIC = 4,
-  HS_PROCEDURE = 8,
-  HS_FUNCTION = 16,
-  HS_PUT = 32,
-  HS_UNKNOWN = 64,
-  HS_NOPARAM = 128,
-  HS_COMPARE = 256,
+  HS_ERROR 			= 0,
+  HS_PUBLIC 		= 1 << 0,
+  HS_STATIC 		= 1 << 1,
+  HS_DYNAMIC 		= 1 << 2,
+  HS_PROCEDURE 	= 1 << 3,
+  HS_FUNCTION 	= 1 << 4,
+  HS_PUT 				= 1 << 5,
+  HS_UNKNOWN 		= 1 << 6,
+  HS_PROPERTY		= 1 << 7,
+  HS_NOPARAM 		= 1 << 8,
+  HS_COMPARE 		= 1 << 9,
+	HS_ATTACH 		= 1 << 10
   };
 
 typedef
@@ -61,7 +63,7 @@ typedef
   HEADER_SPECIAL;
 
 
-PUBLIC void HEADER_do(void);
+void HEADER_do(void);
 
 #endif
 
