@@ -46,8 +46,11 @@
 	JOB->nobreak = nobreak; \
 }
 
-
+#if defined(__GNUC__) && __GNUC__ == 8
+static volatile int ctrl_level; // Workaround a gcc compiler optimization bug
+#else
 static int ctrl_level;
+#endif
 static int ctrl_id;
 static int ctrl_local;
 
@@ -695,6 +698,8 @@ void TRANS_on_goto_gosub(void)
 		
 		if (n == 127)
 			THROW("Too many labels");
+
+		TRANS_newline();
 	}
 	
 	pos = CODE_set_current_pos(pos);
