@@ -525,8 +525,6 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 	// for virtual terminal
 	int fd_master = -1;
 	char *slave = NULL;
-	//struct termios termios_stdin;
-	//struct termios termios_check;
 	struct termios termios_master;
 	const char *exec;
 
@@ -610,17 +608,14 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 		fprintf(stderr, "run_process: slave = %s\n", slave);
 		#endif
 		
-		if (mode & PM_TERM)
-		{
-			if (tcgetattr(fd_master, &termios_master))
-				goto __ABORT_ERRNO;
+		if (tcgetattr(fd_master, &termios_master))
+			goto __ABORT_ERRNO;
 
-			cfmakeraw(&termios_master);
-			//termios_master.c_lflag &= ~ECHO;
+		cfmakeraw(&termios_master);
+		//termios_master.c_lflag &= ~ECHO;
 
-			if (tcsetattr(fd_master, TCSANOW, &termios_master))
-				goto __ABORT_ERRNO;
-		}
+		if (tcsetattr(fd_master, TCSANOW, &termios_master))
+			goto __ABORT_ERRNO;
 	}
 	else
 	{
@@ -724,13 +719,8 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 		bool pwd;
 		int ch_i, ch_n;
 
-		//bool stdin_isatty = isatty(STDIN_FILENO);
-
 		sigprocmask(SIG_SETMASK, &old, NULL);
 
-		if (mode & PM_SHELL)
-			setpgid(0, 0);
-
 		if (mode & PM_TERM)
 		{
 			close(fd_master);
@@ -797,6 +787,9 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 			}
 		}
 
+		if (mode & PM_SHELL)
+			setpgid(0, 0);
+
 		pwd = FALSE;
 
 		if (env)
@@ -912,16 +905,18 @@ static void callback_child(int signum, intptr_t data)
 
 static void init_child(void)
 {
-	if (_init)
-		return;
+	if (!_init)
+	{
+		#ifdef DEBUG_ME
+		fprintf(stderr, "init_child()\n");
+		#endif
 
-	#ifdef DEBUG_ME
-	fprintf(stderr, "init_child()\n");
-	#endif
+		_SIGCHLD_callback = SIGNAL_register(SIGCHLD, callback_child, 0);
 
-	_SIGCHLD_callback = SIGNAL_register(SIGCHLD, callback_child, 0);
-
-	_init = TRUE;
+		_init = TRUE;
+	}
+	
+	SIGNAL_check(SIGCHLD);
 }
 
 static void exit_child(void)
@@ -983,15 +978,15 @@ void CPROCESS_wait_for(CPROCESS *process, int timeout)
 
 	OBJECT_REF(process);
 
-	sigfd = SIGNAL_get_fd();
-
 	ON_ERROR_1(error_CPROCESS_create, process)
 	{
 		while (process->running)
 		{
+			sigfd = SIGNAL_get_fd();
 			#ifdef DEBUG_ME
-			fprintf(stderr, "Watch process %d\n", process->pid);
+			fprintf(stderr, "Watch process %d (end = %d)\n", process->pid, sigfd);
 			#endif
+			SIGNAL_check(SIGCHLD);
 			ret = WATCH_process(sigfd, process->out, process->err, timeout);
 			#ifdef DEBUG_ME
 			fprintf(stderr, "Watch process %d ->%s%s%s%s\n", process->pid, ret & WP_END ? " END" : "", ret & WP_OUTPUT ? " OUTPUT" : "", ret & WP_ERROR ? " ERROR" : "", ret & WP_TIMEOUT ? " TIMEOUT" : "");
@@ -1229,42 +1224,6 @@ BEGIN_METHOD_VOID(Process_CloseInput)
 
 END_METHOD
 
-/*
-static int calc_mode(int arg)
-{
-	int mode = 0;
-	
-	if (arg & R_OK) mode += PM_READ;
-	if (arg & W_OK) mode += PM_WRITE;
-	if (arg & X_OK) mode += PM_SHELL;
-	
-	return mode;
-}
-
-BEGIN_METHOD(Process_Exec, GB_OBJECT command; GB_INTEGER mode; GB_OBJECT env)
-
-	CARRAY *command = (CARRAY *)VARG(command);
-	CARRAY *env = (CARRAY *)VARG(env);
-	
-	if (GB_CheckObject(command))
-		return;
-
-	init_process(THIS);
-	run_process(THIS, calc_mode(VARGOPT(mode, 0)), command, env);
-
-END_METHOD
-
-BEGIN_METHOD(Process_Shell, GB_STRING command; GB_INTEGER mode; GB_OBJECT env)
-
-	char *command = GB_ToZeroString(ARG(command));
-	CARRAY *env = (CARRAY *)VARG(env);
-	
-	init_process(THIS);
-	run_process(THIS, calc_mode(VARGOPT(mode, 0)) | PM_SHELL, command, env);
-
-END_METHOD
-*/
-
 #endif
 
 GB_DESC NATIVE_Process[] =
