@@ -178,6 +178,9 @@ int COMMON_get_unicode_char()
 	int lc;
 	uint uc;
 	
+	if (COMMON_pos > COMMON_len)
+		return -1;
+	
 	lc = STRING_utf8_get_char_length(*COMMON_get_current());
 	if (COMMON_pos + lc > COMMON_len)
 		return -1;
@@ -396,7 +399,7 @@ BEGIN_METHOD(String_Index, GB_STRING str; GB_INTEGER pos)
 END_METHOD
 
 
-static void String_Mid(ushort code)
+static void String_Mid(ushort code, VALUE *sp)
 {
 	char *str;
 	char *ref;
@@ -415,7 +418,7 @@ static void String_Mid(ushort code)
 		THROW(E_ARG);
 
 	if (null)
-		goto _SUBR_MID_FIN;
+		return;
 	
 	ref = PARAM->_string.addr;
 	str = ref + PARAM->_string.start;
@@ -425,7 +428,7 @@ static void String_Mid(ushort code)
 	if (ulen >= len)
 	{
 		VOID_STRING(PARAM);
-		goto _SUBR_MID_FIN;
+		return;
 	}
 	
 	PARAM->_string.start += ulen;
@@ -463,15 +466,10 @@ static void String_Mid(ushort code)
 	}
 	else
 		PARAM->_string.len = ulen;
-
-_SUBR_MID_FIN:
-
-	SP -= NPARAM;
-	SP++;
 }
 
 
-static void String_Left(ushort code)
+static void String_Left(ushort code, VALUE *sp)
 {
 	int val;
 	char *ref;
@@ -500,13 +498,10 @@ static void String_Left(ushort code)
 		ulen = utf8_get_pos(ref, str, len, val);
 		PARAM->_string.len = ulen;
 	}
-
-	SP -= NPARAM;
-	SP++;
 }
 
 
-static void String_Right(ushort code)
+static void String_Right(ushort code, VALUE *sp)
 {
 	int val;
 	char *str;
@@ -535,13 +530,12 @@ static void String_Right(ushort code)
 			val = utf8_get_length(str, len) - val;
 		
 		ulen = utf8_get_pos(ref, str, len, val);
+		if (ulen > len)
+			ulen = len;
 		
 		PARAM->_string.start += ulen;
 		PARAM->_string.len -= ulen;
 	}
-
-	SP -= NPARAM;
-	SP++;
 }
 
 
@@ -756,8 +750,10 @@ BEGIN_METHOD(String_Code, GB_STRING str; GB_INTEGER index)
 	
 	pos = utf8_get_pos(VARG(str).addr, str, len, index - 1);
 	lc = STRING_utf8_get_char_length(str[pos]);
-	
-	GB_ReturnInteger(STRING_utf8_to_unicode(&str[pos], lc));
+	if ((lc + pos) > len)
+		GB_ReturnInteger(-1);
+	else
+		GB_ReturnInteger(STRING_utf8_to_unicode(&str[pos], lc));
 	
 END_METHOD
 
@@ -832,7 +828,7 @@ END_METHOD
 
 #define IS_VALID(_char)                        \
 		((_char) < 0x110000 &&                     \
-		(((_char) & 0xFFFFF800) != 0xD800) &&     \
+		(((_char) & 0xFFFFF800) != 0xD800) &&      \
 		((_char) < 0xFDD0 || (_char) > 0xFDEF) &&  \
 		((_char) & 0xFFFE) != 0xFFFE)
 
