@@ -41,6 +41,7 @@
 #include "gbx_string.h"
 #include "gbx_signal.h"
 #include "gbx_event.h"
+#include "gbx_component.h"
 
 #include "gbx_c_file.h"
 #include "gbx_c_task.h"
@@ -93,7 +94,7 @@ static void has_forked(void)
 	EXEC_Hook.post = NULL;
 	//EXEC_Hook.quit = NULL;
 	
-	stream = CSTREAM_stream(CFILE_out);
+	stream = CSTREAM_TO_STREAM(CFILE_out);
 
 	stream->common.eol = 0;
 	STREAM_blocking(stream, TRUE);
@@ -292,6 +293,8 @@ static bool start_task(CTASK *_object)
 	if (has_error && pipe(fd_err) != 0)
 		goto __ERROR;
 
+	COMPONENT_before_fork();
+
 	// Block SIGCHLD
 
 	sigemptyset(&sig);
@@ -729,6 +732,7 @@ GB_DESC TaskDesc[] =
 	GB_PROPERTY_READ("Running", "b", Task_Running),
 
 	GB_METHOD("Stop", NULL, Task_Stop, NULL),
+	GB_METHOD("Kill", NULL, Task_Stop, NULL),
 	GB_METHOD("Wait", NULL, Task_Wait, NULL),
 
 	GB_EVENT("Read", NULL, "(Data)s", &EVENT_Read),
