@@ -922,7 +922,7 @@ void EXEC_function_loop()
 					PROPAGATE();
 				}
 
-				if (EXEC_break_on_error)
+				if (EXEC_break_on_error && EXEC_debug)
 					DEBUG.Main(TRUE);
 
 				if (ERROR->info.code == E_ASSERT)
@@ -994,7 +994,7 @@ void EXEC_function_loop()
 
 				ERROR_set_last(TRUE);
 
-				if (EXEC_debug && !FP->fast && !STACK_has_error_handler())
+				if (EXEC_debug && !FP->fast_linked && !STACK_has_error_handler())
 				{
 					ERROR_hook();
 					
@@ -1018,11 +1018,11 @@ void EXEC_function_loop()
 					
 					// We can only leave stack frames for non-JIT functions.
 					ERROR_lock();
-					while (PC != NULL && EC == NULL && !FP->fast)
+					while (PC != NULL && EC == NULL && !FP->fast_linked)
 						EXEC_leave_drop();
 					ERROR_unlock();
 
-					if (FP && FP->fast)
+					if (FP && FP->fast_linked)
 						PROPAGATE();
 					
 					// If we got the void stack frame, then we remove it and raise the error again
@@ -1639,7 +1639,7 @@ bool EXEC_special(int special, CLASS *class, void *object, int nparam, bool drop
 	{
 		if (desc->method.subr)
 		{
-			((EXEC_FUNC_CODE)(EXEC.class->table[index].desc->method.exec))(nparam);
+			((EXEC_FUNC_CODE_SP)(EXEC.class->table[index].desc->method.exec))(nparam, SP);
 		}
 		else
 		{
@@ -1942,6 +1942,7 @@ void EXEC_do_quit(void)
 {
 	GAMBAS_DoNotRaiseEvent = TRUE;
 
+	JIT_abort();
 	HOOK(quit)();
 
 	THROW(E_ABORT);
