@@ -753,11 +753,18 @@ _PUSH_ME:
 	}
 	else
 	{
-		if (LIKELY(OP != NULL))
+		if (OP)
 		{
 			SP->_object.class = CP;
 			SP->_object.object = OP;
 		}
+		/*else if (CP->auto_create)
+		{
+			OP = EXEC_auto_create(CP, FALSE);
+			SP->_object.class = CP;
+			SP->_object.object = OP;
+			OP = NULL;
+		}*/
 		else
 		{
 			SP->type = T_CLASS;
@@ -768,7 +775,7 @@ _PUSH_ME:
 	if (GET_UX() & 2)
 	{
 		// The used class must be in the stack, because it is tested by exec_push && exec_pop
-		if (LIKELY(OP != NULL))
+		if (OP)
 		{
 			SP->_object.class = SP->_object.class->parent;
 			SP->_object.super = EXEC_super;
@@ -795,7 +802,7 @@ _PUSH_MISC:
 	{
 		static const void *_jump[] =
 			{ &&__PUSH_NULL, &&__PUSH_VOID, &&__PUSH_FALSE, &&__PUSH_TRUE, &&__PUSH_LAST, &&__PUSH_STRING, &&__PUSH_PINF, &&__PUSH_MINF, &&__PUSH_COMPLEX,
-				&&__PUSH_VARGS, &&__PUSH_DROP_VARGS, &&__JIT_RETURN };
+				&&__PUSH_VARGS, &&__PUSH_DROP_VARGS, &&__JIT_RETURN, &&__PUSH_END_VARGS };
 			//, &&__POP_LAST };
 
 		goto *_jump[GET_UX()];
@@ -830,7 +837,7 @@ _PUSH_MISC:
 
 		SP->type = T_OBJECT;
 		SP->_object.object = EVENT_Last;
-		OBJECT_REF(EVENT_Last);
+		OBJECT_REF_CHECK(EVENT_Last);
 		SP++;
 		goto _NEXT;
 
@@ -873,6 +880,11 @@ _PUSH_MISC:
 		
 	__JIT_RETURN:
 		return;
+		
+	__PUSH_END_VARGS:
+	
+		EXEC_end_vargs();
+		goto _NEXT;
 
 	/*__POP_LAST:
 
@@ -1145,9 +1157,8 @@ _CALL:
 
 		__EXEC_ENTER:
 
-			if (EXEC.func->fast)
+			if (EXEC.func->fast && !JIT_exec(TRUE))
 			{
-				JIT_exec(TRUE);
 				goto _NEXT;
 			}
 			else
@@ -1231,9 +1242,8 @@ _CALL:
 				EXEC.class = EXEC.desc->class;
 				EXEC.func = &EXEC.class->load->func[EXEC.index];
 				
-				if (EXEC.func->fast)
+				if (EXEC.func->fast && !JIT_exec(TRUE))
 				{
-					JIT_exec(TRUE);
 					goto _NEXT;
 				}
 				else
@@ -1251,9 +1261,10 @@ _CALL:
 
 		__CALL_SUBR:
 
-			((EXEC_FUNC_CODE)(EXEC.class->table[val->_function.index].desc->method.exec))(code);
-			SP[-2] = SP[-1];
-			SP--;
+			ind = GET_3X();
+			((EXEC_FUNC_CODE_SP)(EXEC.class->table[val->_function.index].desc->method.exec))(ind, SP);
+			SP -= ind;
+			SP[-1] = SP[0];
 			goto _NEXT;
 		}
 
@@ -1411,9 +1422,8 @@ _CALL_SLOW:
 
 			EXEC.func = &EXEC.class->load->func[EXEC.index];
 
-			if (EXEC.func->fast)
+			if (EXEC.func->fast && !JIT_exec(TRUE))
 			{
-				JIT_exec(TRUE);
 				goto _NEXT;
 			}
 			else
@@ -1639,7 +1649,7 @@ _PUSH_FUNCTION:
 	SP->_function.index = GET_7XX();
 	SP->_function.defined = TRUE;
 
-	OBJECT_REF(OP);
+	OBJECT_REF_CHECK(OP);
 	SP++;
 
 	goto _NEXT;
@@ -3658,7 +3668,7 @@ __PUSH_NATIVE_ARRAY:
 	for (i = 1; i <= np; i++)
 		VALUE_conv_integer(&val[i]);
 
-	data = CARRAY_get_data_multi((CARRAY *)object, (GB_INTEGER *)&val[1], np);
+	data = CARRAY_get_data_multi((CARRAY *)object, (GB_INTEGER *)&val[1], np - 1);
 	if (!data)
 		PROPAGATE();
 
@@ -3852,15 +3862,12 @@ __POP_NATIVE_ARRAY:
 	EXEC_object_array(val, class, object);
 	array = (CARRAY *)object;
 
-	/*VALUE_copy(&swap, &val[0]);
-	VALUE_copy(&val[0], &val[-1]);
-	VALUE_copy(&val[-1], &swap);*/
-
-	//VALUE_conv(&val[0], type);
+	CARRAY_check_not_read_only(array);
+	
 	for (i = 1; i < np; i++)
 		VALUE_conv_integer(&val[i]);
 
-	data = CARRAY_get_data_multi((CARRAY *)object, (GB_INTEGER *)&val[1], np - 1);
+	data = CARRAY_get_data_multi((CARRAY *)object, (GB_INTEGER *)&val[1], np - 2);
 	if (data == NULL)
 		PROPAGATE();
 
@@ -3887,11 +3894,8 @@ __POP_NATIVE_ARRAY_SIMPLE:
 	EXEC_object_array(val, class, object);
 	array = (CARRAY *)object;
 
-	/*VALUE_copy(&swap, &val[0]);
-	VALUE_copy(&val[0], &val[-1]);
-	VALUE_copy(&val[-1], &swap);*/
-
-	//VALUE_conv(&val[0], type);
+	CARRAY_check_not_read_only(array);
+	
 	VALUE_conv_integer(&val[1]);
 
 	data = CARRAY_get_data(array, val[1]._integer.value);
@@ -3907,10 +3911,8 @@ __POP_NATIVE_ARRAY_INTEGER:
 	EXEC_object_array(val, class, object);
 	array = (CARRAY *)object;
 
-	/*VALUE_copy(&swap, &val[0]);
-	VALUE_copy(&val[0], &val[-1]);
-	VALUE_copy(&val[-1], &swap);*/
-
+	CARRAY_check_not_read_only(array);
+	
 	VALUE_conv_integer(&val[-1]);
 	VALUE_conv_integer(&val[1]);
 
@@ -3927,10 +3929,8 @@ __POP_NATIVE_ARRAY_FLOAT:
 	EXEC_object_array(val, class, object);
 	array = (CARRAY *)object;
 
-	/*VALUE_copy(&swap, &val[0]);
-	VALUE_copy(&val[0], &val[-1]);
-	VALUE_copy(&val[-1], &swap);*/
-
+	CARRAY_check_not_read_only(array);
+	
 	VALUE_conv_float(&val[-1]);
 	VALUE_conv_integer(&val[1]);
 
@@ -3980,20 +3980,20 @@ void EXEC_quit(ushort code)
 {
 	switch(code & 3)
 	{
-		case 0:
+		case 0: // QUIT
 			EXEC_do_quit();
 			break;
 
-		case 1:
+		case 1: // STOP
 			if (EXEC_debug && CP) // && CP->component == COMPONENT_main)
 				DEBUG.Breakpoint(0);
 			break;
 
-		case 2:
+		case 2: // STOP EVENT
 			GAMBAS_StopEvent = TRUE;
 			break;
 
-		case 3:
+		case 3: // QUIT <return value>
 			VALUE_conv(&SP[-1], T_BYTE);
 			SP--;
 			EXEC_quit_value = (uchar)SP->_integer.value;
@@ -4004,6 +4004,24 @@ void EXEC_quit(ushort code)
 
 static void _break(ushort code)
 {
+	if (!EXEC_trace && !EXEC_debug)
+	{
+		*PC = C_NOP;
+		return;
+	}
+	
+	if (EXEC_trace)
+	{
+		double timer;
+		char *addr;
+		int len;
+		
+		DATE_timer(&timer, TRUE);
+		LOCAL_format_number(timer, LF_GENERAL_NUMBER, NULL, 0, &addr, &len, FALSE);
+		fprintf(stderr, "[%d.%06d] %s\n", (int)timer, (int)(timer * 1000000) % 1000000, DEBUG_get_current_position());
+		fflush(stderr);
+	}
+	
 	if (EXEC_debug)
 	{
 		/*TC = PC + 1;
@@ -4011,7 +4029,7 @@ static void _break(ushort code)
 
 		//fprintf(stderr, "%s\n", DEBUG_get_current_position());
 
-		if (CP && CP->component == COMPONENT_main)
+		if (CP && CP->component == NULL)
 		{
 			if (EXEC_profile_instr)
 				DEBUG.Profile.Add(CP, FP, PC);
@@ -4045,8 +4063,6 @@ static void _break(ushort code)
 			DEBUG.Breakpoint(code);
 		}
 	}
-	else
-		*PC = C_NOP;
 }
 
 void SUBR_left(ushort code)
