@@ -1,17 +1,12 @@
 /***************************************************************************
 
-  Number.h
+  gbx_number.h
 
-  Numbers management routines
-
-  Datatype management routines. Conversions between each Gambas datatype,
-  and conversions between Gambas datatypes and native datatypes.
-
-  (c) 2000-2005 Benot Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -21,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -43,7 +39,7 @@ enum {
   };
 
 
-PUBLIC bool NUMBER_from_string(int option, const char *str, long len, VALUE *value);
-PUBLIC void NUMBER_int_to_string(unsigned long long nbr, int prec, int base, VALUE *value);
+bool NUMBER_from_string(int option, const char *str, int len, VALUE *value);
+void NUMBER_int_to_string(uint64_t nbr, int prec, int base, VALUE *value);
 
 #endif /* */
