@@ -1,14 +1,12 @@
 /***************************************************************************
 
-  subr_math_temp.h
+  gbx_subr_math_temp.h
 
-  Mathematical routines templates
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,13 +16,14 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
 #if SMT_TYPE == 1
 
-PUBLIC void SMT_NAME(void)
+void SMT_NAME(void)
 {
   #ifdef SMT_FLOAT
 
@@ -115,7 +114,7 @@ __LONG:
 
 __FLOAT:
 
-  VALUE_conv(P1, T_FLOAT);
+  VALUE_conv_float(P1);
 
   #ifdef SMT_OP
   P1->_float.value = SMT_OP ( P1->_float.value );
@@ -138,7 +137,7 @@ __ERROR:
 
 __VARIANT_END:
 
-  VALUE_conv(P1, T_VARIANT);
+  VALUE_conv_variant(P1);
 
 __END:
   return;
@@ -151,7 +150,7 @@ __END:
 
 #if SMT_TYPE == 2
 
-PUBLIC void SMT_NAME(void)
+void SMT_NAME(void)
 {
 
   #ifdef SMT_FLOAT
@@ -234,8 +233,8 @@ __DATE:
 
 __FLOAT:
 
-  VALUE_conv(P1, T_FLOAT);
-  VALUE_conv(P2, T_FLOAT);
+  VALUE_conv_float(P1);
+  VALUE_conv_float(P2);
 
   #ifdef SMT_OP
   P1->_float.value SMT_OP P2->_float.value;
@@ -277,10 +276,10 @@ __VARIANT:
     VARIANT_undo(P2);
 
   if (TYPE_is_string(P1->type))
-    VALUE_conv(P1, T_FLOAT);
+    VALUE_conv_float(P1);
 
   if (TYPE_is_string(P2->type))
-    VALUE_conv(P2, T_FLOAT);
+    VALUE_conv_float(P2);
 
   if (TYPE_is_null(P1->type) || TYPE_is_null(P2->type))
     type = T_NULL;
@@ -301,7 +300,7 @@ __ERROR:
 
 __VARIANT_END:
 
-  VALUE_conv(P1, T_VARIANT);
+  VALUE_conv_variant(P1);
 
 __END:
 
@@ -314,7 +313,7 @@ __END:
 
 #if SMT_TYPE == 3
 
-PUBLIC void SMT_NAME(void)
+void SMT_NAME(ushort code)
 {
 
   static void *jump[] = {
@@ -327,7 +326,7 @@ PUBLIC void SMT_NAME(void)
   P1 = SP - 2;
   P2 = P1 + 1;
 
-  type = *PC & 0x0F;
+  type = code & 0x0F;
   goto *jump[type];
 
 __VARIANT:
