@@ -369,452 +369,6 @@ void SUBR_near(void)
   SP--;
 }
 
-#if 0
-void SUBR_comp(ushort code)
-{
-	static void *jump[17] = {
-		&&__VARIANT, &&__BOOLEAN, &&__BYTE, &&__SHORT, &&__INTEGER, &&__LONG, &&__SINGLE, &&__FLOAT, &&__DATE,
-		&&__STRING, &&__STRING, &&__POINTER, &&__ERROR, &&__ERROR, &&__ERROR, &&__NULL, &&__OBJECT
-		};
-
-	//static void *test[] = { &&__EQ, &&__NE, &&__GT, &&__LE, &&__LT, &&__GE };
-
-	char NO_WARNING(result);
-	VALUE *P1;
-	VALUE *P2;
-
-	P1 = SP - 2;
-	P2 = P1 + 1;
-
-	goto *jump[code & 0x1F];
-
-__BOOLEAN:
-__BYTE:
-__SHORT:
-__INTEGER:
-
-	result = P1->_integer.value == P2->_integer.value;
-	goto __END;
-	
-__LONG:
-
-	VALUE_conv(P1, T_LONG);
-	VALUE_conv(P2, T_LONG);
-
-	result = P1->_long.value == P2->_long.value;
-	goto __END;
-
-__DATE:
-
-	VALUE_conv(P1, T_DATE);
-	VALUE_conv(P2, T_DATE);
-
-	result = DATE_comp_value(P1, P2) == 0;
-	goto __END;
-
-__NULL:
-
-	if (P2->type == T_NULL)
-	{
-		result = VALUE_is_null(P1);
-		goto __END_RELEASE;
-	}
-	else if (P1->type == T_NULL)
-	{
-		result = VALUE_is_null(P2);
-		goto __END_RELEASE;
-	}
-
-__STRING:
-
-	VALUE_conv_string(P1);
-	VALUE_conv_string(P2);
-
-	if (P1->_string.len != P2->_string.len)
-		result = 0;
-	else
-		result = STRING_equal_same(P1->_string.addr + P1->_string.start, P2->_string.addr + P2->_string.start, P1->_string.len);
-	
-	RELEASE_STRING(P1);
-	RELEASE_STRING(P2);
-	goto __END;
-
-__SINGLE:
-__FLOAT:
-
-	VALUE_conv_float(P1);
-	VALUE_conv_float(P2);
-
-	result = P1->_float.value == P2->_float.value;
-	goto __END;
-
-__POINTER:
-
-	VALUE_conv(P1, T_POINTER);
-	VALUE_conv(P2, T_POINTER);
-
-	result = P1->_pointer.value == P2->_pointer.value;
-	goto __END;
-
-__OBJECT:
-
-	result = OBJECT_comp_value(P1, P2) == 0;
-	//RELEASE_OBJECT(P1);
-	//RELEASE_OBJECT(P2);
-	goto __END_RELEASE;
-
-__VARIANT:
-
-	{
-		bool variant = FALSE;
-		TYPE type;
-	
-		if (TYPE_is_variant(P1->type))
-		{
-			VARIANT_undo(P1);
-			variant = TRUE;
-		}
-
-		if (TYPE_is_variant(P2->type))
-		{
-			VARIANT_undo(P2);
-			variant = TRUE;
-		}
-
-		type = Max(P1->type, P2->type);
-
-		if (TYPE_is_object_null(P1->type) && TYPE_is_object_null(P2->type))
-			type = T_OBJECT;
-		else if (TYPE_is_object(type))
-			THROW(E_TYPE, "Object", TYPE_get_name(Min(P1->type, P2->type)));
-
-		if (!variant)
-			*PC |= type;
-
-		goto *jump[type];
-	}
-
-__ERROR:
-
-	THROW(E_TYPE, "Number, Date or String", TYPE_get_name(code & 0x1F));
-
-__END_RELEASE:
-
-	RELEASE(P1);
-	RELEASE(P2);
-
-__END:
-
-	P1->type = T_BOOLEAN;
-	SP--;
-	P1->_boolean.value = -result;
-}
-
-void SUBR_compn(ushort code)
-{
-	static void *jump[17] = {
-		&&__VARIANT, &&__BOOLEAN, &&__BYTE, &&__SHORT, &&__INTEGER, &&__LONG, &&__SINGLE, &&__FLOAT, &&__DATE,
-		&&__STRING, &&__STRING, &&__POINTER, &&__ERROR, &&__ERROR, &&__ERROR, &&__NULL, &&__OBJECT
-		};
-
-	//static void *test[] = { &&__EQ, &&__NE, &&__GT, &&__LE, &&__LT, &&__GE };
-
-	char NO_WARNING(result);
-	VALUE *P1;
-	VALUE *P2;
-
-	P1 = SP - 2;
-	P2 = P1 + 1;
-
-	goto *jump[code & 0x1F];
-
-__BOOLEAN:
-__BYTE:
-__SHORT:
-__INTEGER:
-
-	result = P1->_integer.value == P2->_integer.value;
-	goto __END;
-	
-__LONG:
-
-	VALUE_conv(P1, T_LONG);
-	VALUE_conv(P2, T_LONG);
-
-	result = P1->_long.value == P2->_long.value;
-	goto __END;
-
-__DATE:
-
-	VALUE_conv(P1, T_DATE);
-	VALUE_conv(P2, T_DATE);
-
-	result = DATE_comp_value(P1, P2) == 0;
-	goto __END;
-
-__NULL:
-
-	if (P2->type == T_NULL)
-	{
-		result = VALUE_is_null(P1);
-		goto __END_RELEASE;
-	}
-	else if (P1->type == T_NULL)
-	{
-		result = VALUE_is_null(P2);
-		goto __END_RELEASE;
-	}
-
-__STRING:
-
-	VALUE_conv_string(P1);
-	VALUE_conv_string(P2);
-
-	if (P1->_string.len != P2->_string.len)
-		result = 0;
-	else
-		result = STRING_equal_same(P1->_string.addr + P1->_string.start, P2->_string.addr + P2->_string.start, P1->_string.len);
-	
-	RELEASE_STRING(P1);
-	RELEASE_STRING(P2);
-	goto __END;
-
-__SINGLE:
-__FLOAT:
-
-	VALUE_conv_float(P1);
-	VALUE_conv_float(P2);
-
-	result = P1->_float.value == P2->_float.value;
-	goto __END;
-
-__POINTER:
-
-	VALUE_conv(P1, T_POINTER);
-	VALUE_conv(P2, T_POINTER);
-
-	result = P1->_pointer.value == P2->_pointer.value;
-	goto __END;
-
-__OBJECT:
-
-	result = OBJECT_comp_value(P1, P2) == 0;
-	//RELEASE_OBJECT(P1);
-	//RELEASE_OBJECT(P2);
-	goto __END_RELEASE;
-
-__VARIANT:
-
-	{
-		bool variant = FALSE;
-		TYPE type;
-	
-		if (TYPE_is_variant(P1->type))
-		{
-			VARIANT_undo(P1);
-			variant = TRUE;
-		}
-
-		if (TYPE_is_variant(P2->type))
-		{
-			VARIANT_undo(P2);
-			variant = TRUE;
-		}
-
-		type = Max(P1->type, P2->type);
-
-		if (TYPE_is_object_null(P1->type) && TYPE_is_object_null(P2->type))
-			type = T_OBJECT;
-		else if (TYPE_is_object(type))
-			THROW(E_TYPE, "Object", TYPE_get_name(Min(P1->type, P2->type)));
-
-		if (!variant)
-			*PC |= type;
-
-		goto *jump[type];
-	}
-
-__ERROR:
-
-	THROW(E_TYPE, "Number, Date or String", TYPE_get_name(code & 0x1F));
-
-__END_RELEASE:
-
-	RELEASE(P1);
-	RELEASE(P2);
-
-__END:
-
-	P1->type = T_BOOLEAN;
-	SP--;
-
-	P1->_boolean.value = result - 1; // ? 0 : -1;
-}
-
-#define sgn(_x) \
-({ \
-	int x = _x; \
-	int minusOne = x >> 31; \
-	unsigned int negateX = (unsigned int) -x; \
-	int plusOne = (int)(negateX >> 31); \
-  int result = minusOne | plusOne; \
-  result; \
-})
-
-void SUBR_compi(ushort code)
-{
-	static void *jump[17] = {
-		&&__VARIANT, &&__BOOLEAN, &&__BYTE, &&__SHORT, &&__INTEGER, &&__LONG, &&__SINGLE, &&__FLOAT, &&__DATE,
-		&&__STRING, &&__STRING, &&__POINTER, &&__ERROR, &&__ERROR, &&__ERROR, &&__NULL, &&__OBJECT
-		};
-
-	static void *test[] = { &&__GT, &&__LE, &&__LT, &&__GE };
-
-	char NO_WARNING(result);
-	VALUE *P1;
-	VALUE *P2;
-	TYPE type;
-
-	P1 = SP - 2;
-	P2 = P1 + 1;
-
-	type = code & 0x1F;
-	goto *jump[type];
-
-__BOOLEAN:
-__BYTE:
-__SHORT:
-__INTEGER:
-
-	result = P1->_integer.value > P2->_integer.value ? 1 : P1->_integer.value < P2->_integer.value ? -1 : 0;
-	goto __END;
-	
-__LONG:
-
-	VALUE_conv(P1, T_LONG);
-	VALUE_conv(P2, T_LONG);
-
-	result = P1->_long.value > P2->_long.value ? 1 : P1->_long.value < P2->_long.value ? -1 : 0;
-	goto __END;
-
-__DATE:
-
-	VALUE_conv(P1, T_DATE);
-	VALUE_conv(P2, T_DATE);
-
-	result = DATE_comp_value(P1, P2);
-	goto __END;
-
-__NULL:
-__STRING:
-
-	VALUE_conv_string(P1);
-	VALUE_conv_string(P2);
-
-	result = STRING_compare(P1->_string.addr + P1->_string.start, P1->_string.len, P2->_string.addr + P2->_string.start, P2->_string.len);
-	
-	RELEASE_STRING(P1);
-	RELEASE_STRING(P2);
-	goto __END;
-
-__SINGLE:
-
-	VALUE_conv(P1, T_SINGLE);
-	VALUE_conv(P2, T_SINGLE);
-
-	result = P1->_single.value > P2->_single.value ? 1 : P1->_single.value < P2->_single.value ? -1 : 0;
-	goto __END;
-
-__FLOAT:
-
-	VALUE_conv_float(P1);
-	VALUE_conv_float(P2);
-
-	result = P1->_float.value > P2->_float.value ? 1 : P1->_float.value < P2->_float.value ? -1 : 0;
-	goto __END;
-
-__POINTER:
-
-	VALUE_conv(P1, T_POINTER);
-	VALUE_conv(P2, T_POINTER);
-
-	result = P1->_pointer.value > P2->_pointer.value ? 1 : P1->_pointer.value < P2->_pointer.value ? -1 : 0;
-	goto __END;
-
-__OBJECT:
-
-	result = OBJECT_comp_value(P1, P2);
-	//RELEASE_OBJECT(P1);
-	//RELEASE_OBJECT(P2);
-	goto __END_RELEASE;
-
-__VARIANT:
-
-	{
-		bool variant = FALSE;
-	
-		if (TYPE_is_variant(P1->type))
-		{
-			VARIANT_undo(P1);
-			variant = TRUE;
-		}
-
-		if (TYPE_is_variant(P2->type))
-		{
-			VARIANT_undo(P2);
-			variant = TRUE;
-		}
-
-		type = Max(P1->type, P2->type);
-
-		if (type == T_NULL || TYPE_is_string(type))
-		{
-			TYPE typem = Min(P1->type, P2->type);
-			if (!TYPE_is_string(typem))
-				THROW(E_TYPE, TYPE_get_name(typem), TYPE_get_name(type));
-		}
-		else if (TYPE_is_object(type))
-			goto __ERROR;
-
-		if (!variant)
-			*PC |= type;
-
-		goto *jump[type];
-	}
-
-__ERROR:
-
-	THROW(E_TYPE, "Number, Date or String", TYPE_get_name(type));
-
-__END_RELEASE:
-
-	RELEASE(P1);
-	RELEASE(P2);
-
-__END:
-
-	P1->type = T_BOOLEAN;
-	SP--;
-
-	goto *test[(code >> 8) - (C_GT >> 8)];
-
-__GT:
-	P1->_boolean.value = result > 0 ? -1 : 0;
-	return;
-
-__GE:
-	P1->_boolean.value = result >= 0 ? -1 : 0;
-	return;
-
-__LT:
-	P1->_boolean.value = result < 0 ? -1 : 0;
-	return;
-
-__LE:
-	P1->_boolean.value = result <= 0 ? -1 : 0;
-	return;
-}
-#endif
 
 void SUBR_strcomp(ushort code)
 {
@@ -838,7 +392,6 @@ void SUBR_strcomp(ushort code)
 }
 
 
-
 void SUBR_is(ushort code)
 {
 	VALUE *P1 = SP - 2;
@@ -867,85 +420,130 @@ void SUBR_is(ushort code)
 void SUBR_min_max(ushort code)
 {
   static void *jump[] = {
-    &&__VARIANT, &&__BOOLEAN, &&__BYTE, &&__SHORT, &&__INTEGER, &&__LONG, &&__FLOAT, &&__FLOAT, &&__DATE
+    &&__VARIANT, &&__MIN_BOOLEAN, &&__MIN_BYTE, &&__MIN_SHORT, &&__MIN_INTEGER, &&__MIN_LONG, &&__MIN_SINGLE, &&__MIN_FLOAT, 
+		&&__MIN_DATE, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    &&__VARIANT, &&__MAX_BOOLEAN, &&__MAX_BYTE, &&__MAX_SHORT, &&__MAX_INTEGER, &&__MAX_LONG, &&__MAX_SINGLE, &&__MAX_FLOAT, 
+		&&__MAX_DATE, NULL, NULL, NULL, NULL, NULL, NULL, NULL
     };
 
   TYPE type;
   VALUE *P1, *P2;
   void *jump_end;
-  bool is_max;
 
   P1 = SP - 2;
   P2 = P1 + 1;
 
   jump_end = &&__END;
   type = code & 0x0F;
-  is_max = ((code >> 8) == CODE_MAX);
-  goto *jump[type];
+  goto *jump[code & 0x1F];
 
-__BOOLEAN:
-__BYTE:
-__SHORT:
-__INTEGER:
+__MIN_BOOLEAN:
+__MIN_BYTE:
+__MIN_SHORT:
+__MIN_INTEGER:
 
 	P1->type = type;
 
-	if (is_max)
-	{
-		if (P2->_integer.value > P1->_integer.value)
-			P1->_integer.value = P2->_integer.value;
-	}
-	else
-	{
-		if (P2->_integer.value < P1->_integer.value)
-			P1->_integer.value = P2->_integer.value;
-	}
+	if (P2->_integer.value < P1->_integer.value)
+		P1->_integer.value = P2->_integer.value;
+
+	goto *jump_end;
+
+__MAX_BOOLEAN:
+__MAX_BYTE:
+__MAX_SHORT:
+__MAX_INTEGER:
+
+	P1->type = type;
+
+	if (P2->_integer.value > P1->_integer.value)
+		P1->_integer.value = P2->_integer.value;
 	
 	goto *jump_end;
 
-__LONG:
+__MIN_LONG:
 
   VALUE_conv(P1, T_LONG);
   VALUE_conv(P2, T_LONG);
 
-	if (is_max)
-	{
-		if (P2->_long.value > P1->_long.value)
-			P1->_long.value = P2->_long.value;
-	}
-	else
-	{
-		if (P2->_long.value < P1->_long.value)
-			P1->_long.value = P2->_long.value;
-	}
+	if (P2->_long.value < P1->_long.value)
+		P1->_long.value = P2->_long.value;
 	
 	goto *jump_end;
 
-__FLOAT:
+__MAX_LONG:
+
+  VALUE_conv(P1, T_LONG);
+  VALUE_conv(P2, T_LONG);
+
+	if (P2->_long.value > P1->_long.value)
+		P1->_long.value = P2->_long.value;
+	
+	goto *jump_end;
+
+__MIN_SINGLE:
+
+  VALUE_conv(P1, T_SINGLE);
+  VALUE_conv(P2, T_SINGLE);
+
+	if (P2->_single.value < P1->_single.value)
+		P1->_single.value = P2->_single.value;
+	
+	goto *jump_end;
+
+__MAX_SINGLE:
+
+  VALUE_conv(P1, T_SINGLE);
+  VALUE_conv(P2, T_SINGLE);
+
+	if (P2->_single.value > P1->_single.value)
+		P1->_single.value = P2->_single.value;
+	
+	goto *jump_end;
+
+__MIN_FLOAT:
 
   VALUE_conv_float(P1);
   VALUE_conv_float(P2);
 
-	if (is_max)
-	{
-		if (P2->_float.value > P1->_float.value)
-			P1->_float.value = P2->_float.value;
-	}
-	else
-	{
-		if (P2->_float.value < P1->_float.value)
-			P1->_float.value = P2->_float.value;
-	}
+	if (P2->_float.value < P1->_float.value)
+		P1->_float.value = P2->_float.value;
 	
 	goto *jump_end;
 
-__DATE:
+__MAX_FLOAT:
+
+  VALUE_conv_float(P1);
+  VALUE_conv_float(P2);
+
+	if (P2->_float.value > P1->_float.value)
+		P1->_float.value = P2->_float.value;
+	
+	goto *jump_end;
+
+__MIN_DATE:
 
   VALUE_conv(P1, T_DATE);
   VALUE_conv(P2, T_DATE);
 
-	if (DATE_comp_value(P1, P2) == (is_max ? -1 : 1))
-		*P1 = *P2;
+	if (DATE_comp_value(P1, P2) == 1)
+	{
+		P1->_date.date = P2->_date.date;
+		P1->_date.time = P2->_date.time;
+	}
+
+	goto *jump_end;
+
+__MAX_DATE:
+
+  VALUE_conv(P1, T_DATE);
+  VALUE_conv(P2, T_DATE);
+
+	if (DATE_comp_value(P1, P2) == -1)
+	{
+		P1->_date.date = P2->_date.date;
+		P1->_date.time = P2->_date.time;
+	}
 
 	goto *jump_end;
 
@@ -955,8 +553,9 @@ __VARIANT:
 
   if (TYPE_is_number_date(type))
   {
-    *PC |= type;
-    goto *jump[type];
+		code = type + ((code >> 8) == CODE_MAX) * 16;
+    *PC |= code;
+    goto *jump[code];
   }
 
   if (TYPE_is_variant(P1->type))
@@ -970,7 +569,8 @@ __VARIANT:
   if (TYPE_is_number_date(type))
   {
     jump_end = &&__VARIANT_END;
-    goto *jump[type];
+		code = type + ((code >> 8) == CODE_MAX) * 16;
+    goto *jump[code];
   }
 
   goto __ERROR;
