@@ -362,8 +362,8 @@ void VALUE_convert(VALUE *value, TYPE type)
 	/* g      */ { &&__N,     &&__g2b,   &&__g2c,   &&__g2h,   &&__g2i,   &&__g2l,   &&__OK,    &&__g2f,   &&__g2d,   &&__g2s,   &&__g2s,   &&__N,     &&__2v,    &&__N,     &&__N,     &&__N,     },
 	/* f      */ { &&__N,     &&__f2b,   &&__f2c,   &&__f2h,   &&__f2i,   &&__f2l,   &&__f2g,   &&__OK,    &&__f2d,   &&__f2s,   &&__f2s,   &&__N,     &&__2v,    &&__N,     &&__N,     &&__N,     },
 	/* d      */ { &&__N,     &&__d2b,   &&__d2c,   &&__d2h,   &&__d2i,   &&__d2l,   &&__d2g,   &&__d2f,   &&__OK,    &&__d2s,   &&__d2s,   &&__N,     &&__2v,    &&__N,     &&__N,     &&__N,     },
-	/* cs     */ { &&__N,     &&__s2b,   &&__s2c,   &&__s2h,   &&__s2i,   &&__s2l,   &&__s2g,   &&__s2f,   &&__s2d,   &&__OK,    &&__OK,    &&__N,     &&__s2v,   &&__N,     &&__N,     &&__N,     },
-	/* s      */ { &&__N,     &&__s2b,   &&__s2c,   &&__s2h,   &&__s2i,   &&__s2l,   &&__s2g,   &&__s2f,   &&__s2d,   &&__OK,    &&__OK,    &&__N,     &&__s2v,   &&__N,     &&__N,     &&__N,     },
+	/* cs     */ { &&__N,     &&__s2b,   &&__s2c,   &&__s2h,   &&__s2i,   &&__s2l,   &&__s2g,   &&__s2f,   &&__s2d,   &&__OK,    &&__OK,    &&__s2p,   &&__s2v,   &&__N,     &&__N,     &&__N,     },
+	/* s      */ { &&__N,     &&__s2b,   &&__s2c,   &&__s2h,   &&__s2i,   &&__s2l,   &&__s2g,   &&__s2f,   &&__s2d,   &&__OK,    &&__OK,    &&__s2p,   &&__s2v,   &&__N,     &&__N,     &&__N,     },
 	/* p      */ { &&__N,     &&__p2b,   &&__N,     &&__N,     &&__p2i,   &&__p2l,   &&__N,     &&__N,     &&__N,     &&__p2s,   &&__p2s,   &&__OK,    &&__2v,    &&__N,     &&__N,     &&__N,     },
 	/* v      */ { &&__N,     &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__OK,    &&__N,     &&__v2,    &&__v2,    },
 	/* func   */ { &&__N,     &&__func,  &&__func,  &&__func,  &&__func,  &&__func,  &&__func,  &&__func,  &&__func,  &&__func,  &&__func,  &&__F2p,   &&__func,  &&__OK,    &&__N,     &&__func,  },
@@ -756,6 +756,12 @@ __s2d:
 
 	STRING_unref(&addr);
 	return;
+	
+__s2p:
+
+	value->_pointer.value = value->_string.addr + value->_string.start;
+	value->type = T_POINTER;
+	return;
 
 __n2b:
 
@@ -928,7 +934,7 @@ __OBJECT:
 				{
 					if (!((*class->convert)(NULL, value->type, value)))
 					{
-						OBJECT_REF(value->_object.object);
+						OBJECT_REF_CHECK(value->_object.object);
 						goto __TYPE;
 					}
 				}
@@ -970,7 +976,7 @@ __RETRY:
 		if (!((*class->convert)(value->_object.object, type, value)))
 		{
 			OBJECT_UNREF(unref);
-			OBJECT_REF(value->_object.object);
+			OBJECT_REF_CHECK(value->_object.object);
 			goto __TYPE;
 		}
 	}
@@ -982,7 +988,7 @@ __RETRY:
 		if (!((*class2->convert)(NULL, OBJECT_class(unref), value)))
 		{
 			OBJECT_UNREF(unref);
-			OBJECT_REF(value->_object.object);
+			OBJECT_REF_CHECK(value->_object.object);
 			goto __TYPE;
 		}
 	}
@@ -1052,7 +1058,7 @@ __OBJECT:
 __CLASS:
 {
 	void *object = value->_variant.value._object;
-	OBJECT_REF(object);
+	OBJECT_REF_CHECK(object);
 	VARIANT_free((VARIANT *)addr);
 	((VARIANT *)addr)->type = type;
 	((VARIANT *)addr)->value._object = object;
@@ -1143,7 +1149,7 @@ __OBJECT:
 
 	VALUE_conv(value, type);
 
-	OBJECT_REF(value->_object.object);
+	OBJECT_REF_CHECK(value->_object.object);
 	OBJECT_UNREF(*((void **)addr));
 	*((void **)addr) = value->_object.object;
 	return;
@@ -1335,7 +1341,7 @@ void VALUE_free(void *addr, TYPE type)
 
 
 
-void VALUE_to_string(VALUE *value, char **addr, int *len)
+void VALUE_to_local_string(VALUE *value, char **addr, int *len)
 {
 	static void *jump[16] = {
 		&&__VOID, &&__BOOLEAN, &&__BYTE, &&__SHORT, &&__INTEGER, &&__LONG, &&__SINGLE, &&__FLOAT, &&__DATE,
@@ -1359,15 +1365,13 @@ __BOOLEAN:
 
 	if (value->_boolean.value)
 	{
-		*addr = (char *)LOCAL_gettext("True");
-		*len = strlen(*addr);
+		*addr = LOCAL_local.true_str;
+		*len = LOCAL_local.len_true_str;
 	}
 	else
 	{
-		*addr = (char *)LOCAL_gettext("False");
-		*len = strlen(*addr);
-		//*addr = LOCAL_local.false_str;
-		//*len = LOCAL_local.len_false_str;
+		*addr = LOCAL_local.false_str;
+		*len = LOCAL_local.len_false_str;
 	}
 	return;
 
@@ -1388,9 +1392,11 @@ __LONG:
 	return;
 
 __DATE:
-
-	LOCAL_format_date(DATE_split(value), LF_STANDARD, NULL, 0, addr, len);
-	return;
+	{
+		const DATE_SERIAL *date = DATE_split(value);
+		LOCAL_format_date(date, DATE_SERIAL_has_no_date(date) ? LF_LONG_TIME : LF_STANDARD, NULL, 0, addr, len);
+		return;
+	}
 
 __SINGLE:
 
@@ -1468,7 +1474,7 @@ __FUNCTION:
 }
 
 
-void VALUE_from_string(VALUE *value, const char *addr, int len)
+void VALUE_from_local_string(VALUE *value, const char *addr, int len)
 {
 	while (len > 0 && isspace(*addr))
 		addr++, len--;
@@ -1517,7 +1523,7 @@ void VALUE_class_write(CLASS *class, VALUE *value, char *addr, CTYPE ctype)
 
 		VALUE_conv(value, type);
 
-		OBJECT_REF(value->_object.object);
+		OBJECT_REF_CHECK(value->_object.object);
 		OBJECT_UNREF(*((void **)addr));
 		*((void **)addr) = value->_object.object;
 		//VALUE_write(value, addr, (ctype.value >= 0) ? (TYPE)class->load->class_ref[ctype.value] : T_OBJECT);
