@@ -2,11 +2,11 @@
 
   main.c
 
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -16,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -29,31 +30,41 @@
 #include <ctype.h>
 
 #include "gb_common.h"
-
-#include "CDraw.h"
+#include "cpaint.h"
 #include "main.h"
 
 
 GB_INTERFACE GB EXPORT;
+GEOM_INTERFACE GEOM EXPORT;
+IMAGE_INTERFACE IMAGE EXPORT;
 
 GB_DESC *GB_CLASSES [] EXPORT =
 {
-	CDrawClipDesc,
-	CDrawDesc,
+	PaintExtentsDesc,
+	PaintMatrixDesc,
+	PaintBrushDesc,
+	PaintDesc,
   NULL
 };
 
 void *GB_DRAW_1[] EXPORT =
 {
 	(void *)1,
-	(void *)DRAW_get_current,
-	(void *)DRAW_begin,
-	(void *)DRAW_end,
+	(void *)PAINT_get_current,
+	(void *)PAINT_begin,
+	(void *)PAINT_end,
+	(void *)PAINT_is_painted,
 	NULL
 };
 
+const char *GB_INCLUDE EXPORT = "gb.geom";
+
 int EXPORT GB_INIT(void)
 {
+	GB.Component.Load("gb.geom");
+  GB.GetInterface("gb.geom", GEOM_INTERFACE_VERSION, &GEOM);
+	GB.Component.Load("gb.image");
+  GB.GetInterface("gb.image", IMAGE_INTERFACE_VERSION, &IMAGE);
   return 0;
 }
 
