@@ -2,13 +2,11 @@
 
   eval.h
 
-  Expression evaluator
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -27,6 +26,7 @@
 
 #include "gb_table.h"
 #include "gb_reserved.h"
+#include "gb_error.h"
 #include "eval_read.h"
 /*#include "CCollection.h"*/
 #include "main.h"
@@ -40,21 +40,23 @@ EXTERN EXPRESSION EVAL_read_expr;
 #endif
 
 
-PUBLIC void EVAL_init(void);
-PUBLIC void EVAL_exit(void);
+void EVAL_init(void);
+void EVAL_exit(void);
 
-PUBLIC void EVAL_new(EXPRESSION **expr, char *src, int len);
-PUBLIC void EVAL_free(EXPRESSION **expr);
-PUBLIC bool EVAL_compile(EXPRESSION *expr);
+void EVAL_new(EXPRESSION **expr, char *src, int len);
+void EVAL_free(EXPRESSION **expr);
+bool EVAL_compile(EXPRESSION *expr, bool assign);
 
-PUBLIC GB_VALUE *EVAL_expression(EXPRESSION *expr, EVAL_FUNCTION get_value);
-PUBLIC void EVAL_clear(EXPRESSION *expr);
+GB_VALUE *EVAL_expression(EXPRESSION *expr, EVAL_FUNCTION get_value);
+void EVAL_clear(EXPRESSION *expr, bool keep_error);
 
-PUBLIC int EVAL_add_constant(CLASS_CONST *cst);
-PUBLIC int EVAL_add_class(char *name);
-PUBLIC int EVAL_add_unknown(char *name);
-PUBLIC int EVAL_add_variable(int index);
+int EVAL_add_constant(CLASS_CONST *cst);
+int EVAL_add_class(char *name);
+int EVAL_add_unknown(char *name);
+int EVAL_add_variable(int index);
 
-PUBLIC void EVAL_start(EXPRESSION *expr);
+void EVAL_start(EXPRESSION *expr);
+
+bool EVAL_get_assignment_symbol(EXPRESSION *expr, const char **name, int *len);
 
 #endif
