@@ -53,6 +53,26 @@ static EVAL_COLOR *_color_buffer = NULL;
 
 #define NEXT_UTF8_CHAR(_p) (_p += _utf8_char_length[(uchar)*(_p)])
 
+static PATTERN get_last_pattern(PATTERN *pattern)
+{
+	for(;;)
+	{
+		pattern--;
+		if (PATTERN_type(*pattern) != RT_SPACE)
+			return *pattern;
+	}
+}
+
+static PATTERN get_next_pattern(PATTERN *pattern)
+{
+	for(;;)
+	{
+		pattern++;
+		if (PATTERN_type(*pattern) != RT_SPACE)
+			return *pattern;
+	}
+}
+
 static int get_type(PATTERN *pattern)
 {
 	int type = PATTERN_type(*pattern);
@@ -62,14 +82,14 @@ static int get_type(PATTERN *pattern)
 	{
 		if (index >= RS_COLON)
 		{
-			if (!((index == RS_AND || index == RS_OR) && PATTERN_is(pattern[1], RS_IF)))
+			if (!((index == RS_AND || index == RS_OR) && PATTERN_is(get_next_pattern(pattern), RS_IF)))
 				type = RT_OPERATOR;
 		}
 		else if (RES_is_type(index))
 			type = RT_DATATYPE;
 		else if (index == RS_WITH && pattern > EVAL->pattern)
 		{
-			index = PATTERN_index(pattern[-1]);
+			index = PATTERN_index(get_last_pattern(pattern));
 			if (index == RS_BEGINS || index == RS_ENDS)
 				type = RT_OPERATOR;
 		}
@@ -78,22 +98,22 @@ static int get_type(PATTERN *pattern)
 	return type;
 }
 
-static int is_me_last_kind(PATTERN pattern)
+static bool is_me_last_kind(PATTERN pattern)
 {
 	return PATTERN_is(pattern, RS_ME)
-				|| PATTERN_is(pattern, RS_SUPER)
-				|| PATTERN_is(pattern, RS_LAST)
-				|| PATTERN_is(pattern, RS_TRUE)
-				|| PATTERN_is(pattern, RS_FALSE)
-				|| PATTERN_is(pattern, RS_PINF)
-				|| PATTERN_is(pattern, RS_MINF)
-				|| PATTERN_is(pattern, RS_NULL);
+		|| PATTERN_is(pattern, RS_SUPER)
+		|| PATTERN_is(pattern, RS_LAST)
+		|| PATTERN_is(pattern, RS_TRUE)
+		|| PATTERN_is(pattern, RS_FALSE)
+		|| PATTERN_is(pattern, RS_PINF)
+		|| PATTERN_is(pattern, RS_MINF)
+		|| PATTERN_is(pattern, RS_NULL);
 }
 
-static int is_optional_kind(PATTERN pattern)
+static bool is_optional_kind(PATTERN pattern)
 {
 	return PATTERN_is(pattern, RS_OPTIONAL)
-				|| PATTERN_is(pattern, RS_BYREF);
+		|| PATTERN_is(pattern, RS_BYREF);
 }
 
 static void get_symbol(PATTERN pattern, const char **symbol, int *len)
@@ -223,36 +243,16 @@ static int is_proc(void)
 	for (i = 0;; i++)
 	{
 		pattern = EVAL->pattern[i];
-		if (PATTERN_is_end(pattern))
+		if (PATTERN_is_end(pattern) || PATTERN_is_newline(pattern))
 			return FALSE;
 
-		if (PATTERN_is(pattern, RS_PRIVATE) || PATTERN_is(pattern, RS_PUBLIC) || PATTERN_is(pattern, RS_STATIC) || PATTERN_is(pattern, RS_FAST))
+		if (PATTERN_is(pattern, RS_PRIVATE) || PATTERN_is(pattern, RS_PUBLIC) || PATTERN_is(pattern, RS_STATIC) || PATTERN_is(pattern, RS_FAST) || PATTERN_is_space(pattern))
 			continue;
 
 		return (PATTERN_is(pattern, RS_SUB) || PATTERN_is(pattern, RS_PROCEDURE) || PATTERN_is(pattern, RS_FUNCTION));
 	}
 }
 
-static int get_indent(bool *empty)
-{
-	int i;
-	unsigned char c;
-
-	*empty = TRUE;
-
-	for (i = 0; i < (int)EVAL->len; i++)
-	{
-		c = EVAL->source[i];
-		if (c > ' ')
-		{
-			*empty = FALSE;
-			break;
-		}
-	}
-
-	return i;
-}
-
 static int get_symbol_indent(const char *symbol, int len)
 {
 	int i;
@@ -325,11 +325,18 @@ static void add_result_char(EVAL_ANALYZE *result, char c)
 	_analyze_buffer[_analyze_buffer_pos++] = c;
 }
 
+static void add_result_spaces(EVAL_ANALYZE *result, int nspace)
+{
+	while (nspace > 0)
+	{
+		add_result_char(result, ' ');
+		nspace--;
+	}
+}
+
 static void analyze(EVAL_ANALYZE *result)
 {
 	PATTERN *pattern;
-	int nspace;
-	bool empty = FALSE;
 	int type, old_type, next_type;
 	const char *symbol;
 	const char *p;
@@ -341,29 +348,16 @@ static void analyze(EVAL_ANALYZE *result)
 	EVAL_analyze_exit();
 	
 	pattern = EVAL->pattern;
-	nspace = 0;
 	preprocessor = FALSE;
 
 	if (EVAL->len <= 0)
 		return;
 
-	if (!EVAL->comment)
-	{
-		nspace = get_indent(&empty);
-		add_data(RT_END, nspace);
-	}
-
-	if (empty)
-		return;
-
 	if (!pattern)
 		return;
 
 	init_result();
 	
-	if (nspace)
-		add_result(result, EVAL->source, nspace);
-
 	type = EVAL->comment ? RT_COMMENT : RT_END;
 	next_type = RT_END;
 	old_type = RT_END;
@@ -371,17 +365,30 @@ static void analyze(EVAL_ANALYZE *result)
 
 	for(;;)
 	{
-		old_type = next_type;
 		type = get_type(pattern);
+		
+		if (type == RT_END)
+			break;
+		
+		if (type == RT_SPACE)
+		{
+			if (!EVAL->rewrite || _colors_len == 0 || PATTERN_is_end(pattern[1]) || PATTERN_is_comment(pattern[1]))
+			{
+				len = PATTERN_index(*pattern);
+				add_data(RT_SPACE, len);
+				add_result_spaces(result, len);
+			}
+			goto __NEXT_PATTERN;
+		}
+
+		old_type = next_type;
 		next_type = type;
+		
 		get_symbol(*pattern, &symbol, &len);
 
 		space_before = space_after;
 		space_after = FALSE;
 
-		if (type == RT_END)
-			break;
-
 		//if (in_quote && (type == RT_RESERVED || type == RT_DATATYPE || type == RT_SUBR))
 		//	type = RT_IDENTIFIER;
 
@@ -403,37 +410,19 @@ static void analyze(EVAL_ANALYZE *result)
 					if (old_type != RT_OPERATOR)
 						space_before = TRUE;
 				}
-				else
-					space_before = TRUE;
-				
-				/*if (!is_me_last_kind(*pattern))
-					space_before = TRUE;
-				else
+				else if (PATTERN_is_preprocessor(*pattern))
 				{
-					if (*pattern != RS_OPTIONAL)
-					if (old_type != RT_OPERATOR)
-						space_before = TRUE;
-				}*/
-				
-				if (preprocessor && PATTERN_is(pattern[-1], RS_SHARP))
+					preprocessor = TRUE;
 					space_before = FALSE;
-
-				/*if (PATTERN_index(*pattern) >= RS_COLON)
-				{
-					int i;
-
-					for (i = 0; i < len; i++)
-						usym[i] = GB.toupper(symbol[i]);
-					usym[len] = 0;
-
-					symbol = usym;
-				}*/
-
+				}
+				else
+					space_before = TRUE;
+				
 				break;
 
 			case RT_DATATYPE:
 				//state = Datatype;
-				if (PATTERN_is(pattern[-1], RS_OPEN))
+				if (PATTERN_is(get_last_pattern(pattern), RS_OPEN))
 					type = RT_RESERVED;
 
 				if (old_type != RT_OPERATOR)
@@ -468,7 +457,7 @@ static void analyze(EVAL_ANALYZE *result)
 
 			case RT_COMMENT:
 				//state = Commentary;
-				space_before = *symbol != ' ';
+				space_before = FALSE; //*symbol != ' ';
 				i = get_symbol_indent(symbol, len);
 				if (i <= (len - 2) && symbol[i + 1] == '\'')
 					type = RT_HELP;
@@ -508,8 +497,8 @@ static void analyze(EVAL_ANALYZE *result)
 					space_after = FALSE;
 					//in_quote = *symbol == '{';
 					
-					if (!preprocessor && *symbol == '#' && old_type == RT_END)
-						preprocessor = TRUE;
+					/*if (!preprocessor && *symbol == '#' && old_type == RT_END)
+						preprocessor = TRUE;*/
 				}
 				else if (index("}", *symbol))
 				{
@@ -524,7 +513,7 @@ static void analyze(EVAL_ANALYZE *result)
 				}
 				else if (*symbol == '-' && len == 1)
 				{
-					if (old_type == RT_OPERATOR && (PATTERN_is(pattern[-1], RS_LBRA) || PATTERN_is(pattern[-1],RS_LSQR)))
+					if (old_type == RT_OPERATOR && (PATTERN_is(get_last_pattern(pattern), RS_LBRA) || PATTERN_is(get_last_pattern(pattern),RS_LSQR)))
 						space_before = FALSE;
 					else
 						space_before = TRUE;
@@ -533,7 +522,7 @@ static void analyze(EVAL_ANALYZE *result)
 						space_after = FALSE;
 					else if (old_type == RT_OPERATOR)
 					{
-						get_symbol(pattern[-1], &symbol, &len);
+						get_symbol(get_last_pattern(pattern), &symbol, &len);
 						if (index(")]}", *symbol))
 							space_after = TRUE;
 						else
@@ -545,7 +534,7 @@ static void analyze(EVAL_ANALYZE *result)
 				}
 				else if (PATTERN_is(*pattern, RS_NOT))
 				{
-					if (old_type == RT_OPERATOR && (PATTERN_is(pattern[-1], RS_LBRA) || PATTERN_is(pattern[-1],RS_LSQR)))
+					if (old_type == RT_OPERATOR && (PATTERN_is(get_last_pattern(pattern), RS_LBRA) || PATTERN_is(get_last_pattern(pattern),RS_LSQR)))
 						space_before = FALSE;
 					else
 						space_before = TRUE;
@@ -568,10 +557,10 @@ static void analyze(EVAL_ANALYZE *result)
 				break;
 		}
 
-		if (space_before && old_type != RT_END)
+		if (EVAL->rewrite && space_before && old_type != RT_END)
 		{
 			add_result_char(result, ' ');
-			add_data(preprocessor ? RT_PREPROCESSOR : RT_END, 1);
+			add_data(preprocessor ? RT_PREPROCESSOR : RT_SPACE, 1);
 		}
 
 		if (type == RT_STRING)
@@ -636,7 +625,7 @@ static void analyze(EVAL_ANALYZE *result)
 			}
 			else if (type == RT_IDENTIFIER)
 			{
-				if (PATTERN_is(pattern[1], RS_COLON))
+				if (PATTERN_is(get_next_pattern(pattern), RS_COLON))
 				{
 					add_result_char(result, ':');
 					add_data(RT_LABEL, len + 1);
@@ -644,15 +633,19 @@ static void analyze(EVAL_ANALYZE *result)
 					pattern ++;
 					goto __NEXT_PATTERN;
 				}
-				else if (old_type == RT_RESERVED && (PATTERN_is(pattern[-1], RS_GOTO) || PATTERN_is(pattern[-1], RS_GOSUB)))
+				else if (old_type == RT_RESERVED && (PATTERN_is(get_last_pattern(pattern), RS_GOTO) || PATTERN_is(get_last_pattern(pattern), RS_GOSUB)))
 				{
 					type = RT_LABEL;
 				}
 			}
 			else if (type == RT_RESERVED)
 			{
-				if (PATTERN_is(*pattern, RS_NULL)
-						|| PATTERN_is(*pattern, RS_TRUE)
+				if (PATTERN_is(*pattern, RS_NULL))
+				{
+					if (!(old_type == RT_RESERVED && PATTERN_is(get_last_pattern(pattern), RS_OPEN)))
+						type = RT_CONSTANT;
+				}
+				else if (PATTERN_is(*pattern, RS_TRUE)
 						|| PATTERN_is(*pattern, RS_FALSE)
 						|| PATTERN_is(*pattern, RS_PINF)
 						|| PATTERN_is(*pattern, RS_MINF))
@@ -700,7 +693,7 @@ static void add_end_pattern(void)
 
 PUBLIC void EVAL_analyze(const char *src, int len, int state, EVAL_ANALYZE *result, bool rewrite)
 {
-	int nspace = 0;
+	//int nspace = 0;
 
 	#ifdef DEBUG
 	printf("EVAL: %*.s\n", expr->len, expr->source);
@@ -708,11 +701,11 @@ PUBLIC void EVAL_analyze(const char *src, int len, int state, EVAL_ANALYZE *resu
 
 	CLEAR(result);
 
-	while (len > 0 && src[len - 1] == ' ')
+	/*while (len > 0 && src[len - 1] == ' ')
 	{
 		len--;
 		nspace++;
-	}
+	}*/
 
 	result->len = 0;
 	result->str = NULL;
@@ -757,12 +750,6 @@ PUBLIC void EVAL_analyze(const char *src, int len, int state, EVAL_ANALYZE *resu
 	{
 		result->proc = FALSE;
 	}
-
-	while (nspace > 0)
-	{
-		result->str = GB.AddString(result->str, "        ", nspace > 8 ? 8 : nspace);
-		nspace -= 8;
-	}
 }
 
 
