@@ -32,7 +32,6 @@
 #include "eval.h"
 #include "eval_read.h"
 
-
 //#define DEBUG 1
 //#define ENABLE_BIG_COMMENT
 
@@ -57,6 +56,7 @@ enum
 {
 	GOTO_BREAK,
 	GOTO_SPACE,
+	GOTO_NEWLINE,
 	GOTO_COMMENT,
 	GOTO_STRING,
 	GOTO_IDENT,
@@ -82,6 +82,8 @@ static void READ_init(void)
 
 			if (i == 0)
 				first_car[i] = GOTO_BREAK;
+			else if (i == '\n')
+				first_car[i] = GOTO_NEWLINE;
 			else if (i <= ' ')
 				first_car[i] = GOTO_SPACE;
 			else if (i == '\'')
@@ -157,6 +159,10 @@ PUBLIC char *READ_get_pattern(PATTERN *pattern)
 			strncpy(_buffer, TABLE_get_symbol_name(EVAL->string, index), BUF_MAX);
 			_buffer[BUF_MAX] = 0;
 			break;
+			
+		case RT_SPACE:
+			snprintf(_buffer, BUF_MAX, "[%d]", index);
+			break;
 
 		default:
 			sprintf(_buffer, "%s?%08X?%s", before, *pattern, after);
@@ -219,6 +225,8 @@ PUBLIC void READ_dump_pattern(PATTERN *pattern)
 		printf("SUBR         %s\n", READ_get_pattern(pattern));
 	else if (type == RT_COMMENT)
 		printf("COMMENT      %s\n", READ_get_pattern(pattern));
+	else if (type == RT_SPACE)
+		printf("SPACE        %s\n", READ_get_pattern(pattern));
 	else
 		printf("?            %d\n", index);
 
@@ -249,33 +257,37 @@ static void add_pattern(int type, int index)
 
 #endif
 
-static PATTERN get_last_last_pattern()
+static PATTERN get_previous_pattern(int n)
 {
-	if (EVAL->pattern_count > 1)
-		return EVAL->pattern[EVAL->pattern_count - 2];
-	else
-		return NULL_PATTERN;
+	int i;
+	PATTERN pattern;
+	
+	for (i = EVAL->pattern_count - 1; i >= 0; i--)
+	{
+		pattern = EVAL->pattern[i];
+		if (!PATTERN_is_space(pattern))
+		{
+			n--;
+			if (n == 0)
+				return pattern;
+		}
+	}
+
+	return NULL_PATTERN;
 }
 
-static PATTERN get_last_pattern()
-{
-	if (EVAL->pattern_count > 0)
-		return EVAL->pattern[EVAL->pattern_count - 1];
-	else
-		return NULL_PATTERN;
-}
+#define get_last_pattern() get_previous_pattern(1)
+#define get_last_last_pattern() get_previous_pattern(2)
 
 static void add_newline()
 {
 	add_pattern(RT_NEWLINE, 0);
-	source_ptr++;
 }
 
 
 static void add_end()
 {
 	add_pattern(RT_END, 0);
-	source_ptr++;
 }
 
 
@@ -425,12 +437,12 @@ END:
 	if (sign && !PATTERN_is_null(last_pattern) && (!PATTERN_is_reserved(last_pattern) || PATTERN_is(last_pattern, RS_RBRA) || PATTERN_is(last_pattern, RS_RSQR)))
 	{
 		add_pattern(RT_RESERVED, RESERVED_find_word(&sign, 1));
-		TABLE_add_symbol(EVAL->table, start + 1, source_ptr - start - 1, &index);
+		index = TABLE_add_symbol(EVAL->table, start + 1, source_ptr - start - 1);
 		add_pattern(RT_NUMBER, index);
 	}
 	else
 	{
-		TABLE_add_symbol(EVAL->table, start, source_ptr - start, &index);
+		index = TABLE_add_symbol(EVAL->table, start, source_ptr - start);
 		add_pattern(RT_NUMBER, index);
 	}
 
@@ -689,7 +701,7 @@ static void add_identifier()
 				{
 					source_ptr++;
 					len++;
-					TABLE_add_symbol(EVAL->table, start, len - 2, &index);
+					index = TABLE_add_symbol(EVAL->table, start, len - 2);
 					continue;
 				}
 			}
@@ -731,7 +743,7 @@ static void add_identifier()
 			break;
 
 		__CLASS:
-			can_be_reserved = canres_car[car] && _begin_line;
+			can_be_reserved = canres_car[car] && (_begin_line || PATTERN_is(last_pattern, RS_END));
 			break;
 
 		__STRUCT:
@@ -806,7 +818,7 @@ static void add_identifier()
 						ret = GB.Call(&func, 1, FALSE);
 						if (!ret->_boolean.value)
 						{
-							exist = TABLE_add_symbol(EVAL->table, start, len, &index);
+							exist = TABLE_add_symbol_exist(EVAL->table, start, len, &index);
 							type = RT_IDENTIFIER;
 						}
 					}
@@ -829,11 +841,13 @@ static void add_identifier()
 
 	if (!EVAL->analyze && PATTERN_is(last_pattern, RS_EXCL))
 	{
-		TABLE_add_symbol(EVAL->string, start, len, &index);
+		index = TABLE_add_symbol(EVAL->string, start, len);
 		type = RT_STRING;
 	}
 	else
-		exist = TABLE_add_symbol(EVAL->table, start, len, &index);
+	{
+		exist = TABLE_add_symbol_exist(EVAL->table, start, len, &index);
+	}
 
 __ADD_PATTERN:
 
@@ -886,7 +900,7 @@ static void add_quoted_identifier(void)
 	{
 		if (car != '}')
 			THROW("Missing '}'");
-		
+	
 		if (len == 2)
 			THROW("Void identifier");
 	}
@@ -894,28 +908,30 @@ static void add_quoted_identifier(void)
 	{
 		if (!car)
 			len--;
+		
+		if (car != '}' || len <= 2)
+			type = RT_ERROR;
 	}
 	
 	if (!EVAL->analyze && PATTERN_is(last_pattern, RS_EXCL))
 	{
-		TABLE_add_symbol(EVAL->string, start + 1, len - 2, &index);
+		index = TABLE_add_symbol(EVAL->string, start + 1, len - 2);
 		type = RT_STRING;
 	}
 	else
 	{
-		if (!EVAL->rewrite)
+		if (!EVAL->rewrite && type != RT_ERROR)
 		{
 			start++;
 			len -= 2;
 		}
-		TABLE_add_symbol(EVAL->table, start, len, &index);
+		index = TABLE_add_symbol(type == RT_ERROR ? EVAL->string : EVAL->table, start, len);
 	}
 
 	add_pattern(type, index);
 }
 
 
-
 static void add_operator()
 {
 	unsigned char car;
@@ -1076,7 +1092,7 @@ static void add_string()
 	//fprintf(stderr, "EVAL_read: add_string (end): [%d] %s\n", source_ptr - EVAL->source, source_ptr);
 	if (len > 0)
 	{
-		TABLE_add_symbol(EVAL->string, start + 1, len, &index);
+		index = TABLE_add_symbol(EVAL->string, start + 1, len);
 		add_pattern(RT_STRING, index);
 	}
 	else
@@ -1155,7 +1171,7 @@ static void add_big_comment()
 		source_ptr++;
 	}
 
-	TABLE_add_symbol(EVAL->string, start, len, &index);
+	index = TABLE_add_symbol(EVAL->string, start, len);
 	type = RT_COMMENT;
 	add_pattern(type, index);
 }
@@ -1168,36 +1184,10 @@ static void add_comment()
 	int len;
 	int index;
 	int type;
-	bool space = FALSE;
 
 	start = source_ptr;
 	len = 1;
 
-	for(;;)
-	{
-		if (start == EVAL->source)
-			break;
-
-		start--;
-		car = *start;
-		if (car == '\n')
-			break;
-
-		if (car > ' ')
-		{
-			start++;
-			space = TRUE;
-			break;
-		}
-		len++;
-	}
-
-	if (!space)
-	{
-		start = source_ptr;
-		len = 1;
-	}
-
 	for(;;)
 	{
 		source_ptr++;
@@ -1207,7 +1197,7 @@ static void add_comment()
 		len++;
 	}
 
-	TABLE_add_symbol(EVAL->string, start, len, &index);
+	index = TABLE_add_symbol(EVAL->string, start, len);
 	type = RT_COMMENT;
 
 	add_pattern(type, index);
@@ -1245,7 +1235,7 @@ static void add_string_for_analyze()
 	if (car == '"')
 		source_ptr++;
 
-	TABLE_add_symbol(EVAL->string, start + 1, len, &index);
+	index = TABLE_add_symbol(EVAL->string, start + 1, len);
 	type = RT_STRING;
 
 	add_pattern(type, index);
@@ -1253,12 +1243,33 @@ static void add_string_for_analyze()
 }
 
 
+static void add_spaces()
+{
+	unsigned char car;
+	int len;
+
+	len = 1;
+
+	for(;;)
+	{
+		source_ptr++;
+		car = get_char();
+		if (car > ' ' || car == '\n' || car == 0)
+			break;
+		len++;
+	}
+
+	add_pattern(RT_SPACE, len);
+}
+
+
 PUBLIC void EVAL_read(void)
 {
-	static void *jump_char[10] =
+	static void *jump_char[11] =
 	{
 		&&__BREAK,
 		&&__SPACE,
+		&&__NEWLINE,
 		&&__COMMENT,
 		&&__STRING,
 		&&__IDENT,
@@ -1291,13 +1302,19 @@ PUBLIC void EVAL_read(void)
 		THROW(E_SYNTAX);
 
 	__SPACE:
+	
+		if (EVAL->analyze)
+			add_spaces();
+		else
+			source_ptr++;
+		
+		continue;
+		
+	__NEWLINE:
 
 		source_ptr++;
-		if (car == '\n')
-		{
-			add_newline();
-			_begin_line = TRUE;
-		}
+		add_newline();
+		_begin_line = TRUE;
 		continue;
 
 	__COMMENT:
@@ -1352,10 +1369,10 @@ PUBLIC void EVAL_read(void)
 		{
 			_begin_line = FALSE;
 			if (get_char_offset(1) == '!' && EVAL->analyze)
-			{
 				add_comment();
-				continue;
-			}
+			else
+				add_identifier();
+			continue;
 		}
 
 	__OTHER:
