@@ -35,7 +35,7 @@ typedef
 typedef
 	void *EVAL_EXPRESSION;
 
-#define EVAL_COLOR_MAX_LEN (1 << 10)	
+#define EVAL_COLOR_MAX_LEN 1023
 
 typedef
 	struct {
@@ -96,6 +96,9 @@ typedef
 		HIGHLIGHT_ESCAPE,
 		HIGHLIGHT_LABEL,
 		HIGHLIGHT_CONSTANT,
+		HIGHLIGHT_ALTERNATE,
+		HIGHLIGHT_ADDED,
+		HIGHLIGHT_REMOVED,
 		HIGHLIGHT_NUM_COLOR
 	}
 	HIGHLIGHT_COLOR;
