@@ -222,7 +222,7 @@ const char *JIT_get_default_value(TYPE type)
 			
 			if (!_decl_null_object)
 			{
-				JIT_print_decl("  const GB_OBJECT null_object = {GB_T_NULL};\n");
+				JIT_print_decl("  const GB_OBJECT null_object = {GB_T_OBJECT};\n");
 				_decl_null_object = TRUE;
 			}
 			return "null_object";
@@ -494,6 +494,7 @@ char *JIT_translate(const char *name, const char *from)
 	return JIT_end();
 }
 
+
 void JIT_panic(const char *fmt, ...)
 {
 	va_list args;
@@ -510,6 +511,7 @@ void JIT_panic(const char *fmt, ...)
 	abort();
 }
 
+
 int JIT_get_code_size(FUNCTION *func)
 {
 	void *code = func->code;
