@@ -1,14 +1,12 @@
 /***************************************************************************
 
-  common_case.h
+  gb_common_case.h
 
-  Case conversion routines
-
-  Copyright (c) 2000-2007 Benoit Minisini <gambas@freesurf.fr>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,18 +16,23 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
 #ifndef __GB_COMMON_CASE_H
 #define __GB_COMMON_CASE_H
 
+#ifdef __CYGWIN__
+#include <strings.h>
+#endif
+
 #include <ctype.h>
 #include <string.h>
 
-extern char COMMON_tolower[];
-extern char COMMON_toupper[];
+extern const unsigned char COMMON_tolower[];
+extern const unsigned char COMMON_toupper[];
 
 int COMMON_strcasecmp(const char *s1, const char *s2);
 int COMMON_strncasecmp(const char *s1, const char *s2, size_t n);
