@@ -2,16 +2,11 @@
 
   gb_common_swap_temp.h
 
-  common useful routines template
-
-  Datatype management routines. Conversions between each Gambas datatype,
-  and conversions between Gambas datatypes and native datatypes.
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -21,11 +16,12 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
-PUBLIC void SWAP_int(int *val)
+void SWAP_int(int *val)
 {
   char *p = (char *)val;
   char t1, t2;
@@ -38,7 +34,7 @@ PUBLIC void SWAP_int(int *val)
   p[2] = t2;
 }
 
-PUBLIC void SWAP_ints(int *val, int n)
+void SWAP_ints(int *val, int n)
 {
 	while (n > 0)
 	{
@@ -48,7 +44,7 @@ PUBLIC void SWAP_ints(int *val, int n)
 	}
 }
 
-PUBLIC void SWAP_short(short *val)
+void SWAP_short(short *val)
 {
   char *p = (char *)val;
   char t;
@@ -58,7 +54,7 @@ PUBLIC void SWAP_short(short *val)
   p[1] = t;
 }
 
-PUBLIC void SWAP_double(double *val)
+void SWAP_double(double *val)
 {
   char *p = (char *)val;
   char t;
